/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef CVS_INFO_H
#define CVS_INFO_H

#include <set> 
#include <string>
#include <fstream>
#include <iterator>

#define VERSION_INFO_IMPL(class_name, cvs_id) std::string class_name::cpp_version_info(){ \
                                                          return cvs_id; }
#define VERSION_INFO_INLINE(cvs_id) inline static std::string h_version_info(){ return cvs_id; }
#define VERSION_INFO_DECL static std::string cpp_version_info()
#ifndef CVS_REPORT_PATH
#define CVS_REPORT_PATH "/tmp/cvs_report_wool"
#endif
#define SAVE_HEADER_INFO(a) wool::cvs_info::add_info(std::string(#a)+" declaration info '" + a::h_version_info())
#define SAVE_IMPL_INFO(a)  wool::cvs_info::add_info(std::string(#a)+" implemention info '" + a::cpp_version_info())

namespace wool
{

  /** CVS interface.
      @author Vukasin Toroman <vtoroman@astaro.de> (C) Astaro AG 2004
    */
  class cvs_info
  {
    private:
      static std::set<std::string> s_infos;
    public:

      /**
       *  cvs info
       */
      VERSION_INFO_INLINE("$Id: cvs_info.h,v 1.4 2007/01/11 15:18:20 vtoroman Exp $");
      /**
       *  cvs info
       */
      VERSION_INFO_DECL;
      /**
       *  Save the info
       *  @param path file to save the info
       */
      inline static void save_info (std::string path = CVS_REPORT_PATH)
      {
        std::ofstream fs(path.c_str(), std::ofstream::out);
        std::copy(s_infos.begin(),s_infos.end(),std::ostream_iterator<std::string>(fs,"\n"));
      }

      /**
       *  Add new info
       *  @param info message
       */
      inline static void add_info (std::string info)
      {
          s_infos.insert(info);
      }
  };

};

#endif



