//
// C++ Interface: stringmap
//
// Description: 
//
//
// Author: Vukasin Toroman <vtoroman@astaro.de>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef WOOL_STRINGMAP_H
#define WOOL_STRINGMAP_H

#include <map>
#include <set>
#include <string>
#include <list>
#include <bitset>
#include <stdexcept>

namespace wool {
	class stringhash {
		public:
			struct ncase_eqstr{
				/**
				*  @param s1 reference to the first string to compare to
				*  @param s2 reference to the second string to compare to
				*  @return true if s1 is equal to s2 ignoring the case else false
				*/
				inline bool operator()(const std::string & s1, const std::string &  s2) const throw(){
					if(s1.length()!=s2.length())
						return s1.length()<s2.length();
					return (strncasecmp(s1.data(), s2.data(),s1.length()) < 0);
				}
			};

			typedef std::string word_type;
			typedef long hash_type;
			typedef std::map<std::string,hash_type,stringhash::ncase_eqstr> word_map;
			typedef std::map<hash_type,std::string> reverse_word_map;

			static const hash_type npos = -1;
			word_map words;
			reverse_word_map revwords;

			inline stringhash() {}
			template <typename iteratorT>
			stringhash(iteratorT b, iteratorT e) {
				while(b!=e) {
					add_hash(*b);
					b++;
				}
			}

			inline ~stringhash(){}
			inline void clear() {	words.clear(); }

			inline hash_type add_hash( const std::string & s ) {
        hash_type v = words.size();
				word_map::iterator it2 = words.find(s);
        reverse_word_map::iterator it = revwords.find(v);

				if(it2!=words.end()) {
          return it2->second;
        }

				revwords[v] = s;
				words[s] = v;
				return v;
			}
	
			inline hash_type hash( const std::string & s ) const {
				word_map::const_iterator pos = words.find(s);
				if(pos==words.end())
					return npos;
				return pos->second;
			}

			inline const std::string & hash( const hash_type & s ) const {
				static const word_type wnpos = "";
				reverse_word_map::const_iterator pos = revwords.find(s);
				if(pos==revwords.end())
					return wnpos;
				return pos->second;
			}
	};

	struct hash_constant {
		typedef std::string word_type;
		typedef long hash_type;

		word_type name;
		hash_type value;

		inline operator hash_type() const {	return value;	}
		inline operator word_type() const {	return name;	}

		inline hash_constant(stringhash & h, const word_type & n) 
		:name(n),value(h.add_hash(n)){}


	};
}



#endif

