#ifndef WOOL_WRAPPER_H
#define WOOL_WRAPPER_H

#include "../base/debug.h"
#include "../base/exception.h"

namespace wool {
  template <typename wrappedT>
  class Wrapper {
  public:
    wrappedT * value;
    bool owns_value;
  public:
    Wrapper( wrappedT * t = NULL, bool owns = true )
      : value(t), owns_value(owns) {
    }

    const bool & is_owner() const {
      return owns_value;
    }

    void acquire_ownership () {
      owns_value = true;
    }
    void release_ownership () {
      owns_value = false;
    }

    void attach ( wrappedT * v = NULL, bool owns = true ) {
      clear();
      if( v == NULL ) {
	owns_value = false;
	value = generate();
	owns_value = true;
      } else {
	value = v;
	owns_value = owns;
      }
    }
 
    wrappedT * get_wrapped() {
      return value;
    }

    const wrappedT * get_wrapped() const {
      return value;
    }

    void pub_update() {
      update();
    }

    virtual ~Wrapper() {

    }

  protected:
    void init () {
      if( value == NULL ) {
	owns_value = false;
	value = generate();
	owns_value = true;
      } 
      update();
    }

    void clear () {
      if(value && owns_value)
	free(value);
    }
    virtual wrappedT * generate() = 0;
    virtual void free( wrappedT * ) = 0;
    virtual void update() = 0;
  };
};
#endif
