//
// C++ Interface: config
//
// Description: 
//
//
// Author: Vukasin Toroman <vtoroman@astaro.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef WOOLCONFIG_H
#define WOOLCONFIG_H

#include <string>
#include <list>
#include <map>
#include <sstream>
#include <xmlwrapp/document.h>
#include <xmlwrapp/tree_parser.h>
#include <iostream>
#include <boost/thread.hpp>
namespace wool {

/**
@author Vukasin Toroman
*/
class Configuration{
public:
    static const Configuration & GetInstance( const std::string & s);

    Configuration();
    Configuration( const Configuration & c);

    virtual ~Configuration();

    void list( const std::string & path, std::list<std::string> & s ) const;
    void extract( const std::string & path, std::string & s ) const;
    void extract( const std::string & path, std::list<std::string> & s ) const;
    void extract( const std::string & path, std::map<std::string,std::string> & s ) const;

    template<typename T>
    void extract( const std::string & path, std::list<T> & t ) const {
      std::list<std::string> l;
      extract(path, l);
      std::list<std::string>::iterator i = l.begin();
      for(;i!=l.end();i++) {
        std::istringstream data_stream(*i);
        T st;
        data_stream >> st;
        t.push_back(st); 
      }
    }

    template<typename T>
    void extract( const std::string & path, std::map<std::string, T> & t ) const {
      std::map<std::string, std::string> l;
      extract(path, l);
      std::map<std::string, std::string>::iterator i = l.begin();
      for(;i!=l.end();i++) {
        std::istringstream data_stream(i->second);
        T & st = t[i->first];
        data_stream >> st;
      }
    }

    
    template<typename T>
    void extract( const std::string & path, T & t ) const {
      std::string data;
      extract(path, data);
      std::istringstream data_stream(data);
      data_stream >> t;
    }

    template <typename T, typename ConfigPathT>
    class Constant {
      private:
        T m_value;
        ConfigPathT path_traits;
      public:
        Constant( const std::string & path) {
	  std::cout << "LOAD: " << path_traits.get_path() << ": " << path << std::endl;
          Configuration::GetInstance(path_traits.get_path()).extract(path,m_value);
        }
        Constant( const std::string & path, const T & defaultVal) 
        : m_value(defaultVal){
	  std::cout << "LOAD: " << path_traits.get_path() << ": " << path << std::endl;
          Configuration::GetInstance(path_traits.get_path()).extract(path,m_value);
        }
        ~Constant() {
        }
        const T & get() const { return m_value; }
        operator T () const {
          return get();
        }
    };
private:
    mutable boost::mutex m_guard;
    void load(const std::string & config_path);
    std::string m_config_path;
    xml::tree_parser * m_documentParser;
    
    time_t m_create_time;
};

}

#endif
