/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "base/debug.h"
#include "dl/dlmodule.h"
#include <dlfcn.h>




namespace wool {

    const dlmodule::load_flag_type dlmodule::lazy = RTLD_LAZY;
    const dlmodule::load_flag_type dlmodule::now = RTLD_NOW;
    const dlmodule::load_flag_type dlmodule::global = RTLD_GLOBAL;
    const dlmodule::load_flag_type dlmodule::local = RTLD_LOCAL;
    const dlmodule::os_module_handle_type dlmodule::null_handle = NULL;

    VERSION_INFO_IMPL(dlmodule, "$Id: dlmodule.cpp,v 1.4 2007/01/11 15:18:24 vtoroman Exp $");

    dlmodule::load_failure::load_failure (wool::exception::string_type str)  throw ()
    :wool::exception(str){}

    dlmodule::load_failure::~load_failure () throw ()
    {}

    dlmodule::symbol_lookup_failure::symbol_lookup_failure (wool::exception::string_type str) throw ()
    :wool::exception(str)
    {}

    dlmodule::symbol_lookup_failure::~symbol_lookup_failure  () throw ()
    {}

    dlmodule::dlmodule () throw ()
    :m_path(""),m_handle(null_handle)
    {}

    dlmodule::dlmodule (const dlmodule & src) throw ()
    :m_path(""),m_handle(null_handle)
    {
      if(src)
      {
        try
        {
          path(src.path(),src.m_flags);
        }
        catch( const dlmodule::load_failure & lf)
        {
          WOOL_DEBUG ("Failed to load module '"<<src.m_path<<"':"<<lf.what());
        }
      }
    }

    dlmodule::dlmodule (const path_type & __path ,load_flag_type flags) throw()
    :m_path(""),m_handle(null_handle)
    {
        try
        {
          path(__path,flags);
        }
        catch( const dlmodule::load_failure & lf)
        {
          WOOL_DEBUG ("Failed to load module '"<<__path<<"':"<<lf.what());
        }
    }

    dlmodule::~dlmodule () throw()
    {
      close();
    }

    void dlmodule::path(const path_type & path, load_flag_type flags) throw( dlmodule::load_failure)
    {
      close();
      if (path.empty()) WOOL_DEBUG ("loading main");
      m_handle = ::dlopen((path.empty())?NULL:path.c_str(),flags);
      if(m_handle == null_handle)
        throw dlmodule::load_failure(::dlerror());
      m_path = path;
      m_flags = flags;
    }

    void dlmodule::close() throw ()
    {
      if(m_handle==null_handle)
        return;
      ::dlclose(m_handle);
      m_handle=null_handle;
      m_path.clear();


    }

    dlmodule & dlmodule::operator &=(const dlmodule & src) throw()
    {
      close();
      if(src)
      {
        try
        {
          path(src.path(),src.m_flags);
        }
        catch( const dlmodule::load_failure & lf)
        {
          WOOL_DEBUG ("Failed to load module '"<<src.m_path<<"':"<<lf.what());
        }
      }
      return (*this);
    }

  dlmodule::symbol_type dlmodule::symbol (const symbol_name_type &name) const
      throw( dlmodule::symbol_lookup_failure)
  {
    if(!*this)
      throw dlmodule::symbol_lookup_failure("No module loaded");

    ::dlerror(); //reset the error
    symbol_type res = ::dlsym(m_handle,name.c_str());
    char * errstring = ::dlerror();
    if ( errstring==NULL )
      return res;
    throw dlmodule::symbol_lookup_failure(std::string("Symbol not found: ")+name+" error: " +errstring);
  }


};
