/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#include "dns/dns.h"

#include <netdb.h>
#include <sys/types.h>
#include <stdio.h>

#include <error.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/netfilter_ipv4.h>
#include <unistd.h>

namespace wool
{
namespace inet 
{
    int Dns::Timeout = 60;
    int Dns::SetTimeout( int i ) {
	int j = Timeout;
	Timeout = i;
	return j;
    }

    int Dns::GetTimeout( ) {
	return Timeout;
    }

    Dns::exception::exception ( const std::string & hname ) throw() : wool::exception (hname) {
	m_error_message.insert(0,"dns error: Unknown host ");
    }

    Dns::exception::exception ( int __errno  ) throw () : wool::exception (gai_strerror(__errno)){
          m_error_message.insert(0,"dns error:");
    }

    Dns::exception::~exception () throw () {      }
    /**
     *  Resolves the given hostname into an IPv4 address.
     *  @param hostname hostname
     *  @param timeout timeout for resolve the hostname into the IPv4 address
     *  @return IPv4 address
     */
    in_addr_t Dns::Resolve (const char * hostname) throw(exception) {
	return Resolve(hostname, GetTimeout());
    }
    
    in_addr_t Dns::Resolve (const char * hostname, int timeout) throw(exception) {
	 in_addr ip;
         if(inet_pton(AF_INET, hostname, &ip)>0)
         	return ip.s_addr;
 
	struct addrinfo hints, *res, *res0;
        int prev_alarm=0;
        int error;

        memset(&hints, 0, sizeof(hints));

        hints.ai_family = PF_INET;
        hints.ai_socktype = SOCK_STREAM;

        struct sigaction sigalrm, old_sigalrm;

        sigalrm.sa_handler = __throw_timeout;
        sigalrm.sa_flags = 0;
        sigemptyset (&sigalrm.sa_mask);
        sigaction (SIGALRM, &sigalrm, &old_sigalrm);    // Set connect() timeout

        if (timeout > 0)
                prev_alarm=alarm (timeout);
        else
                prev_alarm=alarm (0);

        error = getaddrinfo(hostname, NULL, &hints, &res0);

        alarm (prev_alarm);
        sigaction (SIGALRM, &old_sigalrm, NULL);

        if (error!=0)
                throw Dns::exception(error);

        res = NULL;

        for (res = res0; res; res = res->ai_next)
        {
                ip = ((sockaddr_in*) res->ai_addr)->sin_addr;
                if(     ((res->ai_socktype&SOCK_STREAM)>0)&&
                        ((res->ai_protocol&PF_INET)>0)
                        )
                        break;
        }
        freeaddrinfo(res0);

        if(!res)
                throw exception(hostname);
        return ip.s_addr;
    }
  };

};


