/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef WOOL_NETWORK_H
#define WOOL_NETWORK_H

#include "../base/wool_stl_includes.h"
#include "../base/cvs_info.h"
#include "../base/exception.h"

#include <netinet/in.h>

#include <unistd.h>


namespace wool
{
namespace inet 
{
  
  inline void __throw_timeout (int i ){}

  /**
   * This class is used to translate the name of a host into its IP address.<br>
   * Example: www.moon.mars will be resolved as 192.168.100.2 perhaps.
   */
  class Dns
  {
  private:
    static int Timeout;
    inline Dns() {};
  public:
    static int SetTimeout( int i );
    static int GetTimeout( );

    /**
      * dns_exception definition
      */
    class exception : public wool::exception {
  	public:
	  exception ( const std::string & hostname ) throw ();
          exception ( int __errno ) throw ();
          virtual ~exception () throw (); 
    };
    /**
     *  Resolves the given hostname into an IPv4 address.
     *  @param hostname hostname
     *  @param timeout timeout for resolve the hostname into the IPv4 address
     *  @return IPv4 address
     */
    static in_addr_t Resolve (const char * hostname) throw(Dns::exception);
    static in_addr_t Resolve (const char * hostname, int timeout) throw(Dns::exception);
  };

};
};
#endif

