/***************************************************************************
 *   Copyright (C) 2005 by Vukasin Toroman                                 *
 *   vtoroman@astaro.de                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef JACKALXMLRPC_H
#define JACKALXMLRPC_H

#include <string>
#include <vector>
#include <map>
#include <iostream>

#include <xmlwrapp/xmlwrapp.h>
#include "../wool/message_rfc2822.h"
#include "../wool/url.h"

using namespace std;

namespace wool {

namespace http {
struct request {
    std::string method;
    std::string path;
    std::string protocol;
    wool::rfc2822::header header;
    wool::swappable_data body;
};

struct response {
    unsigned int status_code;
    std::string message;
    std::string protocol;
    wool::rfc2822::header header;
    wool::swappable_data body;
};
};

template<>
struct message_traits<wool::http::request> {
    typedef header_traits<rfc2822::header> header_traits_type;
    typedef body_traits<wool::swappable_data> body_traits_type;

    template <typename massageLoaderT>
    static void eval_header (massageLoaderT & msg) {
    	WOOL_DEBUG(" header parsed");
        msg.message.header.erase("Accept");
        msg.message.header.erase("Range");
        msg.body_loader.required_size = 0;
        rfc2822::header::iterator it = msg.message.header.find("Content-length");
        if(it!=msg.message.header.end()) {
            msg.body_loader.required_size = ::strtol(it->second.c_str(), (char**)NULL, 10);
            if(msg.body_loader.required_size<0) {
                msg.message.header.erase(it);
                msg.body_loader.required_size = 0;
            }
        }
        
    }

    template <typename massageLoaderT>
    static void eval_body (massageLoaderT & msg) {
        WOOL_DEBUG("loaded body");
    }
};

template<>
struct message_traits<wool::http::response> {
    typedef header_traits<rfc2822::header> header_traits_type;
    typedef body_traits<wool::swappable_data> body_traits_type;

    template <typename massageLoaderT>
    static void eval_header (massageLoaderT & msg) {
        msg.message.header.erase("Accept");
        msg.message.header.erase("Range");
        msg.body_loader.required_size = -1;
        rfc2822::header::iterator it = msg.message.header.find("Content-length");
        if(it!=msg.message.header.end()) {
            msg.body_loader.required_size = ::strtol(it->second.c_str(), (char**)NULL, 10);
            if(msg.body_loader.required_size<0) {
                msg.message.header.erase(it);
            }
        }
        //std::cerr << "loaded header: requested body size "<<msg.body_loader.required_size<<std::endl;
    }

    template <typename massageLoaderT>
    static void eval_body (massageLoaderT & msg) {
        WOOL_DEBUG("loaded body");
    }
};
};

namespace std {
inline std::istream & operator >> ( std::istream & is, wool::http::request & h) {
	WOOL_DEBUG("loading request");
    wool::message_loader<wool::http::request> msg_loader(h);
    h.header.clear();
    h.body.clear();
    std::string l;
    while((std::getline(is,l))&&((l=="\r")||(l=="\n")||(l=="")));
    std::istringstream lis(l);
    WOOL_DEBUG(" request line '" << l << "'");
    lis >> h.method >> h.path >> h.protocol >> std::ws;
    WOOL_DEBUG(" request '" << h.method << "' '"<< h.path <<"' '"<< h.protocol<<"'");
   	msg_loader.parse(is);
    WOOL_DEBUG(" request parsed");
    return is;
}

inline std::istream & operator >> ( std::istream & is, wool::http::response & h) {
    wool::message_loader<wool::http::response> msg_loader(h);
    
    std::string l;
    while((std::getline(is,l))&&((l=="\r")||(l=="\n")||(l=="")));
    std::istringstream sl(l);
    sl >> h.protocol >> h.status_code;
    std::getline(sl,h.message);
    msg_loader.parse(is);
    return is;
}
};


namespace jackal {

namespace xmlrpc {
/**
@author Vukasin Toroman
*/

struct ignored_param {
    inline ignored_param () {}
}
;

class base64{
	public:
		base64();
		~base64(); 
		
		void set_decoded( const std::string & s);
		void set_encoded( const std::string & s);
		const std::string & get_decoded() const;
		const std::string & get_encoded() const;
	private:
		void encode() const;
		void decode() const;

		mutable bool m_encoded;
		mutable bool m_decoded;
		mutable std::string m_encoded_data;
		mutable std::string m_decoded_data;
};

typedef long long int_type;
typedef std::string string_type;
typedef double float_type;
typedef bool bool_type;
typedef base64 base64_type;

extern const ignored_param ignore;
extern const std::string name_i4;
extern const std::string name_int;
extern const std::string name_string;
extern const std::string name_base64;
extern const std::string name_empty;
extern const std::string name_double;
extern const std::string name_bool;
extern const std::string name_array;
extern const std::string name_struct;
extern const std::string name_param;
extern const std::string name_value;
extern const std::string name_data;

class exception
    :public std::exception {
public:
    /**
    *  String definition
    */
    typedef std::string string_type;
    /**
    *  @param s message
    */
    exception( string_type s , int id) throw ():m_error_message(s), m_id(id) {}
    ;

    const char * what () const throw () {
        return m_error_message.c_str();
    }

    inline int id () const throw () {
        return m_id;
    }

    static inline string_type c_error ( int error_code = errno ) {
        char error_message [1024];
        return std::string(strerror_r(error_code,error_message,1024));
    }

    virtual ~exception() throw () {}
    ;

protected:
    std::string m_error_message;
    int m_id;
};

template <typename extractTraitsT, typename encodeTraitsT> class variant;


struct extract_traits {
    void extract ( base64_type & , xml::node::const_iterator ); // allowed for base64
    void extract ( string_type & , xml::node::const_iterator ); // allowed for string
    void extract ( int_type &, xml::node::const_iterator );   // allowed for int
    void extract ( float_type &, xml::node::const_iterator );      // allowed for float
    void extract ( bool_type & ,xml::node::const_iterator );

    inline void extract (const ignored_param &, xml::node::const_iterator ) {};
    template <typename T>
    void extract ( std::vector<T> & vec, xml::node::const_iterator __v) {
         vec.clear();
        	xml::node::const_iterator r_v = find_value( __v );
			xml::node::const_iterator e_v = r_v->end();
			xml::node::const_iterator v = r_v->end();
			if ((v=r_v->find(name_array.c_str()))==e_v)
            throw xmlrpc::exception("failed xmlrpc extraction: array expected",0);
        xml::node::const_iterator data = v->find("data"); //<-- not name_data.c_str() to maintain thread safety
        if(data==v->end())
            throw xmlrpc::exception("failed xmlrpc extraction: array(data) expected",0);
        xml::node::const_iterator value = data->begin();
        while((value=data->find("value",value))!=data->end()) {
            typename std::vector<T>::iterator vit = vec.insert(vec.end(),T());
            extract(*vit, value);
            value++;
        }
    }
    
    template <typename encodeTraitsT>
    void extract ( variant<extract_traits, encodeTraitsT> & v, xml::node::const_iterator i) {
      v = variant<extract_traits, encodeTraitsT>(i);
    }

    template <typename T>
    void extract ( std::map<std::string, T> & v, xml::node::const_iterator __v) {
        v.clear();
        
        xml::node::const_iterator r_v = find_value( __v );
		  xml::node::const_iterator e_v = r_v->end();
		  xml::node::const_iterator hash = r_v->end();
		  //WOOL_DEBUG("load struct from "<<*r_v);
		  if ((hash=r_v->find(name_struct.c_str()))==e_v)
            throw xmlrpc::exception("failed xmlrpc extraction: expected struct",0);
        xml::node::const_iterator member = hash->begin();
        while( ( member=hash->find("member",member) )!=hash->end() ) {
            T __val;
            xml::node::const_iterator name = member->find("name");
            xml::node::const_iterator value = member->find("value");
            if((value==member->end())||(name==member->end()))
                throw xmlrpc::exception("failed xmlrpc extraction: expected name/value pair",0);
            extract(__val,value);
            v.insert( typename std::map<std::string, T>::value_type(name->get_content(),__val));
            member++;
        }
    }

    xml::node::const_iterator find_value( xml::node::const_iterator ) const;

};

struct encode_traits {
    void encode ( const base64_type & , xml::node::iterator ); // allowed for base64
    void encode ( const char * , xml::node::iterator  ); // allowed for string and base64
    void encode ( const string_type & , xml::node::iterator  ); // allowed for string and base64
    void encode ( const int_type &, xml::node::iterator  );   // allowed for int
    void encode ( const float_type &, xml::node::iterator  );      // allowed for float
    void encode ( const bool_type & , xml::node::iterator  );
    

    template <typename T>
    void encode ( const std::vector<T> & v , xml::node::iterator i ) {
        if(name_value != i->get_name())
          i=i->insert(xml::node(name_value.c_str()));
        
        typename std::vector<T>::const_iterator it = v.begin();
        xml::node::iterator target = i->insert(xml::node("array"))->insert(xml::node("data"));
        for(;it!=v.end();++it) {
            //WOOL_DEBUG("ENCODING: "<<)
            encode(*it,target);
        }
    }
    
    template <typename T>
    void encode ( const std::map<std::string, T> & v, xml::node::iterator i) {
        if(name_value != i->get_name())
          i=i->insert(xml::node(name_value.c_str()));
        typename std::map<std::string, T>::const_iterator it = v.begin();
        xml::node::iterator target = i->insert(xml::node("struct"));
        for(;it!=v.end();++it) {
            xml::node::iterator target_member = target->insert(xml::node("member"));
            target_member->insert(xml::node("name",it->first.c_str()));
            encode(it->second,target_member);
        }
    }
    
    template <typename extractTraitsT>
    void encode ( const variant<extractTraitsT, encode_traits> & v, xml::node::iterator i) {
      if(name_value != i->get_name()) {
        i->insert(v.encoded());
      } else {
	xml::node local(v.encoded());
	for(xml::node::iterator it=local.begin();it!=local.end();it++)
 	  i->push_back(*it);
        //*i = v.encoded();
      }
    }
};

template <typename extractTraitsT, typename encodeTraitsT>
class variant {
  private:
    xml::node m_local_node;

    mutable extractTraitsT m_extractor ;
    mutable encodeTraitsT m_encoder;
       
  public:
    typedef variant<extractTraitsT, encodeTraitsT> own_type;
  
    variant( ) : m_local_node("value") {
    }
    
    variant( xml::node::const_iterator it) {    
        m_local_node = *m_extractor.find_value(it);
    }
    variant( const own_type & v) {
      m_local_node = v.m_local_node;
    }
    
    xml::node encoded() const {
      return m_local_node;
    }
    
    template <typename T>
    variant(const T & v) :m_local_node("value") {
      m_encoder.encode(v,m_local_node.self());
    }
    
    
    own_type & operator = (const own_type & v) {
      m_local_node = v.m_local_node;
      return *this;
    }
    
    template <typename T>
    own_type & operator = (const T & v) {
      m_local_node = xml::node("value");
      m_encoder.encode(v,m_local_node.self());
      return *this;
    }
    
    template <typename T>
    void extract ( T & t) const {
      m_extractor.extract(t,m_local_node.self());
    }

    
    inline operator string_type () const { std::string res; extract(res); return res; }
    inline operator int_type () const 	 { int_type res; extract(res); return res; }
    inline operator float_type () const  { float_type res; extract(res); return res; }
    inline operator bool_type () const 	 { bool_type res; extract(res); return res; }
    inline operator base64_type () const { base64_type res; extract(res); return res; }
    
    template <typename T>
    operator std::map<std::string,T> () const { 
    	std::map<std::string,T> res; 
    	extract(res); 
    	return res;
    }
    
    template <typename T>
    operator std::vector<T> () const { 
    	std::vector<T> res; 
    	extract(res); 
    	return res;
    }
    
     
};

//typedef variant_type<

template <typename extractTraitsT, typename encodeTraitsT>
class params {
public:
    typedef variant<extractTraitsT,encodeTraitsT> variant_type;
    typedef std::map<std::string, variant_type>   variant_map_type;
    typedef std::vector<variant_type>   variant_array_type;
    
    inline params() {}

    inline void set_root( xml::node::iterator root ) {
        m_root = root;
        m_current_output_node = root; 
        m_current_input_node = m_root->begin();
        if(m_current_input_node != m_root->end()) {
            WOOL_DEBUG("input node " << *m_current_input_node);
        }
    }

    inline void rewind () {
	    set_root(m_root);
    }

    inline ~params() {}
    ;

    template <typename T>
    params & operator >> ( T & t) {
        extract(t);
        return (*this);
    }

    template <typename T>
    params & operator << ( T & t) {
        encode(t);
        return (*this);
    }

protected:
    inline void check_eof() const {
        while(m_current_input_node!=m_root->end()) {
            if(m_current_input_node->get_type()==xml::node::type_element)
                return;
            m_current_input_node++;
        }
        throw xmlrpc::exception("xmlrpc payload eof",1);
    }

    template <typename T>
    void encode ( const T & t) {
        m_encoder.encode(t,m_current_output_node->insert(xml::node(name_param.c_str()))->insert(xml::node(name_value.c_str())));
    }

    template <typename T>
    void extract ( T & t) const {
        check_eof();
        m_extractor.extract(t,m_current_input_node);
        m_current_input_node++;
    }

protected:
    mutable extractTraitsT m_extractor ;
    mutable encodeTraitsT m_encoder ;
private:
    mutable xml::node::const_iterator m_current_input_node;
    xml::node::iterator m_current_output_node;
    xml::node::iterator m_root;
};


class request :public params<extract_traits,encode_traits> {
public:    
    inline request () : m_doc("methodCall"),m_path("/") {
	m_doc.set_encoding("utf-8");
        m_method = m_doc.get_root_node().insert(xml::node("methodName"));
        set_root(m_doc.get_root_node().insert(xml::node("params")));
    }


    inline request & operator = (const request & p ) {
        m_doc = p.m_doc;
	m_doc.set_encoding("utf-8");
        reset_root_node();
        return *this;
    }

    inline void load ( std::istream & is ) {
        wool::http::request msg;
        is >> msg;
        WOOL_DEBUG("HTTP parsed");
        std::ostringstream body;
        body << msg.body << " ";


        WOOL_DEBUG("request body '"<<body.str()<<"'");
        set_path(msg.path);
		WOOL_DEBUG("request path '"<<get_path()<<"'");
		std::string s(body.str());
        xml::tree_parser parser(s.data(),s.length(),true);
   	m_doc = parser.get_document();
	m_doc.set_encoding("utf-8");
        WOOL_DEBUG("reset root node");
        reset_root_node();

    }

    inline void save ( std::ostream & os ) const {
        wool::http::request msg;

        msg.method = "POST";
        msg.path = m_path;
        msg.protocol = "HTTP/1.0";

        wool::swappable_data::ostream bs;
        bs.open(msg.body);
        bs << m_doc << std::flush;
        bs.close();
        std::ostringstream cl;
        cl << msg.body.size();

        msg.header.insert(msg.header.end(),wool::rfc2822::header::value_type("Content-length",cl.str()));
        msg.header.insert(msg.header.end(),wool::rfc2822::header::value_type("Content-type","text/xml"));
        msg.header.insert(msg.header.end(),wool::rfc2822::header::value_type("Connection","keep-alive"));
        msg.header.insert(msg.header.end(),wool::rfc2822::header::value_type("Keep-alive","300"));

        os << msg.method << ' ' << msg.path << ' ' << msg.protocol << '\r' << '\n'
        << msg.header << '\r' << '\n'
        << msg.body;
    }

    inline ~request() {}

    inline string_type get_path() const {
        return m_path;
    }
    inline void set_path( string_type s) {
        m_path = s;
    }

    string_type get_method() const {
        return m_method->get_content();
    }
    void set_method( string_type s) {
        m_method->set_content(s.c_str());
    }

private:
    xml::document m_doc;
    std::string m_path;
    xml::node::iterator m_method;

    inline void reset_root_node () {
        xml::node::iterator it = m_doc.get_root_node().find("params");
        if(it==m_doc.get_root_node().end())
            it=m_doc.get_root_node().insert(xml::node("params"));
        WOOL_DEBUG("params root node" << *it);
        set_root(it);

        m_method = m_doc.get_root_node().find("methodName");
        if(m_method==m_doc.get_root_node().end())
            throw xmlrpc::exception("methodName tag missing",0);
        //set_method(it->get_content());
    }
};

class response :public params<extract_traits,encode_traits> {
public:
    inline response () : m_doc("methodResponse") {
	m_doc.set_encoding("utf-8");
        set_root(m_doc.get_root_node().insert(xml::node("params")));
    }

    inline void load ( std::istream & is ) {
        wool::http::response msg;
        is >> msg;
        std::ostringstream body;
        body << msg.body;
        WOOL_DEBUG("response body: " << body.str());

        xml::tree_parser parser(body.str().data(),msg.body.size(),false);
        m_doc = parser.get_document();
        //if(m_doc.get_root_node().)
    	//	m_doc.set_root_node(xml::node("methodResponse"));
        m_doc.set_encoding("utf-8");
        reset_root_node();
    }

    response & operator = (const response & p ) {
        m_doc = p.m_doc;
	m_doc.set_encoding("utf-8");
        reset_root_node();
        return *this;
    }
    
    inline void save ( std::ostream & os ) const {
        wool::http::response msg;

        msg.status_code = 200;
        msg.message = "OK";
        msg.protocol="HTTP/1.0";

        wool::swappable_data::ostream bs;
        bs.open(msg.body);
        bs << m_doc << std::flush;
        bs.close();
        std::ostringstream cl;
        cl << msg.body.size();

        msg.header.insert(msg.header.end(),wool::rfc2822::header::value_type("Content-length",cl.str()));
        msg.header.insert(msg.header.end(),wool::rfc2822::header::value_type("Content-type","text/xml"));
        msg.header.insert(msg.header.end(),wool::rfc2822::header::value_type("Connection","keep-alive"));
     //   msg.header.insert(msg.header.end(),wool::rfc2822::header::value_type("Keep-alive","300"));

        os << msg.protocol << ' ' << msg.status_code << ' ' << msg.message <<  '\r' << '\n'
        << msg.header << '\r' << '\n'
        << msg.body;
    }

    inline ~response() {}

private:
    xml::document m_doc;
    inline void reset_root_node () {
      if(m_doc.get_root_node().get_name()!=std::string("methodResponse"))
         throw jackal::xmlrpc::exception("invalid response",0);

      xml::node::iterator it = m_doc.get_root_node().find("fault");
      if(it!=m_doc.get_root_node().end()) {
         xml::node::iterator it2 = it->find("value");
	 std::string reason = "Unknown reason";
	 int code = -1;
	 if(it2!=it->end()) {
         	std::map<std::string, std::string> error;
         	m_extractor.extract(error, it);
         	reason = error["faultString"];
		code =atoi(error["faultCode"].c_str());
	}
	throw jackal::xmlrpc::exception(reason, code);
      }

      it = m_doc.get_root_node().find("params");
      if(it==m_doc.get_root_node().end())
         throw xmlrpc::exception("params tag missing",0);
      set_root(it);
    }
};
class RequestHandler;
class RequestDispatcher {
	public:
		typedef std::map<std::string, RequestHandler*> HandlerMap;
		
		RequestDispatcher();
		virtual ~RequestDispatcher();
		
		void setCloseOnFault( bool b );
		bool getCloseOnFault( ) const;
		
		void handleSession ( std::istream & input, std::ostream & output );
		
		void registerUrl(const std::string & url, RequestHandler * p);
		void unregisterUrl(const std::string & url);
		
	protected:
		virtual void handleBaseError ( const std::exception & e );
		virtual void handleRpcError ( const jackal::xmlrpc::exception & e );
	
	private:
		bool m_closeOnFault;
		HandlerMap m_handlers;
};

class RequestHandler {
	public:
		typedef jackal::xmlrpc::int_type int_type;
		typedef jackal::xmlrpc::string_type string_type;
		typedef jackal::xmlrpc::float_type float_type;
		typedef jackal::xmlrpc::bool_type bool_type;
		typedef jackal::xmlrpc::base64_type base64_type;

		RequestHandler ();
		virtual ~RequestHandler ();
		
		void registerUrl(const std::string & url, RequestDispatcher & parent);
		void unregisterUrl();
		
		virtual bool handleRequest ( request & req, response & res );
		
	private:
		RequestDispatcher * m_parent;
		std::string m_url;
};
#define JACKAL_BEGIN_MAPPING inline bool handleRequest(jackal::xmlrpc::request & req, jackal::xmlrpc::response & res ) { 

#define JACKAL_MAP0(symbol,function) \
if((req.get_method()==symbol)) {\
	jackal::xmlrpc::response::variant_type v = function();\
	res << v;\
	return true;\
}

#define JACKAL_MAP1(symbol,function) \
if((req.get_method()==symbol)) {\
	jackal::xmlrpc::request::variant_type p1;\
	req >> p1;\
	jackal::xmlrpc::response::variant_type v =  function(p1);\
	res << v;\
	return true;\
}

#define JACKAL_MAP2(symbol,function) \
if((req.get_method()==symbol)) {\
	jackal::xmlrpc::request::variant_type p1,p2;\
	req >> p1 >> p2;\
	jackal::xmlrpc::response::variant_type v =  function(p1,p2);\
	res << v;\
	return true;\
}

#define JACKAL_MAP3(symbol,function) \
if((req.get_method()==symbol)) {\
	jackal::xmlrpc::request::variant_type p1,p2,p3;\
	req >> p1 >> p2 >> p3;\
	jackal::xmlrpc::response::variant_type v =  function(p1,p2,p3);\
	res << v;\
	return true;\
}

#define JACKAL_MAP4(symbol,function) \
if((req.get_method()==symbol)) {\
	jackal::xmlrpc::request::variant_type p1,p2,p3,p4;\
	req >> p1 >> p2 >> p3 >> p4;\
	jackal::xmlrpc::response::variant_type v =  function(p1,p2,p3,p4);\
	res << v;\
	return true;\
}

#define JACKAL_MAP5(symbol,function) \
if((req.get_method()==symbol)) {\
	jackal::xmlrpc::request::variant_type p1,p2,p3,p4,p5;\
	req >> p1 >> p2 >> p3 >> p4 >> p5;\
	jackal::xmlrpc::response::variant_type v =  function(p1,p2,p3,p4,p5);\
	res << v;\
	return true;\
}

#define JACKAL_MAP6(symbol,function) \
if((req.get_method()==symbol)) {\
	jackal::xmlrpc::request::variant_type p1,p2,p3,p4,p5,p6;\
	req >> p1 >> p2 >> p3 >> p4 >> p5 >> p6;\
	jackal::xmlrpc::response::variant_type v =  function(p1,p2,p3,p4,p5,p6);\
	res << v;\
	return true;\
}

#define JACKAL_END_MAPPING return false; }

#define JACKAL_MAP_PARENT(__jackal_parent) if(__jackal_parent::handleRequest(req,res)) return true;


};

};

namespace std {
inline std::istream & operator >> (std::istream & is, jackal::xmlrpc::request & r ) {
    r.load(is);
    return is;
}

inline std::ostream & operator << (std::ostream & os, const jackal::xmlrpc::request & r ) {
    r.save(os);
    return os;
}

inline std::istream & operator >> (std::istream & is, jackal::xmlrpc::response & r ) {
    r.load(is);
    return is;
}

inline std::ostream & operator << (std::ostream & os, const jackal::xmlrpc::response & r ) {
    r.save(os);
    return os;
}

inline std::ostream & operator << (std::ostream & os, const jackal::xmlrpc::exception & r ) {
    /*
    <methodResponse>
    <fault>
    <value>
    <struct>
    <member>
    <name>faultCode</name>
    <value><int>4</int></value>
    </member>
    <member>
    <name>faultString</name>
    <value><string>Too many parameters.</string></value>
    </member>
    </struct>
    </value>
    </fault>
    </methodResponse>
    */
    xml::document d ("methodResponse");
    d.set_encoding("utf-8");
    xml::node::iterator it = d.get_root_node().insert(xml::node("fault"))->insert(xml::node("value"))->insert(xml::node("struct"));
    xml::node::iterator fc = it->insert(xml::node("member"));
    xml::node::iterator fs = it->insert(xml::node("member"));

    std::ostringstream oss;
    oss << r.id();

    fc->insert(xml::node("name","faultCode"));
    fc->insert(xml::node("value"))->insert(xml::node("i4",oss.str().c_str()));

    fs->insert(xml::node("name","faultString"));
    fs->insert(xml::node("value",r.what()));

    std::ostringstream txt;
    txt << d;
    os  << "HTTP/1.0 200 OK\r\n"
    <<"Content-type: text/xml\r\n"
    <<"Content-Length:" << txt.str().length() <<"\r\n"
    <<"Connection: keep-alive\r\n"
    <<"\r\n"
    << txt.str();
    return os;
}
};
#endif
