/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/**************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef WOOL_BASE64_H
#define WOOL_BASE64_H
#include "../wool/debug.h"

namespace wool {
	

	struct filter_pair {
		
	};

	namespace decoder {
		template <typename OutputIterator>
		struct base64{
			typedef const char & const_reference ;
			typedef typename std::back_insert_iterator<base64<OutputIterator > > back_insert_iterator;

			OutputIterator out;
			unsigned char buffer;
			unsigned char bsize;
			unsigned char counter;

			base64(): buffer(0u), bsize(0u), counter(0u) {}
			base64(OutputIterator o): out(o) , buffer(0u), bsize(0u), counter(0u) {}
			base64(const base64<OutputIterator> & b): out(b.out), buffer(b.buffer), bsize(b.bsize), counter(b.counter) {}

			inline void decode ( const_reference c ) {
				if((c>='A')&&(c<='Z')) {
					add_char(c-'A');
				} else if((c>='a')&&(c<='z')) {
					add_char( c-'a' + 'Z'-'A' + 1);
				} else if((c>='0')&&(c<='9')) {
					add_char(c - '0' + 'z'-'a' + 'Z'-'A' + 2);
				} else if (c == '+') {
					add_char(0x3e);
				} else if (c == '/') {
					add_char(0x3f);
				} else {
					WOOL_DEBUG("ignore "<< (void *)c);
				}
			}

			inline void add_char (int __cc) {
				unsigned char cc = static_cast<unsigned char>(__cc);
				if(bsize>=2) {
					*out = (buffer << (8-bsize)) | (cc >> (6 - 8 + bsize));
					out++;
				}
				buffer = cc;
				bsize = (6 + bsize) % 8;
			}
		
			inline void flush() {
				//pad();
			}

			inline void push_back (const_reference cc) {
				decode(cc);
			}
		};
	};

	namespace encoder {
		template <typename OutputIterator>
		struct base64{
			typedef const char & const_reference ;
			typedef typename std::back_insert_iterator<base64<OutputIterator > > back_insert_iterator;

			OutputIterator out;
			unsigned char buffer;
			unsigned char bsize;
			unsigned char counter;

			base64(): buffer(0u), bsize(0u), counter(0u) {}
			base64(OutputIterator o): out(o) , buffer(0u), bsize(0u), counter(0u) {}
			base64(const base64<OutputIterator> & b):out(b.out), buffer(b.buffer), bsize(b.bsize), counter(b.counter) {}
			
			inline void pad() {
				if(bsize>0) {
					add_char(0x3f & ( buffer << (6-bsize) ));
					bsize = 0u;
				}
				while (( counter % 4)>0){
					*out = '=';
					*out++;
					counter ++;
				}
			}

			inline void add_char (int c) {
				WOOL_DEBUG("Add char " << c << " at " << (int)counter );
				static const char * dictionary="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
				*out = dictionary[c];
				out++;
				counter++;
				if(counter >= 76) {
					*out = '\r';
					out++;
					*out = '\n';
					out++;
					counter = 0u;
				}
			}

			inline void encode (const_reference cc) {
				unsigned const char & c = *((unsigned const char * )&cc);
				add_char( 0x3f & ( ( buffer << (6-bsize) ) | ( c >> (2+bsize) ) ) ) ;
				buffer =c;
				if(bsize>=4u) {
					bsize = 0u;
					add_char(0x3f & buffer);
				} else {
					bsize += 2u;
				}
			}
			
			inline void flush() {
				pad();
			}

			inline void push_back (const_reference cc) {
				encode(cc);
			}
		};
	}
};

#endif
