/***************************************************************************
 *   Copyright (C) 2005 by Vukasin Toroman                                 *
 *   vtoroman@vtoroman05                                                   *
 *                                                                         *
 *  For more licensing information consult COPYING                         *
 ***************************************************************************/
#include "../jackal/http.h"
#include <iostream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fstream>

namespace jackal {
  namespace http {
	
    boost::mutex RequestHandler::s_handlerMapGuard;
    RequestHandler::MapType RequestHandler::s_handlerMap;
    
    
	
    void RequestQueue::handle( UnitPtr u ) {
      wool::http::request req;
      wool::http::response res;
      RequestHandler::MapEntryType handler;
      std::string path;
      (*u) >> req;
      
      path = req.path;
      bool ok = false;
      int qsign = path.find('?');
      if(qsign!=std::string::npos)
	path.erase(qsign,path.length());
      path.push_back('/');
      
      res.status_code = 200;
      res.message = "OK";
      res.protocol = "HTTP/1.0";
      
	do {
	  int index = path.find_last_of("/");
	  path.erase((index<0)?0:index, path.length());
	  
	  handler = RequestHandler::ResolvePath(path);
	  ok = handler->shouldHandle(req);
	}while((!ok)&&(path.length()>0));
	try {
	  if(ok) {
	    handler->handle(req, res);
	    (*u) << res;
	    u->flush();
	  } else {
	    throw jackal::http::exception(404);
	  }
	} catch( const jackal::http::exception & e ) {
	  (*u)	<< "HTTP/1.0 "<< e.id << ' ' <<e.what() << std::endl
		<< "Connection: close" << std::endl
		<< "Content-type: text/plain" << std::endl
		<< std::endl
		<< e.what()
		<< std::endl;
	  return;
	}
    }
	
    RequestQueue::RequestQueue(){
    }

    RequestQueue::~RequestQueue(){
    }
  

    void RequestQueue::onError(const std::exception & e, UnitPtr u) {
      std::string error_text = e.what();
      (*u)	<< "HTTP/1.0 500 Internal server error" << std::endl
		<< "Connection: close" << std::endl
		<< "Content-type: text/plain" << std::endl
		<< std::endl
		<< error_text
		<< std::endl;
    }
	
    void RequestHandler::registerPath(const RequestHandler::PathType & p) {
      boost::mutex::scoped_lock l(s_handlerMapGuard);
      s_handlerMap[p] = this;
    }
	
    void RequestHandler::unRegisterPath(const RequestHandler::PathType & p) {
      boost::mutex::scoped_lock l(s_handlerMapGuard);
      MapType::iterator it = s_handlerMap.find(p);
      if(it!=s_handlerMap.end())
	s_handlerMap.erase(it);
    }
	

	
    RequestHandler::~RequestHandler() {
      boost::mutex::scoped_lock l(s_handlerMapGuard);
      RequestHandler::MapType::iterator it = s_handlerMap.begin();
      RequestHandler::MapType::iterator next_it;
      while(it!=s_handlerMap.end()) {
	next_it = it;
	next_it++;
	if(it->second == this)
	  s_handlerMap.erase(it);
	it = next_it;
      }
    }
	
    RequestHandler::MapEntryType RequestHandler::ResolvePath(const RequestHandler::PathType & p) {
      static NullHandler null_handler;
      static MapEntryType null_handler_ptr (&null_handler);
		
      boost::mutex::scoped_lock l(s_handlerMapGuard);
      MapType::iterator it = s_handlerMap.find(p);
      if(it!=s_handlerMap.end())
	return it->second;
      return null_handler_ptr;
    }
	
		
    FileServer::FileServer( const PathType & p ) :m_root(p), m_magicDb(NULL) {
      m_magicDb = magic_open(MAGIC_MIME);
      magic_load(m_magicDb, NULL);
    }
		
    FileServer::~FileServer() {
      if(m_magicDb) {
	magic_close(m_magicDb);
      }
    }
			
    bool FileServer::shouldHandle( const Request & req ) const {
      return true;
    }
		
    void FileServer::handle( const Request & req, Response & res ) {
      MountPointSet::const_iterator it = m_mountPoints.begin();
      for(;it!=m_mountPoints.end();it++) {
	if(req.path.find(*it) == 0) { 
	  PathType path = m_root;
	  path.append(PathType(req.path, it->length(), req.path.length() ));
	  std::cout << "send file '" << path << "'" <<std::endl;
	  std::cout << "file type '" << magic_file(m_magicDb,path.c_str()) << "'" <<std::endl;
					
	  res.body.clear();
	  std::filebuf is;
	  is.open(path.c_str(),std::ios::in|std::ios::binary);
	  wool::swappable_data::ostream os;
	  os.open(res.body);
	  os << &is;
	  os.close();
					
	  std::ostringstream cl;
	  cl << res.body.size();
	  res.protocol = "HTTP/1.0";
	  res.message = "OK";
	  res.status_code = 200;
	  res.header.clear();
	  res.header.insert(wool::http::header::value_type("Connection","close"));
	  res.header.insert(wool::http::header::value_type("Content-Type",magic_file(m_magicDb,path.c_str()) ));
	  res.header.insert(wool::http::header::value_type("Content-Length",cl.str()));
	  return;
	}
      }
      throw jackal::http::exception(404);
    }
		
    void FileServer::registerPath(const PathType & p) {
      RequestHandler::registerPath(p);
      m_mountPoints.insert(p);
    }

    void FileServer::unRegisterPath(const PathType & p) {
      RequestHandler::unRegisterPath(p);
      m_mountPoints.erase(p);
    }

    exception::exception(const int & i ) throw() : id(i) {};
    exception::~exception() throw() {}

#define HTTP_EXCEPTION_NAME(i,name) if(id==i) { static const char * s_name = name; return s_name; }

    const char * exception::what() const throw() {
      HTTP_EXCEPTION_NAME(404,"Not found");
      HTTP_EXCEPTION_NAME(407,"Blocked");
      HTTP_EXCEPTION_NAME(500,"Internal server error");
      static const char * s_unknown = "Error";
      return s_unknown;
    }
		
		
			

  };
};


