/***************************************************************************
 *   Copyright (C) 2005 by Vukasin Toroman                                 *
 *   vtoroman@vtoroman05                                                   *
 *                                                                         *
 *  For more licensing information consult COPYING                         *
 ***************************************************************************/
#ifndef JACKAL_HTTP_H 
#define JACKAL_HTTP_H

#include <boost/shared_ptr.hpp>
#include "../base/queue.h"
#include <iostream>
#include <sstream>
#include "../jackal/http.h"
#include "../rfc2822/http.h"
#include "../jackal/jackal.h"
#include <magic.h>

namespace jackal {
  namespace http {
    typedef wool::http::request Request;
    typedef wool::http::response Response;
	
    class exception : public std::exception {
    public:
      const int id;
      exception(const int & i ) throw();
      virtual ~exception() throw();
      virtual const char * what() const throw();
    };

    class RequestHandler {
    public:
      typedef std::string PathType;
      typedef RequestHandler* MapEntryType;
      typedef std::map<PathType,MapEntryType> MapType;
      virtual ~RequestHandler();
      virtual bool shouldHandle( const Request & req ) const = 0;
      virtual void handle( const Request & req, Response & res ) = 0;
      virtual void registerPath(const PathType & p);
      virtual void unRegisterPath(const PathType & p);
      static MapEntryType ResolvePath(const PathType & p);
    public:
      static boost::mutex s_handlerMapGuard;
      static MapType s_handlerMap;
    };
	
    class NullHandler : public RequestHandler {
    public:
      inline NullHandler(){};
      inline virtual ~NullHandler() {}
      virtual inline bool shouldHandle( const Request & req ) const { return false; };
      virtual inline void handle( const Request & req, Response & res ) {};
    };
	
    class FileServer : public RequestHandler {
    public:
      typedef std::set<PathType> MountPointSet;
      FileServer( const PathType & root );
      virtual ~FileServer();
			
      virtual bool shouldHandle( const Request & req ) const;
      virtual void handle( const Request & req, Response & res );
      virtual void registerPath(const PathType & p);
      virtual void unRegisterPath(const PathType & p);
    private:
      MountPointSet m_mountPoints;
      PathType m_root;
      magic_t m_magicDb;
    };
	
    class RequestQueue : public wool::Queue<std::iostream> {
    public:
      RequestQueue();
      virtual ~RequestQueue();

      virtual void handle(UnitPtr u);
      virtual void onError(const std::exception & e, UnitPtr u );
    };
	
  };
};

#endif
