/***************************************************************************
*   Copyright (C) 2005 by Vukasin Toroman                                 *
*   vtoroman@astaro.de                                                    *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Library General Public License as       *
*   published by the Free Software Foundation; either version 2 of the    *
*   License, or (at your option) any later version.                       *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Library General Public     *
*   License along with this program; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <jackal/xmlrpc.h>
#include <swool/streambuf_bio.h>

#include <jackal/xmlrpc.h>
#include <swool/streambuf_bio.h>
#include <boost/bind.hpp>

namespace jackal
{

  const value value::Zero = value(value::number_type(0.0));
  const value value::EmptyString = value(value::string_type(""));
  const value value::EmptyMap = value(value::map_type());
  const value value::EmptyArray = value(value::array_type());

  const std::string & value::TypeName( type t )
  {
    static std::string StringName = "String";
    static std::string NumberName = "Number";
    static std::string ArrayName = "Array";
    static std::string MapName = "Map";

    switch(t)
    {
    case string:
      return StringName;
    case number:
      return NumberName;
    case array:
      return ArrayName;
    case map:
      return MapName;
    }
    return MapName;
  }


  value::value( const value & v )
      : m_type(v.m_type)
      , m_map(v.m_map)
      , m_array(v.m_array)
      , m_conversions(v.m_conversions)
      , m_string(v.m_string)
      , m_number(v.m_number)
  {}


  value::value( const string_type & v )
      : m_type(value::string)
      , m_conversions(value::string)
      , m_string(v)
  {}

  value::value( const number_type & v )
      : m_type(value::number)
      , m_conversions(value::number)
      , m_number(v)
  {}

  value::value( const array_type & v )
      : m_type(value::array)
      , m_array(v)
      , m_conversions(value::array)
  {}

  value::value( const map_type & v )
      : m_type(value::map)
      , m_map(v)
      , m_conversions(value::map)
  {
  }

  value::~value() {}

  value::type value::get_type() const {	return m_type;	}
  value & value::operator = ( const value & v ) {
	m_type = v.m_type;
	m_map = v.m_map;
      	m_array = v.m_array;
        m_conversions = v.m_conversions;
        m_string = v.m_string;
        m_number = v.m_number;
	return *this;
  }

  value & value::operator = ( const bool & t) {  set_number(t); return *this; }
  value & value::operator = ( const unsigned int & t) {  set_number(t); return *this; }
  value & value::operator = ( const int & t) {  set_number(t); return *this; }
  value & value::operator = ( const unsigned long int & t) {  set_number(t); return *this; }
  value & value::operator = ( const long int & t) {  set_number(t); return *this; }
  value & value::operator = ( const unsigned long long int & t) {  set_number(t); return *this; }
  value & value::operator = ( const long long int & t) {  set_number(t); return *this; }
  value & value::operator = ( const float & t) {  set_number(t); return *this; }
  value & value::operator = ( const double & t) {  set_number(t); return *this; }
  value & value::operator = ( const number_type & t) {  set_number(t); return *this; }
  value & value::operator = ( const char * t) {  set_string(t); return *this; }
  value & value::operator = ( const string_type & t) {  set_string(t); return *this; }
  value & value::operator = ( const map_type & t) {  set_type(map); m_map=t; return *this; }
  value & value::operator = ( const array_type & t) {  set_type(array); m_array=t; return *this; }

  value::operator bool () const {		return get_number();	}
  value::operator int () const {		return get_number();	}
  value::operator long int () const {		return get_number();	}
  value::operator long long int () const {		return get_number();	}
  value::operator unsigned int () const {		return get_number();	}
  value::operator unsigned long int () const {		return get_number();	}
  value::operator unsigned long long int () const {		return get_number();	}
  value::operator float() const {		return get_number();	}
  value::operator double() const {		return get_number();	}
  value::operator number_type() const {		return get_number();	}
  value::operator string_type() const {		return get_string();	}
  value::operator map_type() const {		return get_map();	}
  value::operator array_type() const {		return get_array();	}

  const value & value::operator [] (const int & i) const throw (jackal::exception)
  {
    if(i>=m_array.size())
      return Zero;
    return m_array[i];
  }

  const value & value::operator [] (const key_type & i) const throw (jackal::exception)
  {
    map_type::const_iterator it = m_map.find(i);
    if(it == m_map.end())
      return Zero;
    return it->second;
  }

  const value & value::operator [] (const char * i) const throw (jackal::exception)
  {
    map_type::const_iterator it = m_map.find(i);
    if(it == m_map.end())
      return Zero;
    return it->second;
  }

  value & value::operator [] (const int & i) throw (jackal::exception)
  {
    set_type(array);
    m_array.reserve(i);
    return m_array[i];
  }

  value & value::operator [] (const key_type & i) throw (jackal::exception)
  {
    set_type(map);
    return m_map[i];
  }

  value & value::operator [] (const char * i) throw (jackal::exception)
  {
    set_type(map);
    return m_map[i];
  }


  void value::set_type( type t )
  {
    if(t==m_type)
    {
      return;
    }
    switch(t)
    {
    case string:
    	get_string();
      m_array.clear();
      m_map.clear();
      m_number = 0.0;
      break;
    case number:
    	get_number();
      m_string.clear();
      m_array.clear();
      m_map.clear();
      break;
    case array:
      m_map.clear();
      m_string.clear();
      m_number = 0.0;
      break;
    case map:
      m_array.clear();
      m_number = 0.0;
      m_string.clear();
      break;
    }
    m_type = t;
    m_conversions = t;
  }

  void value::test_type(value::type t) const throw(jackal::exception)
  {
    static const std::string errmsg = "Type missmatch";
    static const int errid = 0;
    if(t!=m_type)
    {
      throw jackal::exception(errmsg, errid);
    }
  }

  value::map_type & value::get_map()
  {
    return m_map;
  }

  const value::map_type & value::get_map() const
  {
    return m_map;
  }

  value::array_type & value::get_array()
  {
    return m_array;
  }

  const value::array_type & value::get_array() const
  {
    return m_array;
  }

  const value::number_type & value::get_number() const
  {
    if((m_conversions&number)==0)
    {
      switch (m_type)
      {
      case string:
        m_number = atof(m_string.c_str());
        break;
      case map:
        m_number = m_map.size();
        break;
      case array:
        m_number = m_array.size();
        break;
      default:
        assert(false);
      }
      m_conversions|=number;
    } else {
    }
    return m_number;
  }

  void value::set_number( const number_type & n )
  {
    m_type = number;
    m_conversions = number;
    m_array.clear();
    m_map.clear();
	 m_string.clear();
    m_number = n;
  }

  const value::string_type & value::get_string() const
  {
    if( (m_conversions&string)==0)
    {
      std::ostringstream ostr;
      switch (m_type)
      {
      case number:
        ostr << m_number;
        break;
      case array:
        ostr << "array[ " ;
        for(array_type::const_iterator it = m_array.begin(); it != m_array.end(); ++it)
        {
          if(it!=m_array.begin())
            ostr << std::endl << " ,'";
          else
            ostr << std::endl << "  '";
          ostr << (std::string)(*it) << "'";
        }
        ostr << std::endl << " ]" << std::endl;
        break;
      case map:
        ostr << "map{ ";
        for(map_type::const_iterator it = m_map.begin(); it != m_map.end(); ++it)
        {
          if(it!=m_map.begin())
            ostr << std::endl << " ,'";
          else
            ostr << std::endl << "  '";
          ostr << it->first << "'=>'" << (std::string)(it->second) << "'";
        }
        ostr << std::endl << " }" << std::endl;
        break;
      default:
        assert(false);
      }
      m_string = ostr.str();
      m_conversions|=string;
    }
    return m_string;
  }

  void value::set_string( const value::string_type & n )
  {
    m_type = string;
    m_conversions = string;
    m_array.clear();
    m_map.clear();
    m_string = n;
  }
};
