/***************************************************************************
 *   Copyright (C) 2005 by Vukasin Toroman                                 *
 *   vtoroman@astaro.de                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef JACKAL_H
#define JACKAL_H

#include <string>
#include <vector>
#include <map>
#include <iostream>

#include <xmlwrapp/xmlwrapp.h>
#include "../rfc2822/message_rfc2822.h"
#include "../rfc2822/http.h"
#include "../rfc3986/url.h"


using namespace std;
inline std::string serialize_url_path ( wool::url::path_type & p ) {
    std::string  res = "";
    for(wool::url::path_type::const_iterator i=p.begin();i!=p.end();++i) {
        res += "/";
        res += *i;
    }

    return res;
}
#ifndef stringify
#define stringify(s) #s
#endif

#ifndef xstr
#define xstr(s) stringify(s)
#endif

#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <sstream>
#include <xmlwrapp/xmlwrapp.h>
#include "../rfc2822/message_rfc2822.h"
#include "../rfc2822/http.h"
#include "../rfc3986/url.h"

using namespace std;

namespace jackal {
class exception
    :public std::exception {
public:
    inline exception( int id, std::string s) throw ():m_error_message(s), m_id(id) {}
    inline exception( std::string s, int id) throw ():m_error_message(s), m_id(id) {}
    
    inline const char * what () const throw () {
        return m_error_message.c_str();
    }
    inline int id () const throw () {
        return m_id;
    }
    inline virtual ~exception() throw () {}
    
protected:
    std::string m_error_message;
    int m_id;
};


class value {
public:
    typedef long double number_type;
    typedef std::string string_type;
    typedef std::string key_type;
    typedef std::map<key_type, value> map_type;
    typedef std::vector<value> array_type;
    typedef enum { map=0x01, array=0x02, string=0x04, number=0x08 } type;

    static const std::string & TypeName( type t );

    static const value Zero;
    static const value EmptyString;
    static const value EmptyMap;
    static const value EmptyArray;

    value( const value & v = EmptyString );
    value( const string_type & v );
    value( const number_type & v );
    value( const array_type & v );
    value( const map_type & v );
    ~value();

    void set_type(type t);
    type get_type() const;
    void test_type(type t) const throw (jackal::exception);

    map_type & get_map();
    const map_type & get_map() const;

    array_type & get_array();
    const array_type & get_array() const;

    const number_type & get_number() const;
    void set_number( const number_type & n );

    const string_type & get_string() const;
    void set_string( const string_type & n );


    value & operator [] (const int & i) throw (jackal::exception);
    value & operator [] (const key_type & i) throw (jackal::exception);
    value & operator [] (const char * i) throw (jackal::exception);

    const value & operator [] (const int & i) const throw (jackal::exception);
    const value & operator [] (const key_type & i) const throw (jackal::exception);
    const value & operator [] (const char * i) const throw (jackal::exception);
	 value & operator = ( const value & t);
	 value & operator = ( const bool & t);
    value & operator = ( const int & t);
    value & operator = ( const long int & t);
    value & operator = ( const long long int & t);

    value & operator = ( const unsigned int & t);
    value & operator = ( const unsigned long int & t);
    value & operator = ( const unsigned long long int & t);

    value & operator = ( const float & t);
    value & operator = ( const double & t);
    
    value & operator = ( const number_type & t);
    value & operator = ( const char * t);
    value & operator = ( const string_type & t);
    value & operator = ( const map_type & t);
    value & operator = ( const array_type & t);


	 operator bool () const;
	 
    operator int () const;
    operator long int () const;
    operator long long int () const;

    operator unsigned int () const;
    operator unsigned long int () const;
    operator unsigned long long int () const;

	 operator float() const;
	 operator double() const;
	
    operator number_type() const;
    operator string_type() const;
    operator map_type() const;
    operator array_type() const;

    template<typename KeyT, typename ValueT>
    value & operator = ( const std::map<KeyT, ValueT> & t) {
        typename std::map<KeyT, ValueT>::const_iterator it = t.begin();
        set_type(map);
        m_map.clear();
        for(;it!=t.end();it++)
            m_map.insert(typename std::pair<KeyT,ValueT>::value_type(it->first, it->second));
        return *this;
    }

    template<typename ValueT>
    value & operator = ( const std::list<ValueT> & t) {
        typename std::list<ValueT>::const_iterator it = t.begin();
        set_type(array);
        m_array.clear();
        m_array.reserve(t.size());
        for(;it!=t.end();it++)
            m_array.insert(m_array.end(),*it);
        return *this;
    }

    template<typename ValueT>
    value & operator = ( const std::vector<ValueT> & t) {
        typename std::vector<ValueT>::const_iterator it = t.begin();
        set_type(array);
        m_array.clear();
        m_array.reserve(t.size());
        for(;it!=t.end();it++)
            m_array.insert(m_array.end(),*it);
        return *this;
    }

private:
    type m_type;
    map_type m_map;
    array_type m_array;

    mutable int m_conversions;
    mutable string_type m_string;
    mutable number_type m_number;
};


class Request {
public:
    std::string method;
    value::array_type parameters;
};

class Response {
public:
	 jackal::Request * request;
    value::array_type parameters;
};

class RequestHandler {
public:
    virtual inline ~RequestHandler() {};
    virtual void handle( const Request & req, Response & res ) = 0;
};


};


#define DECLARE_PREPARE_PROXY( _j_name) \
	jackal::Request  req;\
	jackal::Response res;\
	req.method = stringify(_j_name);\
	res.request = &req;\

#define DECLARE_PROXY_EXEC(type)\
	this->handle(req, res);\
	\
	return res.parameters.empty()?(type)jackal::value::EmptyString:(type)res.parameters[0];

#define DECLARE_JACKAL_PROXY0(_j_name,type) \
inline type _j_name( ) { \
	DECLARE_PREPARE_PROXY(_j_name);\
	DECLARE_PROXY_EXEC(type);\
}

#define DECLARE_JACKAL_PROXY1(_j_name,type, t0) \
inline type _j_name( t0 p0 ) { \
	DECLARE_PREPARE_PROXY(_j_name);\
	req.parameters.clear();\
	req.parameters.push_back(p0);\
	DECLARE_PROXY_EXEC(type);\
}

#define DECLARE_JACKAL_PROXY2(_j_name,type, t0, t1) \
inline type _j_name( t0 p0, t1 p1 ) { \
	DECLARE_PREPARE_PROXY(_j_name);\
	req.parameters.clear();\
	req.parameters.push_back(p0);\
	req.parameters.push_back(p1);\
	DECLARE_PROXY_EXEC(type);\
}

#define DECLARE_JACKAL_PROXY3(_j_name,type, t0, t1, t2) \
inline type _j_name( t0 p0, t1 p1, t2 p2 ) { \
	DECLARE_PREPARE_PROXY(_j_name);\
	req.parameters.clear();\
	req.parameters.push_back(p0);\
	req.parameters.push_back(p1);\
	req.parameters.push_back(p2);\
	DECLARE_PROXY_EXEC(type);\
}

#define DECLARE_JACKAL_PROXY4(_j_name,type, t0, t1, t2, t3) \
inline type _j_name( t0 p0, t1 p1, t2 p2, t3 p3 ) { \
	DECLARE_PREPARE_PROXY(_j_name);\
	req.parameters.clear();\
	req.parameters.push_back(p0);\
	req.parameters.push_back(p1);\
	req.parameters.push_back(p2);\
	req.parameters.push_back(p3);\
	DECLARE_PROXY_EXEC(type);\
}

#define DECLARE_JACKAL_PROXY5(_j_name,type, t0, t1, t2, t3, t4) \
inline type _j_name( t0 p0, t1 p1, t2 p2, t3 p3, t4 p4 ) { \
	DECLARE_PREPARE_PROXY(_j_name);\
	req.parameters.clear();\
	req.parameters.push_back(p0);\
	req.parameters.push_back(p1);\
	req.parameters.push_back(p2);\
	req.parameters.push_back(p3);\
	req.parameters.push_back(p4);\
	DECLARE_PROXY_EXEC(type);\
}

#define DECLARE_JACKAL_PROXY6(_j_name,type, t0, t1, t2, t3, t4, t5) \
inline type _j_name( t0 p0, t1 p1, t2 p2, t3 p3, t4 p4, t5 p5 ) { \
	DECLARE_PREPARE_PROXY(_j_name);\
	req.parameters.clear();\
	req.parameters.push_back(p0);\
	req.parameters.push_back(p1);\
	req.parameters.push_back(p2);\
	req.parameters.push_back(p3);\
	req.parameters.push_back(p4);\
	req.parameters.push_back(p5);\
	DECLARE_PROXY_EXEC(type);\
}

#define DECLARE_JACKAL_PROXY7(_j_name,type, t0, t1, t2, t3, t4, t5, t6) \
inline type _j_name( t0 p0, t1 p1, t2 p2, t3 p3, t4 p4, t5 p5, t6 p6 ) { \
	DECLARE_PREPARE_PROXY(_j_name);\
	req.parameters.clear();\
	req.parameters.push_back(p0);\
	req.parameters.push_back(p1);\
	req.parameters.push_back(p2);\
	req.parameters.push_back(p3);\
	req.parameters.push_back(p4);\
	req.parameters.push_back(p5);\
	req.parameters.push_back(p6);\
	DECLARE_PROXY_EXEC(type);\
}

#define DECLARE_JACKAL_PROXY8(_j_name,type, t0, t1, t2, t3, t4, t5, t6, t7) \
inline type _j_name( t0 p0, t1 p1, t2 p2, t3 p3, t4 p4, t5 p5, t6 p6, t7 p7 ) { \
  DECLARE_PREPARE_PROXY(_j_name);\
  req.parameters.clear();\
  req.parameters.push_back(p0);\
  req.parameters.push_back(p1);\
  req.parameters.push_back(p2);\
  req.parameters.push_back(p3);\
  req.parameters.push_back(p4);\
  req.parameters.push_back(p5);\
  req.parameters.push_back(p6);\
  req.parameters.push_back(p7);\
  DECLARE_PROXY_EXEC(type);\
}

#define jackal_assert(b) if(b) WOOL_DEBUG("Wrong parameter count");

#define JACKAL_BEGIN_MAPPING inline virtual void handle(const jackal::Request & req, jackal::Response & res ) { \
	res.parameters.clear();\
	jackal::value & return_value = (*res.parameters.insert(res.parameters.end(), jackal::value::EmptyString));\

#define JACKAL_FORWARD(symbol,function) \
	if((req.method==symbol)) {\
		function(req,res);\
		return;\
	}\

#define JACKAL_MAP0(symbol,function) \
if((req.method==symbol)) {\
	return_value =  function();\
	return;\
}

#define JACKAL_MAP1(symbol,function) \
if((req.method==symbol)) {\
	jackal_assert(req.parameters.size()>=1);\
	return_value =  function(req.parameters[0]);\
	return;\
}

#define JACKAL_MAP2(symbol,function) \
if((req.method==symbol)) {\
	jackal_assert(req.parameters.size()>=2);\
	return_value =  function(req.parameters[0],req.parameters[1]);\
	return;\
}

#define JACKAL_MAP3(symbol,function) \
if((req.method==symbol)) {\
	jackal_assert(req.parameters.size()>=3);\
	return_value =  function(req.parameters[0],req.parameters[1],req.parameters[2]);\
	return;\
}

#define JACKAL_MAP4(symbol,function) \
if((req.method==symbol)) {\
	jackal_assert(req.parameters.size()>=4);\
	return_value =  function(req.parameters[0],req.parameters[1],req.parameters[2],req.parameters[3]);\
	return;\
}

#define JACKAL_MAP5(symbol,function) \
if((req.method==symbol)) {\
	jackal_assert(req.parameters.size()>=5);\
	return_value =  function(req.parameters[0],req.parameters[1],req.parameters[2],req.parameters[3],req.parameters[4]);\
	return;\
}

#define JACKAL_MAP6(symbol,function) \
if((req.method==symbol)) {\
	jackal_assert(req.parameters.size()>=6);\
	return_value =  function(req.parameters[0],req.parameters[1],req.parameters[2],req.parameters[3],req.parameters[4],req.parameters[5]);\
	return;\
}

#define JACKAL_MAP7(symbol,function) \
if((req.method==symbol)) {\
  jackal_assert(req.parameters.size()>=7);\
  return_value =  function(req.parameters[0],req.parameters[1],req.parameters[2],req.parameters[3],req.parameters[4],req.parameters[5],req.parameters[6]);\
  return;\
}

#define JACKAL_MAP8(symbol,function) \
if((req.method==symbol)) {\
  jackal_assert(req.parameters.size()>=8);\
  return_value =  function(req.parameters[0],req.parameters[1],req.parameters[2],req.parameters[3],req.parameters[4],req.parameters[5],req.parameters[6]),req.parameters[7]);\
  return;\
}

#define JACKAL_END_MAPPING throw jackal::exception(0,"unknown symbol"); }


#endif
