/***************************************************************************
 *   Copyright (C) 2005 by Vukasin Toroman                                 *
 *   vtoroman@astaro.de                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef JACKALXMLRPC_H
#define JACKALXMLRPC_H

#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <sstream>
#include <expat.h>

#include "../rfc2822/message_rfc2822.h"
#include "../rfc2822/http.h"
#include "../rfc3986/url.h"
#include "../jackal/jackal.h"
#include "../jackal/http.h"
#include "../socket/inet.h"
#include "../swool/streambuf_bio.h"

#include <boost/bind.hpp>



using namespace std;
namespace jackal {
namespace xmlrpc {


typedef http::RequestHandler HttpRequestHandler;
typedef jackal::RequestHandler MethodRequestHandler;

    	class Saver {
    		public:
    			void save(const jackal::Request & r, std::ostream & out ) const;
    			void save(const jackal::Response & r, std::ostream & out ) const;
    			void save(const jackal::exception & r, std::ostream & out ) const;
    	};
	
		class Loader {
			public:
				typedef enum{ 	Struct=0, 
									Array=1, 
									Value=2, 
									Name=3, 
									Parameter=4, 
									Member=5, 
									Number=6, 
									String=7, 
									Base64=8,
									MethodName=9,
									Fault=10 } Stage;

				Loader( jackal::Request & r );
				Loader( jackal::Response & r );
				virtual ~Loader();
				
				void load (std::istream & i);
				
				static void startElement(void *userData, const char *name, const char **atts);
				static void charHandler(void *userData, const XML_Char *s, int len);
				static void endElement(void *userData, const char *name);
				
			protected:
				void start_element(const std::string &name );
				void end_element (const std::string &name);
				void text (const std::string &contents);
				
			private:
				XML_Parser parser;
				std::string * method_name;
				jackal::value::array_type & parameters;
				std::list<jackal::value*> current_value;
				std::list<Stage> stage;
				std::string current_name;
				jackal::value fault_val;
				std::string value_text;
				bool implicit_string;
		};
		



class Adapter : public HttpRequestHandler, public MethodRequestHandler  {
	public:
		Adapter( MethodRequestHandler & h );
		virtual ~Adapter();
		
		virtual bool shouldHandle( const http::Request & req ) const;
		virtual void handle ( const jackal::http::Request & req, jackal::http::Response & res );
		virtual void handle ( const jackal::Request & req, jackal::Response & res );
		
	private:
		MethodRequestHandler & m_handler;
};





class Proxy : public MethodRequestHandler  {
	public:
		Proxy( const std::string & url );
		Proxy( const std::string & unix_path, const std::string & object_path );
		
		virtual ~Proxy();		
		virtual void handle ( const jackal::Request & req, jackal::Response & res );
		
	private:
		void connect();
		void do_io(jackal::http::Request & http_req, jackal::http::Response & http_res );
		
		wool::socket * m_socket;
		wool::iostream<wool::socket> * m_stream;
		wool::url m_url;
		std::string m_path;
		wool::address m_address;
};

};

};

#endif
