/**************************************************************************
*   Copyright (C) 2006 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef WOOL_LOGEVENT_H
#define WOOL_LOGEVENT_H

#include <map>
#include <sstream>
#include <unistd.h>
#include <boost/thread.hpp>
#include <algorithm>
#include <xmlwrapp/event_parser.h>

#include "../base/exception.h"
#include "../base/cvs_info.h"
#include "../base/debug.h"
#include "../log/log_perf.h"

namespace wool
{
	class event_error
	:public wool::exception
	{
		public:
    /**
		 * @param s message
		 * @throw exception wool::exception
	 */
			inline event_error(wool::exception::string_type s) throw ():wool::exception(s)
			{}
			inline ~event_error() throw ()
			{}
	};
	

	class Entry
	{
		public:
    /**
     * cvs info
	 */
		VERSION_INFO_INLINE("$Id: event.h,v 1.4 2007/01/11 15:18:27 vtoroman Exp $");
    /**
	 * cvs info
	 */
		VERSION_INFO_DECL;		
	
		typedef std::pair<std::string, std::string> value_pair;
		
		typedef std::list<value_pair> ParamList;

		/**
		 * first  -> level
		 * second -> text
		 */
		
		struct event_type {
			std::string name;
			std::string level;
		};

		typedef std::map <int , event_type> event_map_type;
		struct subsystem_type {
			std::string name;
			event_map_type events;
		} ;
		typedef std::map <int , subsystem_type>  subsystem_map_type;
		struct system_type {
			std::string name;
			subsystem_map_type subsystems;
		} ;
		
		
		



		static std::map <int, system_type> systems;

		static const std::string DEBUG;
		static const std::string INFO;
		static const std::string WARN;
		static const std::string ERROR;
		static const std::string CRIT;

		static const unsigned int DEBUG_LEVEL;
		static const unsigned int INFO_LEVEL;
		static const unsigned int WARN_LEVEL;
		static const unsigned int ERROR_LEVEL;
		static const unsigned int CRIT_LEVEL;

		static bool s_initialized;

		static std::map <std::string, unsigned int >  s_logLevel;

		static void initialize(const std::string & path);
		
		static long long parseID(std::string id);
		static std::string createID(long long id);

		Entry(	const unsigned long long & event_id );
		
		virtual ~Entry();
		
		ParamList & get_params() ;
		
		Entry & add( const std::string & key, const std::string & value );

		template <typename ValueType>
		Entry & add( const std::string & key, const ValueType & value )
		{
			std::ostringstream sv;
			sv << value;
			return add(key, sv.str());

		}
	
		template <typename KeyType, typename ValueType>
		Entry & add( const KeyType & key, const ValueType & value )
		{
			std::ostringstream sk;
			sk << key;

			std::ostringstream sv;
			sv << value;

			return add(sk.str(), sv.str());
		}
	
				
		void commit();
			
		bool is_committed () const;
				
		//! Copy Constructor
		Entry(const Entry & e);
	
				
		private:

			boost::mutex m_mut;
			ParamList m_params;
			bool m_committed;
			unsigned int m_level;
			static unsigned int m_mask;
			
	};
	
	class DocumentParser : public xml::event_parser
	{
		public:
		DocumentParser(std::map <int, Entry::system_type> & systems);
		
		protected:
			virtual bool start_element(const std::string &name, const attrs_type &attrs);
			virtual bool end_element(const std::string &name) ;
			virtual inline bool text(const std::string &name) { return true; };
			
		private:
			int num(const std::string & s);
			std::map <int, Entry::system_type> & m_map;
			Entry::system_type * current_system;
			Entry::subsystem_type * current_subsystem;
	};
};

#endif

