/**************************************************************************
*   Copyright (C) Astaro AG 2004                                          *
*   Author Vukasin Toroman <vtoroman@astaro.de>                           *
*                                                                         *
***************************************************************************/
/***************************************************************************
       Astaro Enterprise Toolkit License

       (1)  The copyright and all related intellectual property rights
       in the Astaro Enterprise Toolkit remain with Astaro.

       (2)  Astaro  grants  individual Users for home based, personal,
       non-commercial purposes  a   non-exclusive  license  to use the
       Astaro Enterprise Toolkit for an unlimited period of time.

       (3) Astaro grants commercial Users and all other Users not fal-
       ling under section 2 (2) one time  a  non-exclusive  license to
       use the Astaro Enterprise Toolkit for commercial  purposes  for
       an evaluation period of four weeks after the first installation
       of the Astaro Enterprise Toolkit (Evaluation License).

       (4)  The  User  is  not  entitled to   decompile,  disassemble,
       translate,  merge  or  otherwise  modify  the  software  unless
       permitted  by  law. No license is granted with  regard  to  the
       source code of the Astaro Enterprise Toolkit.

       (5) The User is not entitled to lease or sub-license the Astaro
       Enterprise Toolkit.

       (6) New versions of the Astaro Security Linux can be downloaded
       by  all  Users  at www.astaro.com. Astaro may  cease  the  free
       distribution  of the Astaro Enterprise Toolkit.  The  automatic
       update  distribution is available only within the framework  of
       the Professional License. Users under the Non-Commercial or the
       Evaluation License are not entitled  to  the  automatic Up2date
       distribution  with regard to their copy of the Astaro  Security
       Linux.
***************************************************************************/

#include "log/log.h"
#include <syslog.h>

namespace wool
{

  VERSION_INFO_IMPL(logger, "$Id: log.cpp,v 1.4 2007/01/11 15:18:27 vtoroman Exp $");

  const logger::mask_type logger::null_bit          = 0x0000;
  const logger::mask_type logger::critical_bit      = 0x0001;
  const logger::mask_type logger::fatal_bit         = 0x0002;
  const logger::mask_type logger::emergency_bit     = 0x0004;
  const logger::mask_type logger::error_bit         = 0x0008;
  const logger::mask_type logger::warning_bit       = 0x0010;
  const logger::mask_type logger::notice_bit        = 0x0020;
  const logger::mask_type logger::info_bit          = 0x0040;
  const logger::mask_type logger::debug_small_bit   = 0x0080;
  const logger::mask_type logger::debug_bit         = 0x0100;
  const logger::mask_type logger::debug_full_bit    = 0xffff;

  logger::mask_type logger::m_mask = logger::critical_bit|logger::fatal_bit|logger::emergency_bit|logger::error_bit|logger::warning_bit;

  void logger::log (const logger::string_type & content, logger::mask_type level,std::string identity )
  {
    static char name_buffer [1024];
    static boost::mutex name_mutex;

    boost::mutex::scoped_lock name_lock(name_mutex);

    if((level&m_mask)>0)
    {
      closelog();
      if(identity!="")
      {
        strncpy(name_buffer, identity.c_str(), 1024);
        openlog(name_buffer, LOG_CONS, 0);
      }
      else
      {
        openlog(NULL, LOG_CONS, 0);
      }
      syslog (LOG_INFO|LOG_DAEMON,"%s",content.c_str());
    }
  }


  bool logger::isEnabled(logger::mask_type severity)
  {
      static boost::mutex mask_mutex;
      boost::mutex::scoped_lock mask_lock(mask_mutex);
      return (severity & m_mask);
  }

};
