/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef WOOL_LOG_H
#define WOOL_LOG_H


#include <unistd.h>

#include <sstream>

#include <boost/thread.hpp>

#include "../base/cvs_info.h"
#include "../base/debug.h"

#include "../log/log.h"

#define WOOL_LOG_ID(message, severity, identity) \
    if (wool::logger::isEnabled(severity)) { \
        std::stringstream tmp; \
        tmp << message; \
        wool::logger::log(tmp.str(), severity, identity); \
    }

#define WOOL_LOG(message, severity) \
    if (wool::logger::isEnabled(severity)) { \
        std::stringstream tmp; \
        tmp << message; \
        wool::logger::log(tmp.str(), severity); \
    }

namespace wool
{
  /** Logging interface. The logs are made using the syslog interface.
    * @author Vukasin Toroman <vtoroman@astaro.de> (C) Astaro AG 2005
    */
  class logger
  {
  public:
    /**
     * Loglevel
     */
    typedef unsigned int mask_type;
    /**
     * String
     */
    typedef std::string string_type;

    /**
     * cvs info
     */
    VERSION_INFO_INLINE("$Id: log.h,v 1.4 2007/01/11 15:18:28 vtoroman Exp $");
    /**
     * cvs info
     */
    VERSION_INFO_DECL;

    /**
     *  nothing to log
     */
    static const mask_type null_bit;
    /**
     *  log only critical messages
     */
    static const mask_type critical_bit;
    /**
     * log fatal errors
     */
    static const mask_type fatal_bit;
    /**
     * log emergency messages
     */
    static const mask_type emergency_bit;
    /**
     * log all errors
     */
    static const mask_type error_bit;
    /**
     * log warnings
     */
    static const mask_type warning_bit;
    /**
     * log notice messages
     */
    static const mask_type notice_bit;
    /**
     * log all info's
     */
    static const mask_type info_bit;
    /**
     *  log important debug messages
     */
    static const mask_type debug_small_bit;
    /**
     * log the mostly debug informations
     */
    static const mask_type debug_bit;
    /**
     * log everything
     */
    static const mask_type debug_full_bit;

    /**
     *  Enables the logging
     *  @param m Loglevel
     */
    static void enable( logger::mask_type m ) {     m_mask|=m;      }
    /**
     *  Disable the logging
     *  @param m Loglevel
     */
    static void disable( logger::mask_type m ) {     m_mask&=~m;      }
    /**
     *  Set Loglevel
     *  @param m Loglevel
     */
    static void set_mask( logger::mask_type m ) {     m_mask=m;      }
    /**
     *  Get the Loglevel
     *  @return Loglevel
     */
    static logger::mask_type get_mask( ) {     return m_mask;      }

    /*
     *  Set the name for the log entries
     *  @param s name

    static void logger::set_identity( string_type s ) {     m_identity = s;      }

     *  Get the name for the log entries
     *  @return name for the log entries

    static logger::string_type logger::get_identity( ) {     return m_identity;      }
   */

    /*
     * check whether log() would log a message with the given severity/loglevel
     * bit.
     * @param severity
     * */
    static bool isEnabled(mask_type severity);

    /**
     *  Writes the messages into the log.
     *  @param content messages
     *  @param level loglevel
     */
    static void log ( const string_type & content, mask_type level = logger::debug_full_bit, std::string identity="");

  private:
    static mask_type m_mask;
  };
};
#endif
