/***************************************************************************
 *   Copyright (C) 2005 by Vukasin Toroman                                 *
 *   vtoroman@vtoroman05                                                   *
 *                                                                         *
 *  For more licensing information consult COPYING                         *
 ***************************************************************************/

#include <stdarg.h>
#include "log/log_perf.h"

#define LOG_ENTRY_SIZE 16384

namespace wool {
char * Log::s_syslogIdentity = NULL;
std::bitset<5> Log::s_levelMask;
static Log::Queue * s_logQueuePtr = new Log::Queue();
Log::Queue & Log::s_logQueue = *s_logQueuePtr;

void Log::setLogIdentity(const char * i) {
  size_t t = std::strlen(i);
  delete s_syslogIdentity;
  s_syslogIdentity = NULL;
  s_syslogIdentity = new char[t+1];
  std::strncpy(s_syslogIdentity, i, t+1);
  //closelog();
  openlog(s_syslogIdentity,LOG_PID,LOG_DAEMON);
}

const char * Log::getLogIdentity( ) {
  return s_syslogIdentity;
}

void Log::setLogMask(const std::bitset<5> & l) {
  s_levelMask = l;
  WOOL_DEBUG("set loglevel: " << s_levelMask);
}

const std::bitset<5> & Log::getLogMask( ) {
  return s_levelMask;
}

Log::Queue::Queue() {
  openlog(s_syslogIdentity,LOG_PID,LOG_DAEMON);
  setMinWorkerCount(1);
  setMaxWorkerCount(10);
  setMaxQueueSize(100);
}

  void Log::log( int facility, int level, const std::string & msg ) {
    WOOL_DEBUG("actual loglevel: " <<level << ":" << !getLogMask().test(level));
    if(!getLogMask().test(level))
      return;

    Queue::UnitPtr res(new LogUnit());
    res->message = msg;
    res->facility = facility;
    res->level = level;
    s_logQueue.add(res);
    WOOL_DEBUG("added message to queue");
  }

void Log::Queue::handle(UnitPtr u) {
  WOOL_DEBUG("log: " << u->message.c_str());
  syslog(u->facility|u->level, "%s", u->message.c_str());
}

void Log::Queue::onError(const std::exception & e, UnitPtr u ) {
  std::cerr << u->message << std::endl;
}


Log::Queue::~Queue(){
}


}
 
