#ifndef HTTP_H_
#define HTTP_H_
#include "../base/debug.h"
#include "../rfc2822/message_rfc2822.h"
#include "../rfc2822/http_header.h"
#include "../rfc2822/http_body.h"
#include <sstream>
namespace wool {
  namespace http {
	struct request {
		std::string method;
		std::string path;
		std::string protocol;
    		wool::rfc2822::header header;
    		wool::swappable_data body;

		inline std::ostream & save ( std::ostream &  out ) const {
			return out << method << ' ' << path << ' ' << protocol << '\r' << '\n' 
			    << header << '\r' << '\n' 
			    << body;
		}

		inline std::istream & load ( std::istream &  in ) {
			wool::swappable_data::ostream ob;
			ob.open(body);
			std::string first_line;
                        WOOL_DEBUG("read init line");
			std::getline(in,first_line);
			WOOL_DEBUG("init line '"<<first_line<<"'");
			first_line.push_back('\n');
			std::istringstream lin(first_line);
			lin.exceptions(std::ios::failbit|std::ios::eofbit);
			lin >> method >>  path;
			std::getline(lin, protocol);
			WOOL_DEBUG("got initial line "<<method<<", " << path<<" protocol");
                        WOOL_DEBUG("read header");
                        WOOL_DEBUG("next char: '"<<static_cast<char>(in.rdbuf()->sgetc())<<"'");
			in >> header;
			WOOL_DEBUG("got header: "<<header);
			header.erase("Accept");
			header.erase("Range");
			long required_size = 0;
			WOOL_DEBUG("header");
			rfc2822::header::iterator it = header.find("Content-length");
			if(it!=header.end()) {
				required_size = std::atol(it->second.c_str());
				if(required_size<0) {
					header.erase(it);
					required_size = 0;
				}
			}
			
			WOOL_DEBUG("body size "<<required_size);
			std::streambuf * pbuf = in.rdbuf();
			while ((required_size>0)&&(pbuf->sgetc()!=EOF)){
				ob << static_cast<char>(pbuf->sbumpc());
				required_size --;
			}
			ob.close();
			return in;
		}

	};

	struct response {
		unsigned int status_code;
		std::string message;
		std::string protocol;
		wool::rfc2822::header header;
		wool::swappable_data body;

		inline std::ostream & save ( std::ostream &  out ) const {
			return out << protocol << ' ' << status_code << ' ' << message << '\r' << '\n' 
			    << header << '\r' << '\n' 
			    << body;
		}

		inline std::istream & load ( std::istream &  in ) {
			wool::swappable_data::ostream ob;
			ob.open(body);
			std::string first_line;
                        std::getline(in,first_line);
                        std::istringstream lin(first_line);
	
			lin >> protocol >>  status_code;
			std::getline(lin, message);
			in >> header;
			header.erase("Accept");
			header.erase("Range");
			long required_size = -1;

			rfc2822::header::iterator it = header.find("Content-length");
			if(it!=header.end()) {
				required_size = std::atol(it->second.c_str());
				if(required_size<0) {
					header.erase(it);
				}
			}
			if(required_size<0) {
				ob << in.rdbuf();
			} else  {
				std::streambuf * pbuf = in.rdbuf();
				while ((required_size>0)&&(pbuf->sgetc()!=EOF)){
					ob << static_cast<char>(pbuf->sbumpc());
					required_size --;
				}
			} 
			ob.close();
			return in;
		}

	};

  };
};

namespace std
{
  inline std::ostream & operator << ( std::ostream & dst, const wool::http::request & h ){ return h.save(dst); }
  inline std::istream & operator >> ( std::istream & src, wool::http::request & h ) { return h.load(src); }
	
  inline std::ostream & operator << ( std::ostream & dst, const wool::http::response & h ){ return h.save(dst); }
  inline std::istream & operator >> ( std::istream & src, wool::http::response & h ) { return h.load(src); }

};

#endif /*HTTP_BODY_H_*/
