/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/**************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef RFC2822MESSAGE_H
#define RFC2822MESSAGE_H

#include "../base/unsorted_multimap.h"
#include "../storage/swappable_data.h"
#include "../base/debug.h"
#include "../base/exception.h"

//#include "../../wool/message.h"
#include "../rfc2822/message_body.h"
#include "../rfc2822/header_rfc2822.h"

namespace wool
{
  /**
   *  This namespace is used to read and write messages which are conform to the RFC 2822.
   *  @author Vukasin Toroman <vtoroman@astaro.de> (C) Astaro AG 2005
   *  @see http://www.faqs.org/rfcs/rfc2822.html
   */
  namespace rfc2822
  {
    typedef std::string string;
    /**
     * case-insensitive string comparisons
     * @author Vukasin Toroman <vtoroman@astaro.de> (C) Astaro AG 2005
     */
    struct ncase_eqstr
    {
      /**
       *  @param s1 reference to the first string to compare to
       *  @param s2 reference to the second string to compare to
       *  @return true if s1 is equal to s2 ignoring the case else false
       */
      inline bool operator()(const std::string & s1, const std::string &  s2) const throw()
      {
        return (strcasecmp(s1.c_str(), s2.c_str()) < 0); 
      }
    };

    typedef wool::unsorted_multimap<string,string,ncase_eqstr> header;
    typedef wool::rfc2822::header_loader_template<header> header_loader;

/*
    typedef std::string path_type;
    typedef std::string method_type;
    typedef std::string version_type;

    		method_type & method;
		path_type & path;
		version_type & version;

		:method(__method)
		,path(__path)
		,version(__version)
*/
    	template < typename bodyT,
		   typename bodyTraitsT,
		   typename messageTraitsT>
	struct message_loader {
		wool::rfc2822::header & header;
		bodyT & body;
		

		
		wool::rfc2822::header_loader header_loader;
		typename wool::rfc2822::body_loader<bodyT, bodyTraitsT> body_loader;
		messageTraitsT traits;

		message_loader( wool::rfc2822::header & __header,
				bodyT & __body )
		:header(__header)
		,body(__body)
		,traits(*this)
		{
		}
		
		template <typename iteratorT>
		iteratorT parse (iteratorT begin, iteratorT end) {
			if(header_loader.current_state!=header_loader::done) {
				begin = header_loader.parse(begin,end);
				if(header_loader.current_state!=header_loader::done) {
					traits.on_header_done();
				}
			}
			
			begin = body_loader.parse(begin,end);
			return begin;
		}
	};
  }


};


namespace std
{
  struct header_writer {
    std::ostream & d;
    inline void operator() (const wool::rfc2822::header::value_type & v ) {
        if((!v.first.empty()) || (!v.second.empty())) {
            d << v.first << ':' << ' ' << v.second << '\r' << '\n';
        }
    }
  };
  
  inline std::ostream & operator << ( std::ostream & dst, const wool::rfc2822::header & h ) {
  	header_writer hw = {dst};
    std::for_each(h.begin(), h.end(), hw);
    return dst;
  }
  
  inline std::istream & operator >> ( std::istream & src, wool::rfc2822::header & h ) {
  	std::istreambuf_iterator<char> it_begin(src.rdbuf());
  	std::istreambuf_iterator<char> it_end;
  	
  	wool::rfc2822::header_loader loader(h);
  	loader.parse(it_begin, it_end);
  	if(loader.current_state!=wool::rfc2822::header_loader::done)
  		src.setstate(src.rdstate()|std::ios::failbit);
  	return src;
  }

};

#endif
