/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef WOOL_USERNAME_H
#define WOOL_USERNAME_H

#include "../base/wool_stl_includes.h"
#include "../base/debug.h"
#include "../base/cvs_info.h"


namespace wool
{
  /**
   * This class holds the information about an user in form "name@domain", e.g. "nobody@somewhere.moon".
   * @author Vukasin Toroman <vtoroman@astaro.com> (C) Astaro AG 2005
   */
  class user
  {
    public:

      /**
       * user name
       */
      std::string name;

      /**
       * user domain
       */
      std::string domain;

      /**
       * cvs info
       */
      VERSION_INFO_INLINE("$Id: username.h,v 1.4 2007/01/11 15:18:36 vtoroman Exp $");

      /**
       * @param __name user name
       * @param __domain user domain
       */
      inline user(std::string __name,std::string __domain)
      :name(__name),domain(__domain)
      {}

      /**
       * Separates the name and the domain of the user by locating "@". If no "@" was found, the user domain is empty.
       * @param username_domain user name as string in form ("nobody@somewhere.moon")
       */
      inline user(std::string username_domain)
      {
        unsigned _loc = username_domain.find('<');
        if(_loc!=std::string::npos)
          username_domain.erase(0,_loc+1);
        _loc = username_domain.find('>');
        if(_loc!=std::string::npos)
          username_domain.erase(_loc,username_domain.length());
        unsigned loc = username_domain.find("@");
        if(loc<username_domain.length())
        {
          name = std::string(username_domain,0,loc);
          domain = std::string(username_domain,loc+1,username_domain.length()-loc-1);
        }
        else
        {
          name = username_domain;
          domain = "";
        }
      }

      inline ~user()
      {  }

      /**
       * String operator
       * @return user as string in form "name@domain"
       */
      inline operator std::string () const
      {  return name+"@"+domain;  }

      /**
       * Equal operator
       * @param o user to check if is equal
       * @return true if name and domain are equal else false
       */
      inline bool operator == (const user & o) const
      {  return (name==o.name) && (domain==o.domain);  }

      /**
       * Lower operator
       * @param o user to check if is lower
       * return true if domain are equal and name are lower or if domain are lower else false
       */
      inline bool operator < (const user & o) const
      {
        if(domain==o.domain)
          return (name<o.name);
        return (domain<o.domain);
      }

      /**
       * @param o user to check if it containes in
       * @return 1 if user containes in else 0
       */
      inline int count (const user & o) const
      {

        if(((domain=="*")||(domain==o.domain))&&((name==o.name)||(name=="*")))
        {
          WOOL_DEBUG("'"<<name+"@"+domain<<"' included '"<<o.name+"@"+o.domain<<"'");
          return 1;
        }
        WOOL_DEBUG("'"<<name+"@"+domain<<"' not included '"<<o.name+"@"+o.domain<<"'");
        return 0;
      }
  };

  //typedef container_set<weed::user> user_set;

};

#endif

