/**************************************************************************
*  Copyright (C) 2006 by Jens Finkhaeuser <unwesen@users.sourceforge.net> *
**************************************************************************/
/**************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
**************************************************************************/

#ifndef WOOL_URI_DETAIL_GENLOC_H
#define WOOL_URI_DETAIL_GENLOC_H

// own includes
#include "rfc3986/detail/uri_implbase.h"

namespace wool {
namespace detail {


// ===========================================================================
// === class GenericLocationURIImpl
/**
 * GenericLocationURIImpl defines functionality common to several URL-like
 * URI schemes. Most of these schemes will actually denote URLs, but as some
 * might not, the class is not named URLImpl or something similar.
 *
 * Specifically, isURL() returns true for this class, and the path component
 * is parsed into segments separated by '/'. An attempt is made to parse the
 * userinfo into seperate username and password fields.
 *
 * The Query is not parsed further.
 **/
class GenericLocationURIImpl
    : public wool::URI::ImplBase
{
public:
    // =======================================================================
    // === public ctor/dtor
    GenericLocationURIImpl();
    GenericLocationURIImpl(URI * uri, std::string const & uristring);
    virtual ~GenericLocationURIImpl();

    // =======================================================================
    // === public methods
    virtual boost::shared_ptr<wool::URI::ImplBase>
    clone(URI * uri, std::string const & uristring) const;

    virtual bool isURL() const;
    virtual bool isURN() const;

    virtual std::vector<std::string> getPathParts() const;
    virtual std::string getUserName() const;
    virtual std::string getPassword() const;
    uint16_t getPort() const;


    virtual bool equals(boost::shared_ptr<ImplBase> other) const;

protected:
    std::vector<std::string>    m_path_parts;
    std::string                 m_username;
    std::string                 m_password;

private:
    void parseUserinfo();
    void parsePath();
    void addPathSegment(char const * begin, char const * end);
};

}} // namespaces

#endif // guard
