/**************************************************************************
*  Copyright (C) 2006 by Jens Finkhaeuser <unwesen@users.sourceforge.net> *
**************************************************************************/
/**************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
**************************************************************************/

#ifndef WOOL_URI_DETAIL_HTTPIMPL_H
#define WOOL_URI_DETAIL_HTTPIMPL_H

// own includes
#include "rfc3986/detail/uri_genlocimpl.h"

namespace wool {
namespace detail {


// ===========================================================================
// === class URI::ImplBase
/**
 * Implements parsing of HTTP and HTTPS URIs by adding parsing of the query
 * component to the parsing provided by the generic location implementation.
 *
 * For those schemes, a query parameter
 * - may or may not be associated with a value via the = character
 * = may occur more than once.
 **/
class HTTPURIImpl
    : public wool::detail::GenericLocationURIImpl
{
public:
    // =======================================================================
    // === public ctor/dtor
    HTTPURIImpl();
    HTTPURIImpl(URI * uri, std::string const & uristring);
    virtual ~HTTPURIImpl();

    // =======================================================================
    // === public methods
    virtual boost::shared_ptr<wool::URI::ImplBase>
    clone(URI * uri, std::string const & uristring) const;

    virtual uint16_t getPort() const;

    virtual std::multimap<std::string, std::string> getQueryParts() const;

    virtual bool equals(boost::shared_ptr<ImplBase> other) const;


protected:
    // =======================================================================
    // === protected methods
    /**
     * parses the query component, using addPart() and addPartValue() to store
     * the kv_pairs.
     **/
    void parseQuery();

    /**
     * Add a parameter to the part map
     **/
    void addPart(char const * begin, char const * end);

    /**
     * Set the value for the part last added with addPart(). Throws error if
     * addPart() has never been called.
     **/
    void addPartValue(char const * begin, char const * end);

    // =======================================================================
    // === protected members
    std::multimap<std::string, std::string>             m_query_parts;
    std::multimap<std::string, std::string>::iterator   m_last_part;
};

}} // namespaces

#endif // guard
