/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef WOOL_URL_H
#define WOOL_URL_H
#include "../base/debug.h"
#include "../base/cvs_info.h"
#include "../base/wool_stl_includes.h"
namespace wool
{
  /**
   * This struct represents a Uniform Resource Locator, a pointer to a "resource" on the World Wide Web. It contains information about the protocol, the host, the path and query.
   * @author Vukasin Toroman <vtoroman@astaro.com> (C) Astaro AG 2005
   */
  struct url
  {
    /**
     *  object type to hold information about the subdirectories
     */
    typedef std::vector<std::string> path_type;

    /**
     *  cvs info
     */
    VERSION_INFO_INLINE("$Id: url.h,v 1.4 2007/01/11 15:18:38 vtoroman Exp $");

    inline bool operator < (const url & u ) const {
      if(protocol != u.protocol)
        return protocol < u.protocol;
      if(host != u.host)
        return host < u.host;
      if(path != u.path)
        return path < u.path;
      return query < u.query;
    }

    inline std::string get_path() const {
        std::string p = "";
        if(path.empty())
      	  p = "/";
    	else {
      	  for(wool::url::path_type::const_iterator i=path.begin();i!=path.end();++i) {
	    if(*i!="") {
              p.push_back('/');
	      p.append(*i);
	    }
          }
	  if(p=="")
	    p = "/";
	}
        return p;
    }

    inline url get_level (path_type::size_type l ) const {
      url r(*this);
      r.path.resize(l);
      return r;
    }

    url()
    {
    }

	url(const wool::url & u) : 
		protocol(u.protocol), 
		host(u.host),
		query(u.query),
		path(u.path){
	}
	
    /**
     *  constructor
     * @param u url
     */
    url(std::string u)
    {
      int _limit;
      _limit = u.find('?');
      if(_limit>0) {
        query.assign(u.begin()+_limit+1,u.end());
        u.erase(u.begin()+_limit,u.end());
      }

      _limit = u.find("://");

      if(_limit<0)
      {
        protocol = "http";
        _limit = 0;
      } else {
        protocol.assign(u.begin(),u.begin()+_limit);
        if(protocol.find_first_not_of("abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLOPQRSTVUWXYZ")==std::string::npos) {
            u.erase(0,_limit+3);
        } else {
            protocol = "http";
            _limit = 0;
        }
      }

      _limit = u.find('/');
      if(_limit<0)
      {
        host.assign(u);
        path.clear();
        return;
      }

      host.assign(u.begin(),u.begin()+_limit);
      u.erase(0,_limit+1);

      while(!u.empty())
      {
        _limit = u.find('/');
        if(_limit<0)
        {
          path.push_back(u);
          return;
        }
        if(_limit>=0){
          path.push_back(std::string(u,0,_limit));
        }
        u.erase(0,_limit+1);
      }
      path.push_back("");
    }
    /**
     *  used protocol, e.g. http or https
     */
    std::string protocol;
    /**
     *  host, e.g. www.google.com
     */
    std::string host;
    /**
     *  query, e.g. ?hl=de&q=Astaro&btnG=Google-Suche&meta=lr%3Dlang_en
     */
    std::string query;
    /**
     *  path, e.g. /search
     */
    path_type path;
  };
};

namespace std
{
  inline std::ostream & operator << ( std::ostream & out, const wool::url & u)
  {
		if(u.protocol.length()>0)
    	out << u.protocol;
		if(u.host.length()>0)
			out << "://" << u.host;
    if(u.path.empty())
      out << '/';
    else
      for(wool::url::path_type::const_iterator i=u.path.begin();i!=u.path.end();++i)
        out<<'/'<<*i;
    if(!u.query.empty())
      out<<'?'<<u.query;
    return out;

  }
};
#endif

