#include "socket/inet.h"
#include "dns/dns.h"

namespace wool {
	namespace inet {
		const ip ip::any = INADDR_ANY;
		const ip ip::loopback  = INADDR_LOOPBACK;
		const ip ip::broadcast = INADDR_BROADCAST ;
		const ip ip::none = INADDR_NONE;
		
		ip::ip() {}
		
		ip::ip(const in_addr & i) {
			s_addr = i.s_addr;
		}
		
		ip::ip(const char * i) {
			s_addr = Dns::Resolve(i);	
		}

		ip::ip(const std::string & i) {
			s_addr = Dns::Resolve(i.c_str());
                }
		
		ip::ip(const in_addr_t & i) {
			s_addr = i;
		}
		
		
		const unsigned char & ip::operator [] (const int & i) const {
			return ((const unsigned char *)&s_addr)[i];
		}
		unsigned char & ip::operator [] (const int & i) {
			return ((unsigned char *)&s_addr)[i];
		}
		
		ip::operator std::string () const {
			char buffer[INET_ADDRSTRLEN];
			return inet_ntop(address::inet, this, buffer, INET_ADDRSTRLEN);
		}

		address::operator std::string () const {
			char port_s [8];
			std::string res = get_ip();
			res.push_back(':');
			snprintf(port_s, 8, "%i",get_port());
			res.append(port_s);
			return res;
		}
		
		ip & ip::operator = ( const char * ip_string) {
			s_addr = Dns::Resolve(ip_string);
			return *this;
		}
		ip & ip::operator = ( const std::string & ip_string) {		
			return ((*this)=ip_string.c_str());
		}
		
		ip & ip::operator = ( const ::in_addr & a ) {
			s_addr = a.s_addr;
			return *this;
		}
		
		ip & ip::operator = ( const ::in_addr_t & a ) {
			s_addr = a;
			return *this;
		}
		
		ip & ip::operator = (const unsigned char * a ) {
			s_addr = *((const in_addr_t *)a);
			return *this;
		}
		
		bool ip::operator < ( const ip & a2 ) const {
			return s_addr < a2.s_addr;
		}

		bool ip::operator > ( const ip & a2 ) const {
			return s_addr > a2.s_addr;
		}
		
		bool ip::operator == ( const ip & a2 ) const {
			return s_addr == a2.s_addr;
		}
		
		bool ip::operator != ( const ip & a2 ) const {
			return s_addr != a2.s_addr;
		}
		
		bool ip::operator <= ( const ip & a2 ) const {
			return s_addr <= a2.s_addr;
		}
		
		bool ip::operator >= ( const ip & a2 ) const {
			return s_addr >= a2.s_addr;
		}
		
		ip ip::operator & ( const ip & a2 ) const {
			ip res ;
			res = s_addr & a2.s_addr;
			return res;
		}
		
		ip ip::operator | ( const ip & a2 ) const {
			ip res ;
			res = s_addr | a2.s_addr;
			return res;
		}
	
		ip & ip::operator &= ( const ip & a2 ) {
			s_addr &= a2.s_addr;
			return *this;
		}
		
		ip & ip::operator |= ( const ip & a2 ) {
			s_addr |= a2.s_addr;
			return *this;
		}
		
		address::address(const ip & i, const int & p) : wool::address(address::inet) { 
			set_ip(i);
			set_port(p);
		}
	
		address::address() : wool::address(address::inet) { 
			set_ip(ip::none);
			set_port(0);
		}

	  address::address( const ::sockaddr & f ): wool::address(f) {
	    assert(get_family()==socket::inet);
	  }
		
		address::address( const std::string & a ) : wool::address(address::inet) { 
			*this = a;
		}

		address::address( const char * a ) : wool::address(address::inet) { 
			*this = a;
		}


		const ip & address::get_ip() const {
			const sockaddr_in * addr = (const sockaddr_in *)this;
			const ip * ip_addr = (const ip *)(&addr->sin_addr);
			return *ip_addr;
		}
		
		int address::get_port() const {
			const sockaddr_in * addr = (const sockaddr_in *)this;
			const in_port_t * port_addr = (const in_port_t *)(&addr->sin_port);
			return ntohs(*port_addr);
		}
		
		address & address::set_ip( const ip & i ) {
			sockaddr_in * addr = (sockaddr_in *)this;
			ip * ip_addr = (ip *)(&addr->sin_addr);
			*ip_addr = i;
			return *this;
		}
		
		address & address::set_port( const int & i ) {
			sockaddr_in * addr = (sockaddr_in *)this;
			in_port_t * port_addr = (in_port_t *)(&addr->sin_port);
			*port_addr = htons(i);
			return *this;
		}
 
		address & address::operator = ( const char * __saddr ) {
			const char * e = std::find(__saddr, __saddr+4096, '\0');
			const char * s = std::find(__saddr,e,':');

			std::string str_ip (__saddr, s-__saddr);
			set_ip(ip(str_ip));
			set_port((e==s)?0:atoi(s+1));	
			return *this;
		}
		
		address & address::operator = ( const std::string & __saddr ) {
			return (*this)=__saddr.c_str();
		}

		address & address::operator = ( const sockaddr & s ) {
			::memmove(&sa_family, &s, sizeof(sockaddr));
			return *this;
		}
	
		bool address::operator == (const sockaddr & s ) const{
			if(s.sa_family!=get_family())
				return false;
			static const size_t cmp_size = sizeof(in_port_t) + sizeof(in_addr_t);
			return (memcmp(get_data(), s.sa_data, cmp_size) == 0);
		}

		bool address::operator < (const address & s ) const{
			return (get_ip()<s.get_ip())?true:get_ip()==s.get_ip()?false:(get_port()<s.get_port());
		}
	};
};
