#ifndef WOOL_INET_H_
#define WOOL_INET_H_

#include <error.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string>

#ifdef USE_NETFILTER
#include <linux/netfilter_ipv4.h>
#endif

#include <unistd.h>
#include "../socket/socket.h"
#include "../dns/dns.h"

namespace wool {
  namespace inet {
    struct ip :public ::in_addr {
      static const ip any;
      static const ip loopback;
      static const ip broadcast;
      static const ip none;
			
      ip();
      ip(const in_addr & i);
      ip(const in_addr_t & i);
      ip(const char * i);
      ip(const std::string & i);

			
      const unsigned char & operator [] (const int & i) const;
      unsigned char & operator [] (const int & i);
			
      operator std::string () const;
			
      ip & operator = ( const char * ip_string);
      ip & operator = ( const std::string & ip_string);
      ip & operator = ( const ::in_addr & a );
      ip & operator = ( const ::in_addr_t & a );
      ip & operator = (const unsigned char * a );
			
      bool operator < ( const ip & a2 ) const;
      bool operator > ( const ip & a2 ) const;
      bool operator == ( const ip & a2 ) const;
      bool operator != ( const ip & a2 ) const;
      bool operator <= ( const ip & a2 ) const;
      bool operator >= ( const ip & a2 ) const;
			
      ip operator & ( const ip & a2 ) const;
      ip operator | ( const ip & a2 ) const;
			
      ip & operator &= ( const ip & a2 );
      ip & operator |= ( const ip & a2 );
    };
		
    struct address : public wool::address {
      address();
      address( const sockaddr & s );
      address( const ip & i, const int & port );
      address( const char * i );
      address( const std::string & i );
			
      const ip & get_ip() const;
      int get_port() const;		
			
      address & set_ip( const ip & i );
      address & set_port( const int & i );
      operator std::string () const;
	 
      address & operator = ( const char * __saddr );
      address & operator = ( const std::string & __saddr );
      address & operator = ( const sockaddr & __saddr );
			
      bool operator == ( const sockaddr & __saddr ) const;
      bool operator != ( const sockaddr & __saddr );
      bool operator < ( const address & __saddr ) const;
      bool operator > ( const address & __saddr );
    };
		
    static const socket::type_record inet_stream = { socket::inet, socket::stream_socket, socket::unspec }; 
  };
};

#endif /*INET_H_*/
