//
// C++ Implementation: poll
//
// Description: 
//
//
// Author: Vukasin Toroman <vtoroman@astaro.de>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <sys/poll.h>
#include <sys/epoll.h>
#include <assert.h>
#include "../socket/socket.h"
#include "../socket/poll.h"
namespace wool
{


  poller::poller ( size_type s )
    :m_begin(NULL),m_end(NULL),m_capacity(s)
  {
    assert(s>0);
    m_begin = new value_type[s];
    m_end = m_begin;
    m_capacity = s;
    m_poll_fd = epoll_create(s);
    if(m_poll_fd < 0 )
      throw poll_exception();
  }

  poller::~poller ()
  {
    ::close(m_poll_fd);
    delete m_begin;
  }

  poller::size_type poller::poll( int __timeout )
  {
    int avail = capacity()-size();
    if(avail<=0)
      throw poll_exception(ENOMEM);
    int res = epoll_wait(m_poll_fd, (epoll_event*)end(),avail, __timeout);
    if(res<0)
      throw poll_exception();
    m_end +=res;
    return res;
  }

  poller::iterator poller::begin () { return this->m_begin; }
  poller::const_iterator poller::begin () const { return this->m_begin; }

  poller::iterator poller::end () { return this->m_end; }
  poller::const_iterator poller::end () const { return this->m_end; }

  poller::size_type poller::size () const { return std::distance(begin(), end()); }
  poller::size_type poller::capacity () const { return this->m_capacity; }
  void poller::clear () { m_end = m_begin; };

  poller::size_type poller::reserve ( size_type s )
  {
    s = std::max<size_type>(size(),std::max<size_type>(1,s));
    iterator __new_begin = new value_type[s];
    if(size()>0)
      ::memcpy(m_begin,__new_begin,sizeof(value_type)*size());
    m_end += (__new_begin-m_begin);
    delete m_begin;
    m_begin = __new_begin;
    m_capacity = s;
    return capacity();
  };

  void poller::start_polling(EPOLL_EVENTS poll_events,fd_type & socket)
  {
    epoll_event ev;
    ev.events = poll_events;
    ev.data.fd = socket;

    WOOL_DEBUG("insert a fd: "<<m_traits.get_fd(socket));
    WOOL_DEBUG("inserting into fd: "<<m_poll_fd);
    if (  (::epoll_ctl(m_poll_fd, EPOLL_CTL_ADD, ev.data.fd, &ev) < 0)
	  &&(::epoll_ctl(m_poll_fd, EPOLL_CTL_MOD, ev.data.fd, &ev) < 0) )
      {
        throw exception();
      }
  }

  void poller::update_pollmask(EPOLL_EVENTS poll_events, fd_type & socket )    {
    epoll_event ev;
    ev.events = poll_events;
    ev.data.fd = socket;

    if ( (::epoll_ctl(m_poll_fd, EPOLL_CTL_MOD, ev.data.fd, &ev) < 0)
	 &&(::epoll_ctl(m_poll_fd, EPOLL_CTL_ADD, ev.data.fd, &ev) < 0))
      {
        throw exception();
      }
  }

  void poller::stop_polling(fd_type & socket)  {
    epoll_event ev;
    ev.events = 0;
    ev.data.fd =socket;

    if (::epoll_ctl(m_poll_fd, EPOLL_CTL_DEL, ev.data.fd, &ev) < 0)   {
        WOOL_DEBUG("deletion errors are handled silently");
        //throw exception();
      }
  }
  
  struct finder  {
    int fd;
    finder(int f) : fd(f){}
    bool operator ()( const poller::value_type & v ) { return (v.data.fd == fd);}
  };
  
  poller::iterator poller::find( fd_type & fd )  {
    poller::iterator res = begin();
    for(; (res!=end())&&(res->data.fd!=fd) ;res++);
    return res;
  }

  poller::const_iterator poller::find( fd_type & fd ) const  {
    poller::const_iterator res = begin();
    for(; (res!=end())&&(res->data.fd!=fd) ;res++);
    return res;

  }


};
