/**************************************************************************
 *   Copyright (C) 2004 by Vukasin Toroman                                 *
 *   vukasin@toroman.org                                                   *
 **************************************************************************/
/**************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef WOOL_POLL_H
#define WOOL_POLL_H

#include <sys/epoll.h>
#include <assert.h>
#include "../socket/socket.h"
namespace wool
{

  template<typename socketT>
    class poll_traits    {
  public:
    typedef socketT socket_type;
    typedef int os_fd_type;
    inline os_fd_type get_fd ( socket_type & c ) {	return c.fd();	}
  };

  template<>
    class poll_traits<const int>    {
  public:
    typedef const int socket_type;
    typedef const int os_fd_type;
    inline os_fd_type get_fd ( socket_type & c ) {	return c;	}
  };

  template<>
    class poll_traits<int>  {
  public:
    typedef int socket_type;
    typedef int os_fd_type;
    inline os_fd_type get_fd ( socket_type & c ) {	return c;	}
  };

 class poll_exception : public wool::socket::exception  {
  public:
  poll_exception ( int __errno = errno ) throw () : wool::socket::exception (__errno) {
      m_error_message.insert(0,"poll error:");
    }
    virtual ~poll_exception() throw () {
    }
  };


  class poller  {
  public:
    typedef epoll_data poll_data_type;
    typedef epoll_event poll_event;
    typedef int fd_type;

    typedef poll_event* iterator;
    typedef const poll_event* const_iterator;

    typedef poll_event value_type;
    typedef size_t size_type;


    poller ( size_type s=1024 );
    ~poller ();
    size_type poll( int __timeout );
    
    iterator begin ();
    const_iterator begin () const;
    
    iterator end ();
    const_iterator end () const;
    
    size_type size () const;
    size_type capacity () const;
    void clear ();
    
    size_type reserve ( size_type s );
    void start_polling( EPOLL_EVENTS poll_events,fd_type & socket);
    void update_pollmask( EPOLL_EVENTS poll_events, fd_type & socket );
    void stop_polling(fd_type & socket);
    iterator find( fd_type & socket );
    const_iterator find( fd_type & socket ) const;
    
    template<typename socket_type >
      void start_polling( EPOLL_EVENTS poll_events,socket_type & socket)  {
      poll_traits<socket_type> traits;
      start_polling(poll_events, traits.get_fd(socket));
    }
    
    template<typename socket_type >
      void update_pollmask( EPOLL_EVENTS poll_events, socket_type & socket ) {
      poll_traits<socket_type> traits;
      update_pollmask(poll_events, traits.get_fd(socket));
    }

    template<typename socket_type>
      void stop_polling(socket_type & socket) {
      poll_traits<socket_type> traits;
      stop_polling(traits.get_fd(socket));
    }
    
    template<typename socket_type>
      iterator find( socket_type & socket ) {
      poll_traits<socket_type> traits;
      return find(traits.get_fd(socket));
    }

    template<typename socket_type>
      const_iterator find( socket_type & socket ) const {
      poll_traits<socket_type> traits;
      return find(traits.get_fd(socket));
    }
  private:
    int m_poll_fd;
    iterator m_begin;
    iterator m_end;
    size_type m_capacity;
  };
};

#endif
