/***************************************************************************
 *   Copyright (C) 2004 by Vukasin Toroman                                 *
 *   vtoroman@cyclops                                                      *
 *                                                                         *
 *   Astaro Enterprise Toolkit License                                     *
 *                                                                         *
 *   (1)  The copyright and all related intellectual property rights       *
 *   in the Astaro Enterprise Toolkit remain with Astaro.                  *
 *                                                                         *
 *   (2)  Astaro  grants  individual Users for home based, personal,       *
 *   non-commercial purposes  a   non-exclusive  license  to use the       *
 *   Astaro Enterprise Toolkit for an unlimited period of time.            *
 *                                                                         *
 *   (3) Astaro grants commercial Users and all other Users not fal-       *
 *   ling under section 2 (2) one time  a  non-exclusive  license to       *
 *   use the Astaro Enterprise Toolkit for commercial  purposes  for       *
 *   an evaluation period of four weeks after the first installation       *
 *   of the Astaro Enterprise Toolkit (Evaluation License).                *
 *                                                                         *
 *   (4)  The  User  is  not  entitled to   decompile,  disassemble,       *
 *   translate,  merge  or  otherwise  modify  the  software  unless       *
 *   permitted  by  law. No license is granted with  regard  to  the       *
 *   source code of the Astaro Enterprise Toolkit.                         *
 *                                                                         *
 *   (5) The User is not entitled to lease or sub-license the Astaro       *
 *   Enterprise Toolkit.                                                   *
 *                                                                         *
 *   (6) New versions of the Astaro Security Linux can be downloaded       *
 *   by  all  Users  at www.astaro.com. Astaro may  cease  the  free       *
 *   distribution  of the Astaro Enterprise Toolkit.  The  automatic       *
 *   update  distribution is available only within the framework  of       *
 *   the Professional License. Users under the Non-Commercial or the       *
 *   Evaluation License are not entitled  to  the  automatic Up2date       *
 *   distribution  with regard to their copy of the Astaro  Security       *
 *   Linux.                                                                *
 ***************************************************************************/

#include <base/wool_stl_includes.h>
#include <storage/swappable_data.h>
#include <storage/tmp_file.h>

#include <unistd.h>
#include <iterator>
#include <boost/thread.hpp>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>



namespace wool
{
      VERSION_INFO_IMPL(swappable_data, "$Id: swappable_data.cpp,v 1.4 2007/01/11 15:18:40 vtoroman Exp $");

      const swappable_data::size_type swappable_data::resident_size = 1024*1024;

          swappable_data::streambuf::streambuf():m_data(NULL)
          {}

          swappable_data::streambuf::~streambuf()
          {}

          bool swappable_data::streambuf::is_open ()
          { return m_data!=NULL; }

          void swappable_data::streambuf::open (const swappable_data & data, std::ios_base::openmode om)
          {
            om = om & (~std::ios_base::out);
            om = om & (~std::ios_base::trunc);

            open(const_cast<swappable_data &>(data),om);
          }

          void swappable_data::streambuf::open (swappable_data & data, std::ios_base::openmode om)
          {

            assert(((om&std::ios_base::in) == 0)||((om&std::ios_base::out) ==0));

            close();
            m_mode = om;
            m_data = &data;
            if(om&std::ios_base::trunc)
            {
              m_data->clear();
            }

            if(!m_data->m_fname.empty())
              {
                  WOOL_DEBUG("fstream opening "<<m_data->m_fname);
                  m_filebuf.open(m_data->m_fname.c_str(),om);
                  if(!m_filebuf.is_open())
                    throw std::runtime_error(std::string("could no open temp file ")+m_data->m_fname+": "+strerror(errno));
              }
              else
              {
                  m_offset = (om&std::ios_base::ate)?m_data->m_data.length():0;
              }
            om &=~std::ios_base::trunc;
            om &=~std::ios_base::ate;
          }

          void swappable_data::streambuf::close()
          {
            sync();
            m_filebuf.close();
            m_data = NULL;
            m_offset = 0l;
          }

          int swappable_data::streambuf::sync ()
          {
            if(m_filebuf.is_open())
              return m_filebuf.pubsync();
            return 0;
          }

          void swappable_data::swap( wool::swappable_data & v) {
                m_fname.swap(v.m_fname);
                m_data.swap(v.m_data);
          }

          void swappable_data::streambuf::check_size( std::streamsize needed_size )
          {
            if((!m_filebuf.is_open())&&
                (m_data->size()+needed_size>swappable_data::resident_size))
              {
                WOOL_DEBUG("SIZE BEFORE SWAPOUT "<<m_data->size()+needed_size);
                m_data->swap_out();
                std::ios_base::openmode mode = std::ios_base::out|std::ios_base::ate;
                mode |= m_mode;
                mode &= ~std::ios_base::trunc;
                //open(*m_data,mode);
                WOOL_DEBUG("fstream opening "<<m_data->m_fname<<" with mode "<< mode);
                m_filebuf.close();
                m_filebuf.open(m_data->m_fname.c_str(),mode);
                if(!m_filebuf.is_open())
                {
                    WOOL_DEBUG("could no open temp file "<<m_data->m_fname<<": "<<strerror(errno));
                    throw std::runtime_error(std::string("could no open temp file ")+m_data->m_fname+": "+strerror(errno));
                }
                seekoff(0,std::ios_base::end,mode);
                WOOL_DEBUG("SIZE AFTER SWAPOUT ("<<m_data->m_fname<<")"<<m_data->size()+needed_size);
              }
          }

          std::streampos
          swappable_data::streambuf::seekpos ( std::streampos sp,
                                                std::ios_base::openmode which )
          {
            if(m_filebuf.is_open())
              return m_filebuf.pubseekpos(sp,which);
            if(which&m_mode)
              m_offset = sp;
            return m_offset;
          }

          int swappable_data::streambuf::showmanyc ()
          {
            if(m_filebuf.is_open())
              return m_filebuf.in_avail();
            return std::max<int>(0,m_data->m_data.length()-m_offset);
          }

          swappable_data::streambuf::int_type
          swappable_data::streambuf::pbackfail ( swappable_data::streambuf::int_type c  )
          {
            if(m_filebuf.is_open())
              return m_filebuf.sputbackc(c);
            if((c == traits_type::eof())&&(m_offset>0))
            {
              m_offset=m_offset-(std::streamoff)1;
              return 1;
            }
            return traits_type::eof();
          }

          std::streampos
          swappable_data::streambuf::seekoff ( std::streamoff off,
                                                std::ios_base::seekdir way,
                                                std::ios_base::openmode which)
          {
            if(m_filebuf.is_open())
              return m_filebuf.pubseekoff(off,way,which);
            if(which&m_mode) {
              switch(way) {
                case(std::ios_base::beg): m_offset = off; break;
                case(std::ios_base::cur): m_offset += off; break;
                default:
                  m_offset = (m_data->m_data.length()>off)?m_data->m_data.length()-off:0;
                  break;
              }
              m_offset = std::min<std::streampos>(m_offset,m_data->m_data.length());
            }
            return m_offset;
          }

          swappable_data::streambuf::int_type
          swappable_data::streambuf::underflow     ()
          {
             if(m_filebuf.is_open()) {
	     	swappable_data::streambuf::int_type res = m_filebuf.sgetc();
               return res;
	     }
             if(m_offset<m_data->m_data.length())
	     {
	       return traits_type::to_int_type(m_data->m_data.data()[m_offset]);
	     }
	     WOOL_DEBUG("EOF");
             return traits_type::eof();
          }

          swappable_data::streambuf::int_type
          swappable_data::streambuf::uflow ( )
          {
            swappable_data::streambuf::int_type res = underflow();
	    if(m_filebuf.is_open()) {
	    	m_filebuf.sbumpc();
            }else{
	    if(res!=traits_type::eof())
              m_offset+=1;
	    }
            return res;
          }

          std::streamsize
          swappable_data::streambuf::xsgetn ( char *src, std::streamsize count)
          {
            if(m_filebuf.is_open())
              return m_filebuf.sgetn(src,count);
            std::streamsize  __len = std::min<std::streamsize>(showmanyc(),count);
            ::memmove(src,m_data->m_data.data()+m_offset,__len);
            m_offset+=__len;
            return __len;
          }

          std::streamsize
          swappable_data::streambuf::xsputn (const char *src, std::streamsize count)
          {
            if((std::ios_base::out&m_mode)==0)
            {
              std::cerr << "cannot write to a read-only stream" <<std::endl;
              return 0;
            }
            check_size(count);

            if(m_filebuf.is_open())
              return m_filebuf.sputn(src,count);

            if(m_mode&std::ios_base::app)
                m_offset = m_data->m_data.length();

            long replace_len = 0;
            replace_len = std::min<long>(count,m_data->m_data.length() - m_offset);
            if(replace_len>0l) {
              m_data->m_data.replace(m_offset,replace_len,src,count);
              count -= replace_len;
              src+=replace_len;
            }

            //WOOL_DEBUG("appending "<<count<<" characters at pos "<<m_offset);

            if(count>0)
              m_data->m_data.append(src,count);

            m_offset += replace_len+count;

            //WOOL_DEBUG("new length is "<<m_data->m_data.length());
            //WOOL_DEBUG("new offset is "<<m_offset);

            return replace_len+count;
          }

          swappable_data::streambuf::int_type swappable_data::streambuf::overflow ( swappable_data::streambuf::int_type i )
          {
            if((std::ios_base::out&m_mode)==0)
            {
              std::cerr << "cannot write to a read-only stream" <<std::endl;
              return traits_type::eof();
            }


            char c = static_cast<char_type>(i);

            if(m_mode&std::ios_base::app)
                m_offset = m_data->m_data.length();

            if(i!=traits_type::eof())
            {
              if(m_filebuf.is_open())
                return m_filebuf.sputc(c);
              if(m_offset>=m_data->m_data.length())
                m_data->m_data.push_back(c);
              else
                m_data->m_data[m_offset] = c;
              m_offset+=1;
              //return i;
            }
            sync();
            check_size(1);
            return i;


          }

      swappable_data::swappable_data () throw()
      {
      }

      swappable_data & swappable_data::operator = ( const swappable_data & src )
      {
        swappable_data::ostream os;
        clear();
        os.open(*this);
        os << src;
        return *this;
      }

      swappable_data::swappable_data ( const swappable_data & src )
      {
        swappable_data::ostream os;
        os.open(*this);
        os << src;
      }

      void swappable_data::swap_out()
      {
        static boost::mutex __m;
        assert(m_fname.empty());
        if(!m_fname.empty())
          return;
	boost::mutex::scoped_lock __l(__m);
        char _name[32];
        ::memcpy(_name,"/tmp/wool_XXXXXX",31);
        os_fd_type _fd = mkstemp(_name);
        if(_fd<0)
          throw std::runtime_error ("could not get a temp file");

        while(m_data.length()>0)
	{
          int res = ::write(_fd,m_data.data(),m_data.length());
	  if(res>0)
	   m_data.erase(0,res);
	  else
	   throw std::runtime_error("could not swap out: no space on device?");
        }

        m_fname = _name;
        ::close(_fd);
        WOOL_DEBUG("swapped out to "<<m_fname<<" size "<<size());

        m_data.clear();
      }

      swappable_data::size_type swappable_data::size () const
      {

      	if(m_fname.empty())
        {
          //WOOL_DEBUG("size: "<<m_data.length());
          return m_data.length();
        }
        int fd = ::open (m_fname.c_str(),O_RDONLY);
        swappable_data::size_type res = std::max<long>(::lseek(fd,0,SEEK_END),0l);
        ::close(fd);
        //WOOL_DEBUG("size: "<<res);
        return res;
      }

      void swappable_data::clear() throw ()
      {
          if(!m_fname.empty())
          {
            ::unlink(m_fname.c_str());
            m_fname.clear();
          }
          m_data.clear();
      }

      swappable_data::~swappable_data () throw()
      {
        clear();
      }

       // For CPPUnit-Test only
      std::string swappable_data::swap_file()
      {
        return m_fname;
      }
      
      void swappable_data::load( std::istream & in) {
      	    wool::swappable_data::ostream os;
	    os.open(*this,std::ios::out|std::ios::ate|std::ios::trunc|std::ios::app);
	    os << in.rdbuf() << std::flush;
	    os.close();
      }
      
      void swappable_data::save( std::ostream & os) const {
          if(!m_fname.empty()) {
          	std::filebuf is;
          	is.open(m_fname.c_str(),std::ios::in|std::ios::binary);
          	os << &is;
          } else {
          	os.write(m_data.data(),m_data.length());
          }
      }

};


namespace std {
  std::ostream & operator << ( std::ostream & os, const wool::swappable_data & d )
  {
    d.save(os);
    return os;
  }

  std::istream & operator >> ( std::istream & is, wool::swappable_data & d )
  {
    d.load(is);
    return is;
  }
};
