/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef WOOLTEMP_FILE_H
#define WOOLTEMP_FILE_H

#include "../base/debug.h"
#include "../base/cvs_info.h"
#include "../base/exception.h"


namespace wool
{
/**
 * This class is the implementation to create and use temporary files.
 * @author Vukasin Toroman <vtoroman@astaro.com> (C) Astaro AG 2005
 */
class tempfile
{
private:
  bool m_autoclean;
  std::string m_name;
public:
  /**
   *  cvs info
   */
  VERSION_INFO_INLINE("$Id: tmp_file.h,v 1.4 2007/01/11 15:18:42 vtoroman Exp $");
  /**
   *  @param autoclean delete file if lost the focus
   */
  inline tempfile( bool autoclean = true )
  :m_autoclean(autoclean)
  {
    char _name[32];
    ::memcpy(_name,"/tmp/wool_XXXXXX",31);
    int _tmp_fd = mkstemp(_name);
    if(_tmp_fd<0)
      throw wool::exception ("could not get a temp file");
    ::close(_tmp_fd);
    m_name = _name;
  }
  /**
   *  Returns the size of the temporary file
   *  @return size of the temporary file
   */
  inline size_t size () const
  {
        std::ifstream f(m_name.c_str());
        f.seekg(0,std::ifstream::end);
        if(f.tellg()<0)
        {
                WOOL_DEBUG ("invalid temp file");
                return 0;
        }
        return  f.tellg();

  }

  /**
   * Give up ownership of the file
   */
  inline void release() {
    m_autoclean = false;
  }

  /**
   * If autoclean is true the temporary file will be deleted.
   */
  inline virtual ~tempfile ()
  {
    if(m_autoclean)
      ::unlink(m_name.c_str());
  }

  /**
   *  Returns the full filename with path.
   *  @return filename
   */
  inline const std::string & path() const
  {
    return m_name;
  }
};

};

#endif
