/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <swool/bio.h>
#include <openssl/bio.h>
#include <openssl/evp.h>
#include <openssl/ssl.h>

namespace wool
{
  namespace swool
  {
    bio::method_factory bio::s_null = BIO_s_null;
    bio::method_factory bio::s_file = BIO_s_file;
    bio::method_factory bio::s_mem = BIO_s_mem;
    bio::method_factory bio::s_socket = BIO_s_socket;
    bio::method_factory bio::s_connect = BIO_s_connect;
    bio::method_factory bio::s_accept = BIO_s_accept;
    bio::method_factory bio::s_fd = BIO_s_fd;
    bio::method_factory bio::s_bio = BIO_s_bio;
    bio::method_factory bio::f_null = BIO_f_null;
    bio::method_factory bio::f_buffer = BIO_f_buffer;
    bio::method_factory bio::f_ssl = BIO_f_ssl;
    bio::method_factory bio::f_message_digest = BIO_f_md;
    bio::method_factory bio::f_base64 = BIO_f_base64;
    bio::method_factory bio::f_cipher = BIO_f_cipher;
    
    bio::bio( BIO * b, bool owns ): Wrapper<BIO>(b,owns) {      init();    }
    bio::bio( bio::method m ): Wrapper<BIO>( BIO_new(m), true ) {   init();    }
    bio::bio( bio::method_factory m): Wrapper<BIO>( BIO_new(m()), true ) {        init();      }
    bio::~bio()                                     {   clear();    }
    void bio::push (bio & b)                        {  BIO_push(get_wrapped(),b.get_wrapped());         }
    void bio::pop ()                                {  BIO_pop (get_wrapped());                         }
    int bio::read  (void * dst, int s)              {  return ::BIO_read (get_wrapped() ,dst, s);       }
    int bio::write (const void * src, int s)        {  return ::BIO_write (get_wrapped() ,src, s);      }
    void bio::set_flags( int f ) { get_wrapped()->flags|=f; };
    void bio::reset_flags( int f ) { get_wrapped()->flags&=~f; };
    int bio::get_flags( ) const { return get_wrapped()->flags; };
    long bio::ctrl(int cmd,long larg,void *parg)    {  return BIO_ctrl(get_wrapped(),cmd,larg,parg);    }
    long bio::callback_ctrl (int cmd, callback f)   {  return BIO_callback_ctrl (get_wrapped(), cmd, f);}
    char * bio::ptr_ctrl(int cmd,long larg)         {  return  BIO_ptr_ctrl(get_wrapped(),cmd,larg);    }
    long bio::int_ctrl(int cmd,long larg,int iarg)  {  return BIO_int_ctrl(get_wrapped(),cmd,larg,iarg);}
    int bio::reset()                                {  return BIO_reset(get_wrapped());                 }
    int bio::seek(int ofs)                          {  return BIO_seek(get_wrapped(),ofs);              }
    int bio::tell()                                 {  return BIO_tell(get_wrapped());                  }
    int bio::flush()                                {  return BIO_flush(get_wrapped());                 }
    int bio::eof()                                  {  return BIO_eof(get_wrapped());                   }
    int bio::set_close(long flag)                   {  return BIO_set_close(get_wrapped(),flag);        }
    int bio::get_close()                            {  return BIO_get_close(get_wrapped());             }
    int bio::pending()                              {  return BIO_pending(get_wrapped());               }
    int bio::wpending()                             {  return BIO_wpending(get_wrapped());              }
    size_t bio::ctrl_pending()                      {  return BIO_ctrl_pending(get_wrapped());          }
    size_t bio::ctrl_wpending()                     {  return BIO_ctrl_wpending(get_wrapped());         }
    void bio::free( BIO * b )                       {  BIO_free(b);                                     }
    BIO * bio::generate( )                          {  return BIO_new(s_null());                        }
    void bio::update()                              {   }
  };
};
