#ifndef X509_NAME_H
#define X509_NAME_H
#include <list>
#include <string>
#include <openssl/x509v3.h>
#include <iostream> 
namespace wool {
  namespace swool {
    class Name {
    public:
      struct Entry : public X509_NAME_ENTRY {

	static Entry * Create();
	std::string getData() const;
	std::string getName() const;

	void setName( int d);
	void setName( const std::string & d);
	void setData( const std::string & d);

      private:
	Entry();
      };
    
      typedef Entry value_type;
      typedef Entry * pointer;
      typedef Entry & reference;
      typedef const Entry const_value_type;
      typedef const Entry * const_pointer;
      typedef const Entry & const_reference;
      
      template< typename valueT, typename parentT> 
      struct iterator_base {
	typedef int difference_type;
	typedef std::bidirectional_iterator_tag iterator_category;

	typedef valueT value_type;
	typedef valueT * pointer;
	typedef valueT & reference;
	typedef iterator_base<valueT, parentT> Self;
	
	int id;
	parentT * parent;

	iterator_base ( const int & i = 0, parentT * p = NULL ) : id(i), parent(p) {}
	iterator_base ( const Self & i ) : id(i.id), parent(i.parent) {}
	~iterator_base () {}

	Self & operator -- () { return (*this)+=-1; }
	Self & operator ++ () { return (*this)+=1; }

	Self operator -- ( int i ) { 
	  Self ans = *this;
	  (*this)+=-1; 
	  return ans;
	}

	Self operator ++ ( int i ) { 
	  Self ans = *this;
	  (*this)+=1; 
	  return ans;
	}

	Self & operator += ( int i ) { 
	  if(id>=0) {
	    id+=i;
	    if((id<0)||(id>=parent->size()))
	      id = -1;
	  }
	  return *this;
	}
	Self & operator -= ( int i ) { 
	  return (*this)+=(-i);
	}

	bool operator == ( const Self & it ) const { return ( it.id == id ) && ( it.parent == it.parent ); };
	bool operator != ( const Self & it ) const { return ( it.id != id ) || ( it.parent != it.parent ); }
	bool operator < ( const Self & it ) const { return ( it.id < id ) || ( ( it.id == id ) && ( it.parent < it.parent ) ); }

	difference_type operator - ( const Self & p ) const { return id - p.id; }
	reference operator * () const { return (*parent)[id]; }
	pointer operator -> () const { return &((*parent)[id]);	}
	
      };

      typedef iterator_base<value_type, Name> iterator;
      typedef iterator_base<const_value_type, const Name> const_iterator;

      Name ( X509_NAME * name = NULL, bool owns = false);
      Name ( const Name & n );

      ~Name();
      
      size_t size() const;
      
      inline iterator begin() { return iterator(0, this); }
      inline iterator end() { return iterator(-1, this); }      
      inline const_iterator begin() const { return const_iterator(0,this); }
      inline const_iterator end() const { return  const_iterator(-1, this); }
    
      inline iterator find ( int id ) { return find(id, begin()); }
      inline const_iterator find ( int id ) const { return find(id, begin()); }
      
      inline iterator find ( const std::string & id ) { return find(id, begin()); }
      inline const_iterator find ( const std::string & id ) const { return find(id, begin()); }
      
      iterator find ( int id, iterator start );
      const_iterator find ( int id, const_iterator start ) const;

      iterator find ( const std::string & id, iterator start ); 
      const_iterator find ( const std::string & id, const_iterator start ) const;
      
      static int Resolve( const std::string & param_name );
      static std::string Resolve ( const int & param_id );

      iterator insert ( const std::string & name, const std::string & value );
      iterator insert ( const std::string & name, const std::string & value, iterator where  );

      void erase ( iterator what );
      void erase ( iterator from, iterator to );

      reference operator [] ( size_t id );
      const_reference operator [] ( size_t id ) const;

      X509_NAME * openssl_name();
      const X509_NAME * openssl_name() const;

    private:
      X509_NAME * m_name;
      bool m_owns;
    };
  };
};

#endif

