#include <openssl/pem.h>

#include "../swool/key.hh"
#include "../swool/cert.hh"
#include "../swool/pem.hh"
#include "../swool/streambuf_bio.h"

#define declare_PEM_loader(fname) {\
  wool::swool::streambuf_bio bio(*in.rdbuf());\
  if(fname(bio.get_wrapped(),&d,PemIO::password_cb,(void*)this)==NULL)\
    throw swool::exception();\
}

#define declare_PEM_enc(fname) {\
  wool::swool::streambuf_bio bio(*in.rdbuf());\
  if(fname(bio.get_wrapped(),d,EVP_des_ede3_cbc(),NULL,0,PemIO::password_cb,(void*)this)==0)\
    throw swool::exception();\
}

#define declare_PEM_saver(fname) {\
  wool::swool::streambuf_bio bio(*in.rdbuf());\
  if(fname(bio.get_wrapped(),d)==0)\
    throw swool::exception();\
}

namespace wool {
  namespace swool {

    PemIO::PemIO() : password("") {}

    void PemIO::setPassword( const std::string & p) { password = p; }
    const std::string PemIO::getPassword( ) const { return password; }

    int PemIO::password_cb(char *buf,int num, int rwflag,void *userdata) {
      PemIO * c = static_cast<PemIO*> (userdata);
      const std::string &  pass = c->getPassword();
      if((unsigned)num<pass.length()+1)
	return(0);
      memmove(buf,pass.data(),pass.length());
      return(pass.length());
    }

    void PemIO::loadParameters( DH * d, std::istream & in ) declare_PEM_loader(PEM_read_bio_DHparams);

    void PemIO::loadParameters( DSA * d, std::istream & in )declare_PEM_loader(PEM_read_bio_DSAparams);
    void PemIO::loadPrivateKey( DSA * d, std::istream & in )declare_PEM_loader(PEM_read_bio_DSAPrivateKey);

    void PemIO::loadPublicKey( RSA * d, std::istream & in ) declare_PEM_loader(PEM_read_bio_RSAPublicKey);
    void PemIO::loadPrivateKey( RSA * d, std::istream & in ) declare_PEM_loader(PEM_read_bio_RSAPrivateKey);

    void PemIO::load( EVP_PKEY * d, std::istream & in ) declare_PEM_loader(PEM_read_bio_PrivateKey);
    void PemIO::load( X509 * d, std::istream & in ) declare_PEM_loader(PEM_read_bio_X509);
    void PemIO::load( X509_REQ * d, std::istream & in ) declare_PEM_loader(PEM_read_bio_X509_REQ);
    void PemIO::load( X509_CRL * d, std::istream & in ) declare_PEM_loader(PEM_read_bio_X509_CRL);
    
    
    void PemIO::saveParameters( DH * d, std::ostream & in ) declare_PEM_saver(PEM_write_bio_DHparams);
    
    void PemIO::saveParameters( DSA * d, std::ostream & in )declare_PEM_saver(PEM_write_bio_DSAparams);
    void PemIO::savePrivateKey( DSA * d, std::ostream & in )declare_PEM_enc(PEM_write_bio_DSAPrivateKey);
    
    void PemIO::savePublicKey( RSA * d, std::ostream & in ) declare_PEM_saver(PEM_write_bio_RSAPublicKey);
    void PemIO::savePrivateKey( RSA * d, std::ostream & in ) declare_PEM_enc(PEM_write_bio_RSAPrivateKey);
    
    void PemIO::save( EVP_PKEY * d, std::ostream & in ) declare_PEM_enc(PEM_write_bio_PrivateKey);
    void PemIO::save( X509 * d, std::ostream & in ) declare_PEM_saver(PEM_write_bio_X509);
    void PemIO::save( X509_REQ * d, std::ostream & in ) declare_PEM_saver(PEM_write_bio_X509_REQ);
    void PemIO::save( X509_CRL * d, std::ostream & in ) declare_PEM_saver(PEM_write_bio_X509_CRL);
      
  };
};

