#ifndef SWOOL_PEM_H
#define SWOOL_PEM_H

#include <string>

#include "../base/wrapper.hh"

namespace wool {
  namespace swool {
    struct PemIO {
    public:
      PemIO();
      
      void setPassword( const std::string & p);
      const std::string getPassword( ) const;

      static int password_cb(char *buf,int num, int rwflag,void *userdata);

      void loadParameters( DH * d, std::istream & in );
      void loadParameters( DSA * d, std::istream & in );
      void loadPrivateKey( DSA * d, std::istream & in );
      void loadPublicKey( RSA * d, std::istream & in );
      void loadPrivateKey( RSA * d, std::istream & in );
      void load( EVP_PKEY * d, std::istream & in );
      void load( X509 * d, std::istream & in );
      void load( X509_REQ * d, std::istream & in );
      void load( X509_CRL * d, std::istream & in );

      void saveParameters( DH * d, std::ostream & in );
      void saveParameters( DSA * d, std::ostream & in );
      void savePrivateKey( DSA * d, std::ostream & in );
      void savePublicKey( RSA * d, std::ostream & in );
      void savePrivateKey( RSA * d, std::ostream & in );
      void save( EVP_PKEY * d, std::ostream & in );
      void save( X509 * d, std::ostream & in );
      void save( X509_REQ * d, std::ostream & in );
      void save( X509_CRL * d, std::ostream & in );
      
      
      template < typename T >
      void load( wool::Wrapper<T> & w, std::istream & in ) { load(w.get_wrapped(),in); 	w.pub_update(); }

      template < typename T >
      void loadParameters( wool::Wrapper<T> & w, std::istream & in ) { loadParameters(w.get_wrapped(),in);w.pub_update();  }

      template < typename T >
      void loadPublicKey( wool::Wrapper<T> & w, std::istream & in ) { loadPublicKey(w.get_wrapped(),in); w.pub_update();  }

      template < typename T >
      void loadPrivateKey( wool::Wrapper<T> & w, std::istream & in ) { loadPrivateKey(w.get_wrapped(),in); w.pub_update();  }

      template < typename T >
      void save( wool::Wrapper<T> & w, std::ostream & in ) { save(const_cast<T*>(w.get_wrapped()),in);  }
      
      template < typename T >
      void saveParameters( wool::Wrapper<T> & w, std::ostream & in ) { saveParameters(const_cast<T*>(w.get_wrapped()),in);  }
      
      template < typename T >
      void savePublicKey( wool::Wrapper<T> & w, std::ostream & in ) { savePublicKey(const_cast<T*>(w.get_wrapped()),in);  }
      
      template < typename T >
      void savePrivateKey( wool::Wrapper<T> & w, std::ostream & in ) { savePrivateKey(const_cast<T*>(w.get_wrapped()),in);  }
    private:
      std::string password; 
    };
  };
};

#endif
