#include "../swool/req.hh"
#include "../swool/streambuf_bio.h"
#include "../base/exception.h"
#include <openssl/err.h>

namespace wool {
  namespace swool {
    X509Req::X509Req( X509_REQ * r, bool owns  ) 
      : Wrapper<X509_REQ>(r,owns), 
	m_subject(NULL), 
	m_publicKey(NULL) {
      init();
    }

    void X509Req::free( X509_REQ * r ) {
      X509_REQ_free(r);
    }

    X509_REQ * X509Req::generate() {
      X509_REQ * r = X509_REQ_new();
      if(!r)throw swool::exception();
      return r;
    }


    const X509Req & X509Req::print ( std::ostream & out ) const {
      wool::swool::streambuf_bio bio(*out.rdbuf());
       if(X509_REQ_print(bio.get_wrapped(),const_cast<X509_REQ*>(get_wrapped()))!=1) {
	 throw swool::exception();
       }
      return *this;
    }


    X509Req::~X509Req() {
      delete m_subject;
      delete m_publicKey;
      clear();
    }
  

    void X509Req::setSubject( Name & name ) {
      X509_NAME * n = name.openssl_name();
      if(!X509_REQ_set_subject_name(get_wrapped(), n))
	throw swool::exception();
      update();
    }

    void X509Req::sign (Key & pkey) {
      const EVP_MD * digest = NULL;
      switch (pkey.getType()) {
      case Key::dsa:
	digest = EVP_dss1();
      case Key::rsa:
	digest = EVP_sha1();
      default:
	assert(false);
	break;
      };
      if (!(X509_REQ_sign (get_wrapped(), pkey.get_wrapped(), digest))) {
	throw swool::exception();
      } 
    }
    void X509Req::setPublicKey (Key & pkey) {
      if(X509_REQ_set_pubkey(get_wrapped(), pkey.get_wrapped())!=1)
	throw swool::exception();;
      update();
    }

    const Key & X509Req::getPublicKey ( ) const {
      return *m_publicKey;
    }

    void X509Req::update() {
      delete m_subject;
      m_subject = NULL;
      m_subject = new Name(X509_REQ_get_subject_name(get_wrapped()), false);
      delete m_publicKey;
      m_publicKey = NULL;
      EVP_PKEY * pk;
      if((pk = X509_REQ_get_pubkey(get_wrapped()))!=NULL)
	m_publicKey = new Key(pk, true);
    }

    const Name & X509Req::getSubject() const {
      return *m_subject;
    }

  };
};

