/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "swool/ssl_filter.h"
#include "swool/pem.hh"
namespace wool {

  namespace swool {
    /*
     ssl_method_factory const tls1 = TLSv1_method;
     ssl_method_factory const ssl2 = SSLv2_method;
     ssl_method_factory const ssl23 = SSLv23_method;
     ssl_method_factory const ssl3 = SSLv3_method;
    
     ssl_method_factory const tls1client = TLSv1_client_method;
     ssl_method_factory const ssl2client = SSLv2_client_method;
     ssl_method_factory const ssl23client = SSLv23_client_method;
     ssl_method_factory const ssl3client = SSLv3_client_method;
      
     ssl_method_factory const tls1server = TLSv1_server_method;
     ssl_method_factory const ssl2server = SSLv2_server_method;
     ssl_method_factory const  ssl23server = SSLv23_server_method;
     ssl_method_factory const  ssl3server = SSLv3_server_method;
    */
    SslContext::SslContext ( SSL_CTX * ctx, bool owns)
      : wool::Wrapper<SSL_CTX>(ctx, owns) {
      init();
    }

    SslContext::SslContext ( SslContext::ssl_method_factory method)
      : wool::Wrapper<SSL_CTX>(SSL_CTX_new(method()), true) {
      init();
    }

    SslContext::~SslContext () {
      clear();
    }

    void SslContext::setCert ( X509Cert & cert ) {
      if(!SSL_CTX_use_certificate(get_wrapped(), cert.get_wrapped()))
	throw swool::exception();
      update();
    }

    void SslContext::setPrivateKey ( Key & key ) {
      if(!SSL_CTX_use_PrivateKey(get_wrapped(), key.get_wrapped()))
	throw swool::exception();
      update();
    }
    
    void SslContext::setCALocation (cert_path_type path) {
      if(!(SSL_CTX_load_verify_locations(get_wrapped(),path.c_str(),0)))
	throw swool::exception();
    }
    
    void SslContext::setTmpRsa ( Rsa & rsa ) {
      if(!(SSL_CTX_set_tmp_rsa(get_wrapped(),rsa.get_wrapped())))
	throw swool::exception();
    } 
    
    void SslContext::setTmpDh (  Dh & dh ) {
      if(!(SSL_CTX_set_tmp_dh(get_wrapped(),dh.get_wrapped())))
	throw swool::exception();
    }

    void SslContext::update( ) {   }
    void SslContext::free ( SSL_CTX * r ) { SSL_CTX_free(r); }
    SSL_CTX * SslContext::generate ( ) { 
      throw swool::exception();
    }


     
    
    Ssl::Ssl ( SSL * ssl, bool owns ): Wrapper<SSL>(ssl,owns) {
      m_peer_cert = NULL;
      init();
    }
    
    Ssl::Ssl ( SslContext & ctx) : Wrapper<SSL>(SSL_new(ctx.get_wrapped()), true) {
      m_peer_cert = NULL;
      init();
    }

    Ssl::~Ssl() {
      delete m_peer_cert;
      clear();
    }
      
    const X509Cert & Ssl::getPeerCert() const {
      return * m_peer_cert;
    }

    Ssl::verification_result Ssl::getVerificationResult () const {
      return SSL_get_verify_result(get_wrapped());
    }

    void Ssl::setVerificationMode ( Ssl::verify_mode m) {
      SSL_set_verify(get_wrapped(),m,NULL);
    }
    
    void Ssl::setMethod ( SslContext::ssl_method m ) {
      if(SSL_set_ssl_method(get_wrapped(),m)!=1)
	throw swool::exception();
    }

    const SslContext::ssl_method Ssl::getMethod ( ) const {
      return SSL_get_ssl_method(const_cast<SSL*>(get_wrapped()));
    }
      
    void Ssl::renegotiate () {
      if(SSL_do_handshake(get_wrapped()) != 1)
	throw swool::exception();
    }

    void Ssl::doHandshake () {
      if((SSL_renegotiate(get_wrapped())) != 1)
	  throw swool::exception();
    }
    
    void Ssl::update() {
      delete m_peer_cert;
      m_peer_cert = NULL;
      m_peer_cert = new X509Cert(SSL_get_peer_certificate(get_wrapped()),false);
    }
    
    SSL * Ssl::generate( ) {
      throw swool::exception();
    }

    void Ssl::free( SSL *s ) {
      SSL_free(s);
    }
    

    SslBio::SslBio::SslBio( BIO * b, bool owns  ) : bio(b, owns), m_ssl(NULL) { 
      init();
    }

    SslBio::SslBio( SslContext & s, bool client_mode )  : bio (BIO_new_ssl(s.get_wrapped(), client_mode), true), m_ssl(NULL) {

      std::cerr << "SSLBio create for  " << (void*)get_wrapped()<<std::endl;

      SSL * ss = SSL_new(s.get_wrapped());
      if(ss==NULL) {
	std::cerr << "No SSL from context " << std::endl;
      }
      if(BIO_set_ssl(get_wrapped(),ss,BIO_CLOSE)!=1)
	throw swool::exception();
      init();
    }
    
    SslBio::~SslBio() {
      delete m_ssl;
      clear();
    }
    
    Ssl & SslBio::getSsl() {
      return *m_ssl;
    }

    const Ssl & SslBio::getSsl() const {
      return *m_ssl;
    }

    void SslBio::setSsl( Ssl & s, int close_flag ) {
      BIO_set_ssl(get_wrapped(), s.get_wrapped(), close_flag);
      update();
    }

    void SslBio::free( BIO * b ) {
      BIO_ssl_shutdown(b);
      bio::free(b);
    }

    void SslBio::update() {
      delete m_ssl;
      m_ssl = NULL;
      bio::update();
      SSL * ssl_ptr = NULL;
      BIO_get_ssl(get_wrapped(), ssl_ptr);
      if(ssl_ptr==NULL)
	std::cerr << "Uh oh, no SSL" << std::endl;
      m_ssl = new Ssl( ssl_ptr, false);
    }
    
  }

#define skip 1
#ifndef skip

  namespace swool {
    const ssl_context::ssl_method_factory ssl_context::ssl23= SSLv23_method;
    const ssl_context::ssl_method_factory ssl_client::ssl23 = SSLv23_client_method;
    const ssl_context::ssl_method_factory ssl_server::ssl23 = SSLv23_server_method;
    
    const ssl_context::ssl_method_factory ssl_context::ssl3 = SSLv3_method;
    const ssl_context::ssl_method_factory ssl_client::ssl3  = SSLv3_client_method;
    const ssl_context::ssl_method_factory ssl_server::ssl3  = SSLv3_server_method;
    
    const ssl_context::ssl_method_factory ssl_context::ssl2 = SSLv2_method;
    const ssl_context::ssl_method_factory ssl_client::ssl2  = SSLv2_client_method;
    const ssl_context::ssl_method_factory ssl_server::ssl2  = SSLv2_server_method;
    
    const ssl_context::ssl_method_factory ssl_context::tls1 = TLSv1_method;
    const ssl_context::ssl_method_factory ssl_client::tls1  = TLSv1_client_method;
    const ssl_context::ssl_method_factory ssl_server::tls1  = TLSv1_server_method;
    
    const ssl_context::verify_mode ssl_context::verify_none = SSL_VERIFY_NONE;
    const ssl_context::verify_mode ssl_context::verify_peer = SSL_VERIFY_PEER;
    const ssl_context::verify_mode ssl_context::verify_require_peer_cert = SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
    const ssl_context::verify_mode ssl_context::verify_client_once = SSL_VERIFY_CLIENT_ONCE;


/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef SWOOLSSL_FILTER_H
#define SWOOLSSL_FILTER_H

#include "../base/debug.h"
#include "../swool/bio.h"
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/evp.h>
#include <openssl/x509v3.h>

namespace wool {
  namespace swool {
    class ssl_context {
    private:
      SSL_CTX * m_ssl_ctx;
      std::string m_password;
      
    public:
      /**
       *  ssl method definition
       */
      typedef SSL_METHOD * ssl_method;
      /**
       *  ssl method factory definition
       */
      typedef SSL_METHOD * (*ssl_method_factory) (void);
      /**
       *  certification verification result definition
       */
      typedef long cert_verification_result;
      /**
       *  verification mode definition
       */
      typedef int verify_mode;
      
      /**
       * certificate location type
       */
      typedef std::string cert_path_type;
      
      
      static inline int password_cb(char *buf,int num, int rwflag,void *userdata) {
        ssl_context * c = static_cast<ssl_context*> (userdata);
        std::string pass = c->get_password();
        if((unsigned)num<pass.length()+1)
	  return(0);
        strcpy(buf,pass.c_str());
        return(pass.length());
      }
      
      inline void set_password (std::string s) {
        m_password =s;
      }
      
      inline std::string get_password () {
        return m_password;
      }

      /**
       *  <b>Server mode:</b> the server will not send a client certificate request to the client,
       *  so the client will not send a certificate.<br>
       *  <b>Client mode:</b> if not using an anonymous cipher (by default disabled),
       *  the server will send a certificate which will be checked.
       *  The result of the certificate verification process can be checked after the TLS/SSL
       *  handshake using the SSL_get_verify_result(3) function.
       *  The handshake will be continued regardless of the verification result.
       */
      static const verify_mode verify_none;
      /**
       *  <b>Server mode:</b> the server sends a client certificate request to the client.
       *  The certificate returned (if any) is checked. If the verification process fails,
       *  the TLS/SSL handshake is immediately terminated with an alert message containing
       *  the reason for the verification failure.<br>
       *  <b>Client mode:</b> the server certificate is verified. If the verification process fails,
       *  the TLS/SSL handshake is immediately terminated with an alert message containing the reason
       *  for the verification failure. If no server certificate is sent, because an anonymous cipher is used,
       *  'verify_peer' is ignored.
       */
      static const verify_mode verify_peer ;
      /**
       *  <b>Server mode:</b> if the client did not return a certificate, the TLS/SSL handshake
       *  is immediately terminated with a ``handshake failure'' alert. This flag must be used together with
       *  'verify_peer'.<br>
       *  <b>Client mode:</b> ignored
       */
      static const verify_mode verify_require_peer_cert;
      /**
       *  <b>Server mode:</b> only request a client certificate on the initial TLS/SSL handshake.
       *  Do not ask for a client certificate again in case of a renegotiation.
       *  This flag must be used together with 'verify_peer'.<br>
       *  <b>Client mode:</b> ignored
       */
      static const verify_mode verify_client_once;
      /**
       *  Initialize the SSL library and encryption algorithm.
       */
      static void init(){
	SSL_library_init();
	ERR_load_crypto_strings();
	ERR_load_SSL_strings();
	OpenSSL_add_all_algorithms();
      }
      /**
       *  A TLS/SSL connection established with these methods will only understand the TLSv1 protocol.
       *  A client will send out TLSv1 client hello messages and will indicate that it only understands TLSv1.
       *  A server will only understand TLSv1 client hello messages. This especially means, that it will
       *  not understand SSLv2 client hello messages which are widely used for compatibility reasons,
       *  see SSLv23_*_method(). It will also not understand SSLv3 client hello messages.
       */
      static const ssl_method_factory tls1;
      /**
       *  A TLS/SSL connection established with these methods will only understand the SSLv2 protocol.
       *  A client will send out SSLv2 client hello messages and will also indicate that it only understand SSLv2.
       *  A server will only understand SSLv2 client hello messages.
       */
      static const ssl_method_factory ssl2;
      /**
       *  A TLS/SSL connection established with these methods will understand the SSLv2, SSLv3, and TLSv1 protocol.
       *  A client will send out SSLv2 client hello messages and will indicate that it also understands SSLv3 and TLSv1.
       *  A server will understand SSLv2, SSLv3, and TLSv1 client hello messages.
       *  This is the best choice when compatibility is a concern.
       */
      static const ssl_method_factory ssl23;
      /**
       *  A TLS/SSL connection established with these methods will only understand the SSLv3 protocol.
       *  A client will send out SSLv3 client hello messages and will indicate that it only understands SSLv3.
       *  A server will only understand SSLv3 client hello messages. This especially means,
       *  that it will not understand SSLv2 client hello messages which are widely used for compatibility reasons,
       *  see ssl23_method_factory.
       */
      static const ssl_method_factory ssl3;
      
      
      ssl_context ( ssl_method method ):m_ssl_ctx (SSL_CTX_new(method)) {
	SSL_CTX_set_default_passwd_cb(m_ssl_ctx,ssl_context::password_cb);
	SSL_CTX_set_default_passwd_cb_userdata(m_ssl_ctx, (void *)this);
#if (OPENSSL_VERSION_NUMBER < 0x00905100L)
	SSL_CTX_set_verify_depth(m_ssl_ctx.ctx(),1);
#endif
      };
	
	inline void load_cert_chain ( cert_path_type path ) {
	  if (!SSL_CTX_use_certificate_chain_file(m_ssl_ctx,path.c_str())) {
	    ERR_print_errors_fp(stderr);
	    throw std::ios::failure ("SSL failed to load certificate");
	  }
	}
	inline void load_pam_key(cert_path_type path )  {
	  if(!(SSL_CTX_use_PrivateKey_file(m_ssl_ctx,path.c_str(),SSL_FILETYPE_PEM)))
	    throw std::ios::failure ("Can't read key file");
	}
	
	inline void load_ca_location(cert_path_type path )  {
	  if(!(SSL_CTX_load_verify_locations(m_ssl_ctx,path.c_str(),0)))
	    throw std::ios::failure ("Can't read CA list");
	}

	inline void generate_eph_rsa_key() {
	  RSA * rsa=RSA_generate_key(512,RSA_F4,NULL,NULL);
	  int res = SSL_CTX_set_tmp_rsa(m_ssl_ctx,rsa);
	  RSA_free(rsa);
	  if(!res)
	    throw std::ios::failure ("Couldn't set RSA key");
	}
	
	inline void load_dh_params(cert_path_type path )  {
	  DH *ret=0;
	  BIO *bio =BIO_new_file(path.c_str(),"r") ;
	  if (bio == NULL)
	    throw std::ios::failure("Couldn't open DH file");
	  ret=PEM_read_bio_DHparams(bio,NULL,NULL,NULL);
	  BIO_free(bio);
	  if(SSL_CTX_set_tmp_dh(m_ssl_ctx,ret)<0)
	    throw std::ios::failure("Couldn't set DH parameters");
	}
	
	inline SSL_CTX * ctx () {
	  return m_ssl_ctx;
	}
    };
    
    /**
     *  This is a filter BIO which is a wrapper round the OpenSSL SSL routines adding a BIO ``flavour'' to SSL I/O.
     *  @author Vukasin Toroman <vtoroman@astaro.de> (C) Astaro AG 2005
     */
    class ssl : public wool::swool::bio
    {
    private:
      SSL * m_ssl;
      ssl_context & m_ssl_ctx;
      
    public:
      /**
       *  @param method TSL/SSL protocol version to use
       */
      ssl( ssl_context & ssl_ctx )
	:bio( bio::f_ssl ),m_ssl_ctx(ssl_ctx)
	{
	  m_ssl = SSL_new(m_ssl_ctx.ctx());
	  BIO_set_ssl(m_ssl_bio,m_ssl,BIO_NOCLOSE);
	}
	/**
	 *  Free the SSL context
	 */
	virtual ~ssl()
	  {     }
	
	//bool
	/**
	 *  Returns a pointer to the X509 certificate the peer presented.
	 *  If the peer did not present a certificate, NULL is returned.
	 *  @return pointer to the X509 certification or NULL if not presented.
	 */
	X509 *get_peer_certificate ()
	  {  return SSL_get_peer_certificate(m_ssl);  }
	/**
	 *  Returns the name of the certificate
	 *  @return certificate name
	 */
	std::string get_peer_certificate_name ()
	  {   char peer_CN[1024];
	    X509_NAME_get_text_by_NID(X509_get_subject_name(get_peer_certificate()),
				      NID_commonName,
				      peer_CN,
				      1024);
	    return peer_CN;
	  }
	
	/**
	 *  Returns the result of the verification of the X509 certificate presented by the peer, if any.
	 *  @return X509_V_OK If the verification succeeded or no peer certificate was presented. Else any other value
	 *  documented value in \ref ssl_verify.
	 */
	inline ssl_context::cert_verification_result get_verify_result () const
	  {
	    return SSL_get_verify_result(m_ssl);
	  }
	/**
	 *  Sets the verification mode.
	 *  @param m verify_mode
	 */
	inline void set_verify ( ssl_context::verify_mode m )
	  {
	    return SSL_set_verify(m_ssl, m, NULL);
	  }
	/**
	 *  Sets the TSL/SSL protocol version
	 *  @param method ssl_method_factory
	 */
	inline void set_method( const ssl_context::ssl_method method )
	  {   SSL_set_ssl_method(m_ssl,method);       }
	/**
	 *  Gets the TSL/SSL protocol version
	 *  @return ssl_method_factory
	 */
	inline ssl_context::ssl_method  get_method( )
	  {   return SSL_get_ssl_method(m_ssl);       }
	/**
	 *  Sets ssl to work in client mode.
	 */
	inline void set_connect_state ()
	  {   BIO_set_ssl_mode(m_ssl_bio,1);           }
	/**
	 *  Sets ssl to work in server mode.
	 */
	inline void set_accept_state ()
	  {   BIO_set_ssl_mode(m_ssl_bio,0);            }
	
	/**
	 *  Creates a new set of cipher keys for an existing Secure Sockets Layer (SSL) session.
	 */
	inline void renegotiate ()
	  {
	    if(SSL_renegotiate(m_ssl) <= 0)
	      {
		throw std::ios::failure ("SSL failed to renegotiate");
	      }
	  }
	/**
	 *  Attempts to complete an SSL handshake on the supplied BIO and establish the SSL connection.
	 */
	inline void do_handshake ()
	  {
	    if(BIO_do_handshake(m_ssl_bio) <= 0)
	      {
		WOOL_DEBUG("SSL handshake failed:"<<this->get_verify_result());
		ERR_print_errors_fp(stderr);
		throw std::ios::failure ("SSL handshake error");
	      }
	  }
	
    };
    /**
     *  SSL Client implementation.
     *  @author Vukasin Toroman <vtoroman@astaro.de> (C) Astaro AG 2005
     */
    class ssl_client : public ssl
    {
    public:
      /**
       *  A TLS/SSL connection established with these methods will only understand the TLSv1 protocol.
       *  A client will send out TLSv1 client hello messages and will indicate that it only understands TLSv1.
       *  A server will only understand TLSv1 client hello messages. This especially means, that it will
       *  not understand SSLv2 client hello messages which are widely used for compatibility reasons,
       *  see SSLv23_*_method(). It will also not understand SSLv3 client hello messages.
       */
      static const ssl_context::ssl_method_factory tls1;
      /**
       *  A TLS/SSL connection established with these methods will only understand the SSLv2 protocol.
       *  A client will send out SSLv2 client hello messages and will also indicate that it only understand SSLv2.
       *  A server will only understand SSLv2 client hello messages.
       */
      static const ssl_context::ssl_method_factory ssl2;
      /**
       *  A TLS/SSL connection established with these methods will understand the SSLv2, SSLv3, and TLSv1 protocol.
       *  A client will send out SSLv2 client hello messages and will indicate that it also understands SSLv3 and TLSv1.
       *  A server will understand SSLv2, SSLv3, and TLSv1 client hello messages.
       *  This is the best choice when compatibility is a concern.
       */
      static const ssl_context::ssl_method_factory ssl23;
      /**
       *  A TLS/SSL connection established with these methods will only understand the SSLv3 protocol.
       *  A client will send out SSLv3 client hello messages and will indicate that it only understands SSLv3.
       *  A server will only understand SSLv3 client hello messages. This especially means,
       *  that it will not understand SSLv2 client hello messages which are widely used for compatibility reasons,
       *  see ssl23_method_factory.
       */
      static const ssl_context::ssl_method_factory ssl3;
      /**
       *  @param method TSL/SSL protocol version to use
       */
      ssl_client( ssl_context & m)
	:ssl( m )
	{  set_connect_state(); }
	
	virtual ~ssl_client()    {}
    };
    /**
     *  SSL Server implementation.
     *  @author Vukasin Toroman <vtoroman@astaro.de> (C) Astaro AG 2005
     */
    class ssl_server : public ssl {
    public:
      /**
       *  A TLS/SSL connection established with these methods will only understand the TLSv1 protocol.
       *  A client will send out TLSv1 client hello messages and will indicate that it only understands TLSv1.
       *  A server will only understand TLSv1 client hello messages. This especially means, that it will
       *  not understand SSLv2 client hello messages which are widely used for compatibility reasons,
       *  see SSLv23_*_method(). It will also not understand SSLv3 client hello messages.
       */
      static const ssl_context::ssl_method_factory tls1;
      /**
       *  A TLS/SSL connection established with these methods will only understand the SSLv2 protocol.
       *  A client will send out SSLv2 client hello messages and will also indicate that it only understand SSLv2.
       *  A server will only understand SSLv2 client hello messages.
       */
      static const ssl_context::ssl_method_factory ssl2;
      /**
       *  A TLS/SSL connection established with these methods will understand the SSLv2, SSLv3, and TLSv1 protocol.
       *  A client will send out SSLv2 client hello messages and will indicate that it also understands SSLv3 and TLSv1.
       *  A server will understand SSLv2, SSLv3, and TLSv1 client hello messages.
       *  This is the best choice when compatibility is a concern.
       */
      static const ssl_context::ssl_method_factory ssl23;
      /**
       *  A TLS/SSL connection established with these methods will only understand the SSLv3 protocol.
       *  A client will send out SSLv3 client hello messages and will indicate that it only understands SSLv3.
       *  A server will only understand SSLv3 client hello messages. This especially means,
       *  that it will not understand SSLv2 client hello messages which are widely used for compatibility reasons,
       *  see ssl23_method_factory.
       */
      static const ssl_context::ssl_method_factory ssl3;
      /**
       *  @param method TSL/SSL protocol version to use
       */
      ssl_server( ssl_context & m)
	:ssl( m )
	{  set_accept_state(); }
	
	virtual ~ssl_server(){}
    };
  };
};

/**
   \page ssl_verify  SSL verification results and error codes
   @see http://www.openssl.org/docs/apps/verify.html#<p>
   
   An exhaustive list of the error codes and messages is shown below. Some of the error codes are defined
   but never returned: these are described as ``unused''.<p>
   
   <b>0 X509_V_OK: ok</b><br>
   the operation was successful.<p>
   
   <b>2 X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT: unable to get issuer certificate</b><br>
   the issuer certificate could not be found: this occurs if the issuer certificate of an untrusted
   certificate cannot be found.<p>
   
   <b>3 X509_V_ERR_UNABLE_TO_GET_CRL unable to get certificate CRL</b><br>
   the CRL of a certificate could not be found. Unused.<p>

   <b>4 X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE: unable to decrypt certificate's signature</b><br>
   the certificate signature could not be decrypted. This means that the actual signature value could not be
   determined rather than it not matching the expected value, this is only meaningful for RSA keys.<p>
   
   <b>5 X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE: unable to decrypt CRL's signature</b><br>
   the CRL signature could not be decrypted: this means that the actual signature value could not be
   determined rather than it not matching the expected value. Unused.<p>
   
   <b>6 X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY: unable to decode issuer public key</b><br>
   the public key in the certificate SubjectPublicKeyInfo could not be read.<p>
   
   <b>7 X509_V_ERR_CERT_SIGNATURE_FAILURE: certificate signature failure</b><br>
   the signature of the certificate is invalid.<p>
   
   <b>8 X509_V_ERR_CRL_SIGNATURE_FAILURE: CRL signature failure</b><br>
   the signature of the certificate is invalid. Unused.<p>
   
   <b>9 X509_V_ERR_CERT_NOT_YET_VALID: certificate is not yet valid</b><br>
   the certificate is not yet valid: the notBefore date is after the current time.<p>
   
   <b>10 X509_V_ERR_CERT_HAS_EXPIRED: certificate has expired</b><br>
   the certificate has expired: that is the notAfter date is before the current time.<p>

   <b>11 X509_V_ERR_CRL_NOT_YET_VALID: CRL is not yet valid</b><br>
   the CRL is not yet valid. Unused.<p>
   
   <b>12 X509_V_ERR_CRL_HAS_EXPIRED: CRL has expired</b><br>
   the CRL has expired. Unused.<p>
   
   <b>13 X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD: format error in certificate's notBefore field</b><br>
   the certificate notBefore field contains an invalid time.<p>
   
   <b>14 X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD: format error in certificate's notAfter field</b><br>
   the certificate notAfter field contains an invalid time.<p>
   
   <b>15 X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD: format error in CRL's lastUpdate field</b><br>
   the CRL lastUpdate field contains an invalid time. Unused.<p>
   
   <b>16 X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD: format error in CRL's nextUpdate field</b><br>
   the CRL nextUpdate field contains an invalid time. Unused.<p>

   <b>17 X509_V_ERR_OUT_OF_MEM: out of memory</b><br>
   an error occurred trying to allocate memory. This should never happen.<p>

   <b>18 X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT: self signed certificate</b><br>
   the passed certificate is self signed and the same certificate cannot be found in the list of trusted
   certificates.<p>
   
   <b>19 X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN: self signed certificate in certificate chain</b><br>
   the certificate chain could be built up using the untrusted certificates but the root could not be found locally.<p>
   
   <b>20 X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY: unable to get local issuer certificate</b><br>
   the issuer certificate of a locally looked up certificate could not be found. This normally means the list of
   trusted certificates is not complete.<p>
   
   <b>21 X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE: unable to verify the first certificate</b><br>
   no signatures could be verified because the chain contains only one certificate and it is not self signed.<p>
   
   <b>22 X509_V_ERR_CERT_CHAIN_TOO_LONG: certificate chain too long</b><br>
   the certificate chain length is greater than the supplied maximum depth. Unused.<p>
   
   <b>23 X509_V_ERR_CERT_REVOKED: certificate revoked</b><br>
   the certificate has been revoked. Unused.<p>
   
   <b>24 X509_V_ERR_INVALID_CA: invalid CA certificate</b><br>
   a CA certificate is invalid. Either it is not a CA or its extensions are not consistent with the supplied purpose.<p>
   
   <b>25 X509_V_ERR_PATH_LENGTH_EXCEEDED: path length constraint exceeded</b><br>
   the basicConstraints pathlength parameter has been exceeded.<p>
   
   <b>26 X509_V_ERR_INVALID_PURPOSE: unsupported certificate purpose</b><br>
   the supplied certificate cannot be used for the specified purpose.<p>
   
   <b>27 X509_V_ERR_CERT_UNTRUSTED: certificate not trusted</b><br>
   the root CA is not marked as trusted for the specified purpose.<p>
   
   <b>28 X509_V_ERR_CERT_REJECTED: certificate rejected</b><br>
   the root CA is marked to reject the specified purpose.<p>
   
   <b>29 X509_V_ERR_SUBJECT_ISSUER_MISMATCH: subject issuer mismatch</b><br>
   the current candidate issuer certificate was rejected because its subject name did not match the issuer name of
   the current certificate. Only displayed when the -issuer_checks option is set.<p>
   
   <b>30 X509_V_ERR_AKID_SKID_MISMATCH: authority and subject key identifier mismatch</b><br>
   the current candidate issuer certificate was rejected because its subject key identifier was present and did
   not match the authority key identifier current certificate. Only displayed when the -issuer_checks option is set.<p>
   
   <b>31 X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH: authority and issuer serial number mismatch</b><br>
   the current candidate issuer certificate was rejected because its issuer name and serial number was present and
   did not match the authority key identifier of the current certificate. Only displayed when the -issuer_checks
   option is set.<p>
   
   <b>32 X509_V_ERR_KEYUSAGE_NO_CERTSIGN:key usage does not include certificate signing</b><br>
   the current candidate issuer certificate was rejected because its keyUsage extension does not permit
   certificate signing.<p>
   
   <b>50 X509_V_ERR_APPLICATION_VERIFICATION: application verification failure</b><br>
   an application specific error. Unused.
   
*/
  };
#endif
#endif

};
