#ifndef SWOOL_STORE_H
#define SWOOL_STORE_H

#include "../swool/key.hh"
#include "../swool/cert.hh"
#include "../base/wrapper.hh"

#include <openssl/x509.h>
#include <openssl/x509_vfy.h>

#include <boost/bind.hpp>
#include <boost/function.hpp>

namespace wool {
  namespace swool {  
    class X509Store : public wool::Wrapper<X509_STORE>{
    public:
      typedef int error_code;
      typedef std::set<error_code> error_code_set;

      X509Store( X509_STORE * r = NULL, bool owns = false );
      ~X509Store();
      
     
      void verify (X509Cert & cert);

      void loadLocations( const std::string & ca_file, const std::string & ca_folder);
      void addCrlLookup( const std::string & crl_file, int format = X509_FILETYPE_PEM );

      void setIgnoredErrors( const error_code_set & e );
      const error_code_set & getIgnoredErrors() const;


      void setFlags( unsigned long flags );
      unsigned long getFlags() const;

      static int verify_callback (int ok, X509_STORE_CTX * stor);    
      
    protected:
      virtual void update();
      virtual void free( X509_STORE * c);
      virtual X509_STORE *generate();
      
      error_code_set m_ignored_errors;
      
    };
  };
};


#endif
