/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/**************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef SWOOLSTREAMBUF_BIO_H
#define SWOOLSTREAMBUF_BIO_H

#include "../swool/bio.h"
#include <algorithm>
namespace wool {
namespace swool {
inline bool is_ws ( char c ) {
    switch (c) {
    case '\0':
    case ' ':
    case '\t':
    case '\n':
    case '\r':
        return true;
    default:
        return false;
    }
}

template <typename sink_type, typename traits_type>
class bio_sink : public bio {
public:
    static inline bio::method method_factory (void) {
        static BIO_METHOD s_method = {
                                         (21|0x0400|0x0100),
                                         traits_type::get_name(),
                                         bio_sink<sink_type,traits_type>::bwrite,
                                         bio_sink<sink_type,traits_type>::bread,
                                         bio_sink<sink_type,traits_type>::bputs,
                                         bio_sink<sink_type,traits_type>::bgets,
                                         bio_sink<sink_type,traits_type>::ctrl,
                                         bio_sink<sink_type,traits_type>::create,
                                         bio_sink<sink_type,traits_type>::destroy,
                                         bio_sink<sink_type,traits_type>::callback_ctrl
                                     };
        return & s_method;
    }

    inline static void fill_method ( BIO_METHOD & method, const char * name  ) {
        method.type = (21|0x0400|0x0100);
        method.name = traits_type::get_name();
        method.bwrite = bio_sink<sink_type,traits_type>::bwrite;
        method.bread = bio_sink<sink_type,traits_type>::bread;
        method.bputs = bio_sink<sink_type,traits_type>::bputs;
        method.bgets = bio_sink<sink_type,traits_type>::bgets;
        method.ctrl = bio_sink<sink_type,traits_type>::ctrl;
        method.create = bio_sink<sink_type,traits_type>::create;
        method.destroy = bio_sink<sink_type,traits_type>::destroy;
        method.callback_ctrl = bio_sink<sink_type,traits_type>::callback_ctrl;
    }

    bio_sink(sink_type & s) : bio(bio_sink<sink_type,traits_type>::method_factory) {
        get_wrapped()->ptr = & s;
    }

    bio_sink(sink_type * s) : bio(bio_sink<sink_type,traits_type>::method_factory) {
        get_wrapped()->ptr =  s;
    }

    virtual ~bio_sink() {}

    inline static int bwrite(BIO * b, const char * buffer , int len ) {
      if(len<=0) return 0;
      
      sink_type * ptr = static_cast<sink_type*>( b->ptr);
      int res = 0;
      int tmp_res = 1;
      while((res<len)&& (tmp_res>0)) {
	tmp_res = traits_type::write(ptr,buffer,len-res);
	res+=tmp_res;
	buffer+=tmp_res;
      }
      
      return  res;
    }

    inline static int  bread(BIO *b, char * c , int i) {
      if(i<=0) return 0;
      sink_type * ptr = static_cast<sink_type*> (b->ptr);
      int res = 0;
      int tmp_res = 1;
      while((res<i)&& (tmp_res>0)) {
	tmp_res = traits_type::read(ptr,c,i-res);
	res+=tmp_res;
	c+=tmp_res;
      }
      return res;
    }

    inline static int  bputs(BIO *b, const char * c) {
        const char * end = std::find(c,(const char *)NULL,'\0');
	if(end == c)
	  return 1;
        return bwrite(b,c,end-c);
	
    }

    inline static int  bgets(BIO *b, char * c , int i) {
      char * end = c+i;
      char * begin = c;
      for ( ;c!=end;c++) {
	if(bread(b,c,1)!=1){
	  *c = '\0';
	  return c - begin;
	}
	if (*c=='\n') {
	  c++;
	  if(c!=end)
	    *c = '\0';
	  return c - begin;
	}
      }
      return (i<=0)?1:i;
    }

    inline static int  create(BIO *b) {
        b->init=1;
        b->ptr = NULL;
        b->flags = 0;
        return 1;
    }

    inline static int  destroy(BIO *b) {
        b->ptr = NULL;
        return 1;
    }

    inline static long ctrl(BIO *b, int cmd, long num, void *ptr) {
        sink_type * p = static_cast<sink_type*>(b->ptr);
        return traits_type::ctrl(p, cmd, num, ptr);
    }

    inline static long callback_ctrl(BIO *b, int cmd, bio_info_cb *fp) {
        return 1;
    }

};

class streambuf_traits {
public:
    static const char * get_name () {
        static const char * name = "std::streambuf";
        return name;
    }
    static inline int write ( std::streambuf * sb, const char * b, int len ) {
        return sb->sputn(b,len);
    }
    static inline int read ( std::streambuf * sb, char * b, int len ) {
        return sb->sgetn(b,len);
    }
    static inline int ctrl ( std::streambuf * sb, int cmd, long num, void *ptr) {
      return 1;//sb->pubsync();
    }
};


template <typename io_type>
class simple_traits {
public:
    static const char * get_name () {
        static const char * name = "noname_wrapper";
        return name;
    }

    static inline int write ( io_type * sb, const char * b, int len ) {
        return sb->write(b,len);
    }

    static inline int read ( io_type * sb, char * b, int len ) {
        return sb->read(b,len);
    }

    static inline int ctrl ( io_type * sb, int cmd, long num, void *ptr) {
        return 1;
    }
};

template <typename IteratorType>
class iterator_io {
public:
    typedef IteratorType iterator;
    iterator __begin;
    iterator __end;
    iterator __current;

    iterator_io (iterator b, iterator e) : __begin(b), __end(e), __current(b) {}

    ~iterator_io () {}

}
;

template<typename IteratorType>
class iterator_traits {
public:
    typedef IteratorType iterator;
    typedef iterator_io<IteratorType> io_type;

    static const char * get_name () {
        const char * name = "string";
        return name;
    }

    static inline int write ( io_type * sb, const char * b, int len ) {
        if(len>0)
            std::copy(b,b+len,sb->__current );
        return len;
    }

    static inline int read ( io_type * sb, char * b, int len ) {
        int avail = std::min<int>(len,sb->__end - sb->__current);
        if(avail>0)
            std::copy(b,b+avail,sb->__current);
        return avail;
    }

    static inline int ctrl ( io_type * sb, int cmd, long num, void *ptr) {
        return 1;
    }

};

typedef bio_sink<std::streambuf, streambuf_traits> streambuf_bio;
//typedef socket_sink<wool::socket, simple_traits<wool::socket> > socket_bio;


/*
class streambuf_bio :public bio {
  public:
    static BIO_METHOD s_streambuf_method;
    static bio::method s_streambuf ( );

    streambuf_bio ( std::streambuf * sink = NULL );
    virtual ~streambuf_bio ();

    std::streambuf * set_streambuf ( std::streambuf * sink );

    static int  bwrite(BIO *, const char *, int);
    static int  bread(BIO *, char *, int);
    static int  bputs(BIO *, const char *);
    static int  bgets(BIO *, char *, int);
    static int  create(BIO *);
    static int  destroy(BIO *);
    static long ctrl(BIO *, int cmd, long num, void *ptr);
    static long callback_ctrl(BIO *b, int cmd, bio_info_cb *fp);
};
*/
};
};

#endif
