/***************************************************************************
*   Copyright (C) 2005 by Vukasin Toroman                                 *
*   vtoroman@astaro.de                                                    *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Library General Public License as       *
*   published by the Free Software Foundation; either version 2 of the    *
*   License, or (at your option) any later version.                       *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Library General Public     *
*   License along with this program; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <iostream>
#include <cstdlib>
#include <exception>
#include <jackal/xmlrpc.h>
#include <rfc2822/http.h>
#include <socket/socket.h>
#include <socket/inet.h>
#include <boost/thread.hpp>
#include <boost/bind.hpp>


#include "../dns/dns.h"
using namespace std;

static int max_clients = 5;
static boost::mutex done_m;
static boost::condition done_c;
static int client_count = 0;
static int req_count = 0;
//static boost::mutex total_guard;
  
void handle_server(wool::socket * server) {
  wool::iostream<wool::socket> cs( *server );
  //server->set_timeout(5);
  cs.exceptions(std::ios::failbit|std::ios::eofbit); 
  int i = 0;
  try {
		
 	for( ; i<25000; i++) {
		wool::http::request req;
		wool::http::response res;
		wool::swappable_data & d = req.body;
		req.header.insert(wool::http::header::value_type("Connection", "keep-alive"));
		req.header.insert(wool::http::header::value_type("Keep-alive", "300"));
		req.header.insert(wool::http::header::value_type("Host", "www.google.de"));
                req.header.insert(wool::http::header::value_type("Content-Length", "0"));

		req.method = "GET";
		req.path = "/";
		req.protocol = "HTTP/1.0";

		cs << req;                
		cs >> res;	
		//std::cout << res <<std::endl;
		WOOL_DEBUG("read res " <<res<< std::endl);
 	} 
  } catch ( const std::exception & e ) {
      WOOL_DEBUG("error in session ");
      std::cerr << "Error " << e.what() << std::endl;
  }

  delete server;
  
  {
	boost::mutex::scoped_lock l(done_m);	
	client_count++;
	req_count+=i;
	if(client_count>=max_clients) {
		//std::cout << "Total requests (client): "<<i << std::endl;
		done_c.notify_all();
	}
  }
}

void handle_client(wool::socket * client) {
  wool::iostream<wool::socket> cs( *client );
  cs.exceptions(std::ios::failbit|std::ios::eofbit); 
  //client->set_timeout(5);
  int i;

  try {
 	while(true) {
		wool::http::request req;
		wool::http::response res;
		cs >> req;
		res.body.append("Hello World!!!");
		res.header.insert(wool::http::header::value_type("Content-type", "text/plain"));
		res.header.insert(wool::http::header::value_type("Content-length", "14"));
		res.header.insert(wool::http::header::value_type("Connection", "keep-alive"));
		res.header.insert(wool::http::header::value_type("Keep-alive", "300"));

		res.message = "OK";
		res.status_code = 200;
		res.protocol = "HTTP/1.0";
		cs << res << std::flush;
 	} 
  } catch ( const std::exception & e ) {
      WOOL_DEBUG("error in session ");
      std::cout << "Error " << e.what() << std::endl;
  } catch ( ... ) {
      WOOL_DEBUG("error in session ");
      std::cout << "Error " << std::endl;
  }

  delete client;
}

void connect() {
	wool::inet::address listen_address( wool::inet::Dns::Resolve("localhost"), 8080 );
        boost::mutex::scoped_lock ll(done_m);
	long time_start;
	long time_end;
	
	::time(&time_start);
	for(int i = 0; i < max_clients; i ++ ) {
		wool::socket * server_conn = new wool::socket(wool::inet::inet_stream);
		server_conn->connect(listen_address);
		delete new boost::thread(boost::bind(handle_server,server_conn));
	}
	done_c.wait(ll);
	::time(&time_end);
	std::cout << "Time neeses (s):   	"<<(time_end-time_start) << std::endl;
	std::cout << "Total requests (total):   "<<req_count << std::endl;
	std::cout << "Rate(req/sec): 		"<<(req_count/std::max<long>(1,time_end-time_start)) << std::endl;
	std::cout << "Rate per client(req/sec): "<<((req_count/max_clients)/std::max<long>(1,time_end-time_start)) << std::endl;
}

void serve() {
    wool::inet::address listen_address( "0.0.0.0",8080 );
    wool::socket server(wool::inet::inet_stream);
    
    server.bind( listen_address );
    server.listen( 300 );

	while(true) 
	{
		WOOL_DEBUG("Accept...");
		wool::socket * client_connection = new wool::socket(wool::inet::inet_stream); 
		try{
			server.accept( *client_connection );
			WOOL_DEBUG("Next client...");
			/** create client thread and delete it at once 
			  * so that the client would run in detached mode
			  */
			 boost::thread * t =new boost::thread(boost::bind(handle_client,client_connection));
			delete t;
			client_connection = NULL;
		} catch (...) {

		}
		delete client_connection;
		
	}
}

int main( int argc, char *argv[] ) {
   xml::init xml_parsers;
   xml_parsers.indent_output( false );
   xml_parsers.remove_whitespace( true );
 
   if(argc==1) {
     std::cerr << "Server" << std::endl;
     serve();
   } else {
     std::cerr << "Client" << std::endl;
     connect();
  } 
   return EXIT_SUCCESS;
}
