/***************************************************************************
 *   Copyright (C) 2005 by Vukasin Toroman                                 *
 *   vtoroman@astaro.de                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <jackal/xmlrpc.h>
#include <socket/socket.h>
#include <socket/inet.h>
#include <boost/bind.hpp>
#include <expat.h>
#include <cgicc/Cgicc.h>
#include <cgicc/CgiInput.h>

  	
typedef jackal::xmlrpc::Saver XmlRpcSaver;
typedef jackal::xmlrpc::Loader XmlRpcLoader;
using namespace jackal;

class JackalTestClient : public wool::iostream<wool::socket>{
public:
  wool::socket m_socket;

  JackalTestClient()
    :wool::iostream<wool::socket>(m_socket),
     m_socket(wool::inet::inet_stream) {}
  virtual ~JackalTestClient() {}
};


class JackalTest : public jackal::RequestHandler {
public:
  JACKAL_BEGIN_MAPPING
  JACKAL_MAP0("getTwo", __getTwo);
  JACKAL_MAP2("getProduct", __getProduct);
  JACKAL_MAP2("append", __append);
  JACKAL_END_MAPPING;
		
  DECLARE_JACKAL_PROXY0(getTwo, int);
  DECLARE_JACKAL_PROXY2(getProduct, int, int, int);
		
private:	
  inline std::string __append( std::string s1, std::string s2 ) const {
    return s1+s2;
  }
  inline int __getTwo() const {
    return 2;
  }
  inline int __getProduct(int a, int b) const {
    return a*b;
  }
};
	
class CgiInputAdapter : public cgicc::CgiInput {
public:
  CgiInputAdapter ( const http::Request & r ) :m_request(r) {
    m_input_stream.open(r.body);
    wool::http::header::const_iterator hi = r.header.find("Host");
    if(hi!=r.header.end())
      m_url = "http://" + hi->second + "/" + r.path;
    else
      m_url = "http://127.0.0.1/" + r.path;
  }
				
  virtual ~CgiInputAdapter() {}
				
  virtual size_t read (char *data, size_t length) {
    m_input_stream.read(data,length);
    return m_input_stream.gcount();
  }
				
  virtual std::string 	getenv (const char *varName) {
    std::string n(varName);
    if(n == "REQUEST_METHOD") 
      return m_request.method;
					
    if(n == "CONTENT_TYPE") {
      wool::http::header::const_iterator ct = m_request.header.find("Content-type");
      if(ct!=m_request.header.end())
	return ct->second;
      else
	return "text/plain";
    }
					
    if(n == "CONTENT_LENGTH") {
      std::ostringstream os;
      os << m_request.body.size();
      return os.str();
    }
					
    if(n == "SERVER_PROTOCOL") {
      return m_request.protocol;
    }
					
    if(n == "SERVER_PORT") {
      //return m_url.port;
    }
					
    if(n == "SERVER_NAME") {
      return m_url.host;
    }
					
    if(n == "PATH_INFO") {
      return m_request.path;
    }
					
    if(n == "PATH_TRANSLATED") {
      return m_url.get_path();
    }
					
    if(n == "QUERY_STRING") {
      return m_url.query;
    }
					
    return "";
  }
				
private:
  wool::swappable_data::istream m_input_stream;
  const http::Request & m_request;
  wool::url m_url;
};


class Cgi : public jackal::http::RequestHandler{
public:
  Cgi() {}
  virtual ~Cgi() {};
			
  virtual bool shouldHandle( const http::Request & req ) const {
    return true;
  }
			
  virtual void handle ( const jackal::http::Request & req, jackal::http::Response & res ) {
    CgiInputAdapter input(req);
    cgicc::Cgicc cgi(&input);
    const std::vector<cgicc::FormEntry> & elements = cgi.getElements();
    std::vector<cgicc::FormEntry>::const_iterator it = elements.begin();
				
    wool::swappable_data::ostream out;
    out.open(res.body);
								
    for(;it!=elements.end();it++) {
      out << "Form element '" << it->getName() <<"' = '"<<**it<<"'" << std::endl; 
    }
    out.close();
				
    std::ostringstream os;
    os << res.body.size();
    res.status_code = 200;
    res.protocol = "HTTP/1.0";
    res.message = "OK";

    res.header.insert("Content-length",os.str());
    res.header.insert("Content-type","text/plain");
    res.header.insert("Connection","keep-alive");
				
    std::cout << res << std::endl;
  }
			
};


int main( int argc, char *argv[] ) { 
  JackalTest test_object;
  jackal::xmlrpc::Adapter adapter(test_object);
  adapter.registerPath("/test");
  jackal::xmlrpc::Adapter adapter2(test_object);
  adapter2.registerPath("/test2");
	
	
  std::cout << "getTwo " << test_object.getTwo() << std::endl;
  jackal::http::FileServer desktopFiles("/home/vtoroman/Desktop");
  desktopFiles.registerPath("/Desktop");
	
  jackal::http::RequestQueue requestQueue;
	
  Cgi testCgi;
  testCgi.registerPath("/test.cgi");
	
  wool::inet::address listen_address;
  listen_address.set_ip(wool::inet::ip::any);
  listen_address.set_port(8080);
  wool::socket server(wool::inet::inet_stream);
  server.bind( listen_address );
  server.listen( 300 );
   
  while(true) {
    JackalTestClient * client = NULL;
    try {
      client = new JackalTestClient();
      server.accept(client->m_socket);
      requestQueue.add(client);
      client = NULL;
    } catch (const std::exception & exc) { 
      std::cerr << "accept error: '" << exc.what() << "'" << std::endl;
    }
    delete client;
  }


  return EXIT_SUCCESS;
}

