/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <test/test_content_type.h>

namespace wool
{
namespace test
{

void test_content_type::testEquality()
{
    wool::content_type comp("text/html");
    CPPUNIT_ASSERT( comp == m_type );
    wool::content_type comp2("application/octet_stream");
    CPPUNIT_ASSERT( !(comp2 == m_type) );
}

void test_content_type::testCount()
{
    wool::content_type comp1("*/*");
    wool::content_type comp2("*/html");
    wool::content_type comp3("text/*");
    wool::content_type comp4("text/html");

    CPPUNIT_ASSERT( m_type.count(comp1) == 0);
    CPPUNIT_ASSERT( m_type.count(comp2) == 0);
    CPPUNIT_ASSERT( m_type.count(comp3) == 0);
    CPPUNIT_ASSERT( m_type.count(comp4) == 1);

    CPPUNIT_ASSERT( comp1.count(m_type) == 1);
    CPPUNIT_ASSERT( comp2.count(m_type) == 1);
    CPPUNIT_ASSERT( comp3.count(m_type) == 1);
    CPPUNIT_ASSERT( comp4.count(m_type) == 1);

}

void test_content_type::testString()
{
    std::string comp("text/html");
    CPPUNIT_ASSERT( comp == std::string(m_type));
}

void test_content_type::testAssignment()
{
  wool::content_type ct_zip("application/zip");
  wool::content_type ct_stream("video/stream");
  ct_zip = ct_stream;
  CPPUNIT_ASSERT( ct_zip == ct_stream);
  CPPUNIT_ASSERT( !(ct_zip == m_type));
}

void test_content_type::testCopyConstructor()
{
  wool::content_type ct_zip("application/zip");
  wool::content_type ct_zip_zip(ct_zip);
  CPPUNIT_ASSERT( ct_zip == ct_zip_zip);
}

void test_content_type::testLowerOperator()
{
  wool::content_type ct_zip("application/zip");
  wool::content_type ct_stream("video/stream");
  CPPUNIT_ASSERT( ct_zip < ct_stream);
}

void test_content_type::testGreaterOperator()
{
  wool::content_type ct_zip("application/zip");
  wool::content_type ct_stream("video/stream");
  CPPUNIT_ASSERT( ct_stream > ct_zip);
}


};
};
