/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef WOOLCONTENT_TYPE_TEST_H
#define WOOLCONTENT_TYPE_TEST_H

#include "../rfc2822/content_type.h"

#include <cppunit/TestFixture.h>
#include <cppunit/extensions/HelperMacros.h>


namespace wool
{
namespace test
{

/**
 *  test for content_type class.
 *  @author Manfred Koch <mkoch@astaro.de> (C) Astaro AG 2005
 */
class test_content_type: public CppUnit::TestFixture
{

private:

    wool::content_type m_type;

public:

    inline test_content_type()
            :m_type("text/html"){
            std::cerr << "parsed: "<<m_type << std::endl;
    }


    inline ~test_content_type(){}

    /**
     *  initialize the test
     */
    inline void setUp()
    {
        SAVE_HEADER_INFO(wool::content_type);
    }
    /**
     *  shutdown the test
     */
    inline void tearDown()
    {}
    /**
     *  test if equal content types are recognised
     */
    void testEquality();
    /**
     *  test if a content type contains an other content type.<br>
     *  <b>Example:</b>
     * \code
     *  '*.*' contains all other content types.
     *  'text.*' contains e.g. 'text/html' and 'text/plain'.
     *  \endcode
     */
    void testCount();
    /**
     *  test the conversion into a std::string
     */
    void testString();
    /**
     *  test the assignment operator
     */
    void testAssignment();
    /**
     *  test the copy constructor
     */
     void testCopyConstructor();
     /**
      * test lower operator
      */
      void testLowerOperator();
      /**
       *  test greater operator
       */
       void testGreaterOperator();
      /**
        * creates the test suite for the content type test
        * @return CppUnit::Test
        */
      inline static CppUnit::Test *suite()
      {
        CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "test_content_type" );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_content_type>(
                                   "testEquality",
                                   &test_content_type::testEquality ) );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_content_type>(
                                   "testString",
                                   &test_content_type::testString ) );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_content_type>(
                                   "testCount",
                                   &test_content_type::testCount ) );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_content_type>(
                                   "testAssignment",
                                   &test_content_type::testAssignment ) );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_content_type>(
                                   "testCopyConstructor",
                                   &test_content_type::testCopyConstructor ) );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_content_type>(
                                   "testLowerOperator",
                                   &test_content_type::testLowerOperator ) );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_content_type>(
                                   "testGreaterOperator",
                                   &test_content_type::testGreaterOperator ) );
        return suiteOfTests;
    }
};
};
};

#endif

