/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <test/test_ip4.h>

namespace wool
{
typedef wool::inet::ip ip4;
namespace test
{

void test_ip4::testConstructors()
{
  wool::ip4 ip_first("192.168.6.8");
  wool::ip4 ip_second;

  ip_second[0] = 192;
  ip_second[1] = 168;
  ip_second[2] = 6;
  ip_second[3] = 8;

  CPPUNIT_ASSERT( ip_first == ip_second );
  CPPUNIT_ASSERT(wool::ip4("192.168.8") == wool::ip4("192.168.0.8"));
  CPPUNIT_ASSERT_THROW(wool::ip4 ip_exception("192,168,6,8"),wool::inet::Dns::exception);
  CPPUNIT_ASSERT_THROW(wool::ip4 ip_exception("hello"),wool::inet::Dns::exception);
}

void test_ip4::testCopyConstructorIP4()
{
  wool::ip4 ip_first("192.168.6.8");
  wool::ip4 ip_second(ip_first);
  CPPUNIT_ASSERT( ip_first == ip_second );
}

void test_ip4::testCopyConstructor_c_ip()
{
  wool::ip4 ip_first("192.168.6.8");
  wool::ip4 ip_second(ip_first);
  CPPUNIT_ASSERT( ip_first == ip_second );
}

void test_ip4::testLowerOperator()
{
  wool::ip4 ip_first("1.1.1.1");
  wool::ip4 ip_second("2.2.2.2");
  CPPUNIT_ASSERT( ip_first < ip_second );
}

void test_ip4::testStringOperator()
{
  wool::ip4 ip("192.168.6.8");
  std::string ip_string("192.168.6.8");
  CPPUNIT_ASSERT (ip_string.compare(ip) == 0);
}

void test_ip4::testDNS_resolve()
{
  wool::ip4 ip("192.168.6.8");
  ip=wool::inet::Dns::Resolve("localhost");
  CPPUNIT_ASSERT ( std::string(ip) == "127.0.0.1");
  CPPUNIT_ASSERT_THROW(ip=wool::inet::Dns::Resolve("hello world"),wool::inet::Dns::exception);
}



};
};
