/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef TEST_MESSAGE2822_H
#define TEST_MESSAGE2822_H

#include <cppunit/TestFixture.h>
#include <cppunit/extensions/HelperMacros.h>

#include "../base/wool_stl_includes.h"
#include "../rfc2822/message_rfc2822.h"



namespace wool
{
namespace test
{
  /**
  * test for message_rfc2822.
  * @author Manfred Koch <mkoch@astaro.de> (C) Astaro AG 2005
  */
  class test_message2822 : public CppUnit::TestFixture
  {
  public:
      inline test_message2822(){}
      inline ~test_message2822() throw () {}


      /**
       *  initialize the test
       */
      inline void setUp(){}

      /**
       * shut down the test
       */
      inline void tearDown(){}

      /**
       * test if folding length = 78
       */
      void testFoldingLength();

      /**
       * test if max line length = 1024
       */
      void testMax_line_length();

      /**
       * test end of line character
       */
      void testMaxLenght();

      /**
       * test end of line character
       */
      void testEOL();

      /**
       * test white space character
       */
      void testWS();

      /**
       * test seperator character
       */
      void testSeparator();

      /**
       * test null character
       */
      void testNull();

      /**
       * test terminator character
       */
      void testTerminator();

      /**
       * parse a valid rfc2822 header
       */
      void testParse_RFC_conform_Header();

      /**
       * test if a header line without a field-name is ignored
       */
      void testEmptyFieldName();

      /**
       * test if a header line without a field-body is ignored
       */
      void testEmptyFieldBody();

      /**
       * test if an exception is thrown if a header line is too large
       */
      void testLine_length_Overflow();

      /**
       * test if an exception is thrown if the whole header is too large
       */
      void testMax_Header_Length();

      /**
       * test if there is more than one whitespace, tab or so, will compacted to one
       */
      void testCompactWhitespace();

      /**
       * test different forms of addresses
       */
      void testAddresses();

      /**
       * test a group of addresses
       */
      void testGroupAddresses();

      /**
       * test if non ASCII characters in the header field name are ignored
       */
      void testNonASCIICharacter();

      /**
       * creates the test suite for the message2822
       * @return CppUnit::Test
       */
      inline static CppUnit::Test *suite()
      {
          CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "test_message2822" );

          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testFoldingLength",
                                    &test_message2822::testFoldingLength ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testMax_line_length",
                                    &test_message2822::testMax_line_length ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testEOL",
                                    &test_message2822::testEOL ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testMaxLenght",
                                    &test_message2822::testMaxLenght ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testWS",
                                    &test_message2822::testWS ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testSeparator",
                                    &test_message2822::testSeparator ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testNull",
                                    &test_message2822::testNull ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testTerminator",
                                    &test_message2822::testTerminator ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testParse_RFC_conform_Header",
                                    &test_message2822::testParse_RFC_conform_Header ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testEmptyFieldName",
                                    &test_message2822::testEmptyFieldName ) );

          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testEmptyFieldBody",
                                    &test_message2822::testEmptyFieldBody ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testLine_length_Overflow",
                                    &test_message2822::testLine_length_Overflow ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testMax_Header_Length",
                                    &test_message2822::testMax_Header_Length ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testCompactWhitespace",
                                    &test_message2822::testCompactWhitespace ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testAddresses",
                                    &test_message2822::testAddresses ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testGroupAddresses",
                                    &test_message2822::testGroupAddresses ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_message2822>(
                                    "testNonASCIICharacter",
                                    &test_message2822::testNonASCIICharacter ) );
          return suiteOfTests;
      }

  };
  };
};
#endif
