/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <test/test_regexp.h>

namespace wool
{
namespace test
{

void test_regexp::testSimple()
{
    std::string expression ("Hello World");
    regex exp(expression,0);
    exp.study();
    std::list<std::string> result;
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "Hallo Welt");
    CPPUNIT_ASSERT( result.empty());
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "Hello World");
    CPPUNIT_ASSERT( !result.empty());

    std::ostringstream res;
    std::string expression2 ("lo Wo");
    std::string subject ("Helo lo World");
    regex exp2(expression2,0);

    exp2.replace_all(res,subject,"Wo");

    CPPUNIT_ASSERT(res.str()=="HeWorld");
}

void test_regexp::testDigit()
{
    std::string expression ("[0-9]");
    regex exp(expression,0);
    exp.study();
    std::list<std::string> result;
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "just a few characters");
    CPPUNIT_ASSERT( result.empty());
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "jhfe4kdfk");
    CPPUNIT_ASSERT( !result.empty());
}

void test_regexp::testMantis1826()
{
    std::string expression ("[^?]bagrat\\@ninny\\.avp\\.ru");
    regex exp(expression,0);
    exp.study();
    std::list<std::string> result;
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "anyone@www.com");
    CPPUNIT_ASSERT( result.empty());
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "lalala@bagrat@ninny.avp.ru");
    CPPUNIT_ASSERT( !result.empty());
}

void test_regexp::testMantis1242()
{
    std::string expression ("Augen auf\\! \\(So sieht es aus\\!\\)");
    regex exp(expression,0);
    exp.study();
    std::list<std::string> result;
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "Hello World");
    CPPUNIT_ASSERT( result.empty());
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "Augen auf! (So sieht es aus!)");
    CPPUNIT_ASSERT( !result.empty());
}

void test_regexp::testHTML_img()
{
    std::string expression ("(?i)\\<img  [^\\>]*\\>");
    regex exp(expression,0);
    exp.study();
    std::list<std::string> result;
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "Hello World");
    CPPUNIT_ASSERT( result.empty());
    exp.match( std::insert_iterator< std::list<std::string> > (result,result.begin()), "<td height=\"75\" rowspan=\"3\" valign=\"bottom\"><a href=\"/\"><img border=\"0\" alt=\"Firewall Internet Security Software\" src=\"/design/en/images/astaro-logo-01.gif\"></a></td>");
    CPPUNIT_ASSERT( !result.empty());
}



};
};


