/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef TEST_REGEXP_H
#define TEST_REGEXP_H

#include "../regex/pcre.h"


#include <cppunit/TestFixture.h>
#include <cppunit/extensions/HelperMacros.h>


namespace wool
{
namespace test
{
/**
 *  test for pcre (Perl Compatible Regular Expressions) class.
 *  @author Manfred Koch <mkoch@astaro.de> (C) Astaro AG 2005
 */
class test_regexp : public CppUnit::TestFixture
{

public:

    /**
     *  PCRE
     */
    typedef wool::pcre<char> regex;


    inline test_regexp()
    {}

    inline ~test_regexp()
    {}

    /**
     *  initialize the test
     */
    inline void setUp()
    {
        SAVE_HEADER_INFO(wool::pcre<char>);
    }

    /**
     *  shutdown the test
     */
    inline void tearDown()
    {}

    /**
     * "Hello World" test
     */
    void testSimple();
    /**
     *  test if numbers will be found in the regular expression
     */
    void testDigit();
    /**
     *  test Mantis 1826
     */
    void testMantis1826();
    /**
     *  test MAntis 1242
     */
    void testMantis1242();
    /**
     *  test for regular expression to find image tags in html code
     */
    void testHTML_img();
    /**
     * creates the test suite for the regexp
     * @return CppUnit::Test
     */
    inline static CppUnit::Test *suite()
    {
        CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "test_regexp" );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_regexp>(
                                   "testSimple",
                                   &test_regexp::testSimple ) );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_regexp>(
                                   "testMantis1826",
                                   &test_regexp::testMantis1826 ) );
        suiteOfTests->addTest( new CppUnit::TestCaller<test_regexp>(
                                   "testMantis1242",
                                   &test_regexp::testMantis1242 ) );

        return suiteOfTests;
    }

};
};

};


#endif
