/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#include "socket/socket.h"
#include "socket/inet.h"
//#include <wool/network.h>
#include "rfc2822/message_rfc2822.h"
#include "swool/streambuf_bio.h"
#include "swool/ssl_filter.h"

int main (int argc, char ** argv) {

    try {
        wool::swool::ssl_context::init();
        //wool::swool::ssl_context::set_default_password("pppp");

        wool::swool::ssl_context ssl_ctx (wool::swool::ssl_server::ssl23());
        ssl_ctx.set_password("pppp");
        ssl_ctx.load_cert_chain("/tmp/CERTIFICATE_pop3server.pem");
        ssl_ctx.load_pam_key("/tmp/KEY_pop3server.pem");
        ssl_ctx.load_ca_location("/tmp/CERT_pop3CA.pem");
        ssl_ctx.load_dh_params("/tmp/dh1024.pem");

        wool::socket server_socket(wool::socket::inet, wool::socket::stream_socket, wool::socket::unspec);
        wool::inet::address listen_addr;
        listen_addr.set_ip(wool::inet::ip::loopback);
        listen_addr.set_port(4433);
        
        server_socket.bind(listen_addr);
        server_socket.listen(300);

        while(true) {
            std::cerr << "wait for a client... " << std::endl;
            wool::socket client_socket(wool::socket::inet, wool::socket::stream_socket, wool::socket::unspec);
            server_socket.accept(client_socket);

            wool::streambuf<wool::socket> client_buffer(client_socket);
            wool::swool::streambuf_bio client_bio_buffer(&client_buffer);
            wool::swool::ssl_server ssl_client (ssl_ctx);
            wool::swool::bio_stream client_stream(ssl_client);

            try {
                //ssl_client.set_verify(wool::swool::ssl_context::verify_require_peer_cert|wool::swool::ssl_context::verify_peer);
                ssl_client.set_verify(wool::swool::ssl_context::verify_none);
                ssl_client.push(client_bio_buffer);
                ssl_client.do_handshake();

                std::string client_line;

                while((std::getline(client_stream, client_line)) && (client_line!="") && (client_line!="\r"));

                client_stream << "HTTP/1.0 200 OK"<< std::endl;
                client_stream << "Connection: close" << std::endl;
                client_stream << "Content-type: text/plain" << std::endl;
                client_stream << std::endl;
                client_stream << "it worked!!!" << std::endl << std::flush;

            }catch (const std::exception & c_e) {
                std::cerr << "client error: "<< c_e.what() << std::endl;
            }
            std::cerr << "client session finished... " << std::endl;
        }
    } catch (const std::exception & e) {
        std::cerr << e.what() << std::endl;
    }
    return 1;
}
