/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <test/test_swappable_data.h>

namespace wool
{
namespace test
{

void test_swappable_data::testEmpty()
{
  WOOL_DEBUG("start test");
  wool::swappable_data swap;
  CPPUNIT_ASSERT(swap.size() == 0);
}

void test_swappable_data::testNoSwap()
{
  WOOL_DEBUG("start test");
  wool::swappable_data swap;
  wool::tempfile f_tmp;
  std::ofstream out (f_tmp.path().c_str());
  for (int i = 0; i < (swappable_data::resident_size-1 ); i++)
    out << "A" ;
  out.close();
  std::ifstream in(f_tmp.path().c_str());
  in >> swap;
  CPPUNIT_ASSERT(swap.size() == swappable_data::resident_size-1 );
  CPPUNIT_ASSERT(swap.swap_file() == "");
}

void test_swappable_data::testSwap()
{
  WOOL_DEBUG("start test");
  wool::swappable_data swap;
  wool::swappable_data swap2;
  wool::tempfile f_tmp;
  std::ofstream out (f_tmp.path().c_str());
  unsigned long wanted_length = 8 * swappable_data::resident_size+1024;
  wool::swappable_data::ostream os;


  for (int i = 0; i <wanted_length; i++)
  {
          out<<char(i%256);
  }

  out<<std::flush;
  out.close();


  std::ifstream in(f_tmp.path().c_str());
  in >> swap;

  WOOL_DEBUG("length swap: "<<swap.size()<<" == "<<wanted_length);
  //exit(1);

  std::ifstream is;
  is.open(swap.swap_file().c_str(),std::ios_base::in);
  char buffer[1024];
  long total = 0;
  while(is.good() && !is.eof() )
  {
    is.read(buffer,1024);
    //WOOL_DEBUG("WRITE "<<is.gcount()<<" bytes");
    total+=is.gcount();
  }

  CPPUNIT_ASSERT(total == wanted_length);

  swap2 = swap;

  WOOL_DEBUG("length swap2: "<<swap2.size()<<" == "<<wanted_length);

  CPPUNIT_ASSERT(swap.size() == wanted_length);
  CPPUNIT_ASSERT(swap2.size() == wanted_length);

  CPPUNIT_ASSERT(swap.swap_file() != "");
  CPPUNIT_ASSERT(swap2.swap_file() != "");

  FILE * pFile;
  pFile = fopen (swap.swap_file().c_str(),"r");
  CPPUNIT_ASSERT (pFile!=NULL);
  if (pFile!=NULL)
  {
    long lSize;
    char * buffer;
    fseek (pFile , 0 , SEEK_END);
    lSize = ftell (pFile);
    rewind (pFile);
    buffer = (char*) malloc (lSize);
    fread (buffer,1,lSize,pFile);
    CPPUNIT_ASSERT(lSize == wanted_length );
    free (buffer);
    fclose (pFile);
  }

}

void test_swappable_data::testSwapOut()
{
  WOOL_DEBUG("start test");
  wool::swappable_data swap;
  wool::tempfile f_tmp;
  std::ofstream out (f_tmp.path().c_str());
  for (int i = 0; i < (wool::swappable_data::resident_size ); i++)
    out << "A" ;
  out.close();
  std::ifstream in(f_tmp.path().c_str());
  in >> swap;
  CPPUNIT_ASSERT(swap.size() == wool::swappable_data::resident_size );
  CPPUNIT_ASSERT(swap.swap_file() == "");

  swap.swap_out();
  CPPUNIT_ASSERT(swap.swap_file() != "");
  FILE * pFile;
  pFile = fopen (swap.swap_file().c_str(),"r");
  CPPUNIT_ASSERT (pFile!=NULL);
  if (pFile!=NULL)
  {
    long lSize;
    char * buffer;
    fseek (pFile , 0 , SEEK_END);
    lSize = ftell (pFile);
    rewind (pFile);
    buffer = (char*) malloc (lSize);
    fread (buffer,1,lSize,pFile);
    CPPUNIT_ASSERT(lSize == wool::swappable_data::resident_size);
    free (buffer);
    fclose (pFile);
  }
}

void test_swappable_data::testAppend()
{
  WOOL_DEBUG("start test");
  wool::swappable_data swap;
  wool::swappable_data swap2;
  wool::swappable_data::ostream os;
  wool::tempfile f_tmp;
  std::ofstream out (f_tmp.path().c_str());
  for (int i = -127; i < 0; i++)
    out << (char)(i) ;
  for (int i = 0; i < 1897; i++)
    out << (char)(i%256) ;
  out.close();
  std::ifstream in(f_tmp.path().c_str());
  in >> swap;
  CPPUNIT_ASSERT(swap.size() == 2024 );
  CPPUNIT_ASSERT(swap.swap_file() == "");

  os.open(swap);
  std::string s_append = "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB";
  os<<s_append<<std::flush;
  CPPUNIT_ASSERT(swap.size() == 2024 + s_append.length());
  os.close();

  CPPUNIT_ASSERT(swap2.size()==0);
  swap2 = swap;
  WOOL_DEBUG("swap2 size "<<swap2.size());
  CPPUNIT_ASSERT(swap2.size() == 2024 + s_append.length());
  CPPUNIT_ASSERT(swap.size() == 2024 + s_append.length());
}

void test_swappable_data::testClear()
{
  WOOL_DEBUG("start test");
  wool::swappable_data swap;
  wool::tempfile f_tmp;
  std::ofstream out (f_tmp.path().c_str());
  for (int i = 0; i < (1024); i++)
    out << "A" ;
  out.close();
  std::ifstream in(f_tmp.path().c_str());
  in >> swap;
  swap.swap_out();
  CPPUNIT_ASSERT(swap.size() == 1024 );
  CPPUNIT_ASSERT(swap.swap_file() != "");

  swap.clear();
  CPPUNIT_ASSERT(swap.size() == 0 );
  CPPUNIT_ASSERT(swap.swap_file() == "");

}

};
};


