/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef TEST_SWAPPABLE_DATA_H
#define TEST_SWAPPABLE_DATA_H

#include <cppunit/TestFixture.h>
#include <cppunit/extensions/HelperMacros.h>

#include "../base/wool_stl_includes.h"
#include "../storage/swappable_data.h"
#include "../storage/tmp_file.h"
#include "../base/cvs_info.h"



namespace wool
{
namespace test
{
/**
 * test for the swappable data class.
 * @author Manfred Koch <mkoch@astaro.de> (C) Astaro AG 2005
 */
  class test_swappable_data : public CppUnit::TestFixture
  {
  public:
      inline test_swappable_data(){}
      inline ~test_swappable_data(){}


      /**
       * initial the test
       * prints out cvs info of the swappable_data class
       */
      inline void setUp()
      {
        SAVE_HEADER_INFO(wool::swappable_data);
        SAVE_IMPL_INFO(wool::swappable_data);
      }

      /**
       *  shutdown the test
       */
      inline void tearDown(){}

      /**
       * test if a new swappable_data is empty
       */
      void testEmpty();

      /**
       * test if data is hold in memory and not swap out when data.size is under swappable_data::resident_size
       */
      void testNoSwap();

      /**
       * test if data is swap out when data.size is over swappable_data::resident_size
       */
       void testSwap();

      /**
       * Test explicit swap out, when data.size() is under swappable_data::resident_size
       */
      void testSwapOut();


      /**
       * test if the correct string is append to the swappable_data
       */
      void testAppend();


      /**
       * test to delete the swappable data in memory and in the tempfile
       */
      void testClear();

      /**
       * creates the test suite for the swappable_data
       * @return CppUnit::Test
       */
      inline static CppUnit::Test *suite()
      {
          CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "test_swappable_data" );
//*
          suiteOfTests->addTest( new CppUnit::TestCaller<test_swappable_data>(
                                    "testEmpty",
                                    &test_swappable_data::testEmpty ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_swappable_data>(
                                    "testSwap",
                                    &test_swappable_data::testSwap ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_swappable_data>(
                                    "testNoSwap",
                                    &test_swappable_data::testNoSwap ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_swappable_data>(
                                    "testSwapOut",
                                    &test_swappable_data::testSwapOut ) );
//*/
          suiteOfTests->addTest( new CppUnit::TestCaller<test_swappable_data>(
                                    "testAppend",
                                    &test_swappable_data::testAppend ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_swappable_data>(
                                    "testClear",
                                    &test_swappable_data::testClear ) );
          return suiteOfTests;
      }

  };
};

};


#endif
