/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <test/test_tmpfile.h>

namespace wool
{
namespace test
{

void test_tmpfile::testEmpty()
{
  wool::tempfile testfile(true);
  CPPUNIT_ASSERT(testfile.size()==0);
  m_name = testfile.path();
}

void test_tmpfile::testAutoClean()
{
  FILE * pFile;
  {
    wool::tempfile testfile(true);
    m_name = testfile.path();

    pFile = fopen (m_name.c_str(),"r");
    CPPUNIT_ASSERT (pFile!=NULL);
    fclose(pFile);
  }
  pFile = fopen (m_name.c_str(),"r");
  CPPUNIT_ASSERT (pFile==NULL);
}

void test_tmpfile::testWriteRead()
{
  FILE * pFile;
  wool::tempfile testfile(true);
  m_name = testfile.path();
  pFile = fopen (m_name.c_str(),"r+");
  CPPUNIT_ASSERT (pFile!=NULL);
  if (pFile!=NULL)
  {
    long lSize;
    char * buffer;
    fputs ("fopen example",pFile);
    fseek (pFile , 0 , SEEK_END);
    lSize = ftell (pFile);
    rewind (pFile);
    buffer = (char*) malloc (lSize);
    fread (buffer,1,lSize,pFile);
    CPPUNIT_ASSERT(lSize == strlen("fopen example"));
    CPPUNIT_ASSERT(testfile.size() == strlen("fopen example"));
    free (buffer);
    fclose (pFile);
  }

}

void test_tmpfile::testManualClean()
{
  FILE * pFile;
  {
    wool::tempfile testfile(false);
    m_name = testfile.path();
  }
  pFile = fopen (m_name.c_str(),"r");
  CPPUNIT_ASSERT (pFile!=NULL);
  if(pFile!=NULL)
    fclose(pFile);
  ::unlink(m_name.c_str());
  pFile = fopen (m_name.c_str(),"r");
  CPPUNIT_ASSERT (pFile==NULL);
}


};
};

