/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef TEST_TMFILE_H
#define TEST_TMFILE_H

#include <cppunit/TestFixture.h>
#include <cppunit/extensions/HelperMacros.h>

#include "../base/wool_stl_includes.h"
#include "../storage/tmp_file.h"



namespace wool
{
namespace test
{
/**
 * test for the tempfile class.
 * @author Manfred Koch <mkoch@astaro.de> (C) Astaro AG 2005
 */

  class test_tmpfile : public CppUnit::TestFixture
  {

  private:
      std::string m_name;

  public:
      inline test_tmpfile(){}
      inline ~test_tmpfile(){}


      /**
       * initialise the test
       * prints out cvs info of the tempfile class
       */
      inline void setUp()
      {
        SAVE_HEADER_INFO(wool::tempfile);
      }

      /**
       *  shutdown test
       */
      inline void tearDown(){}

      /**
       * test if a new tempfile is empty
       */
      void testEmpty();

      /**
       * test automatically deletion when the tempfile is out of focus
       */
      void testAutoClean();

      /**
       * Read and write test  with a tempfile
       */
      void testWriteRead();

      /**
       * test manually deletion
       */
      void testManualClean();


      /**
       * creates the test suite for the tmpfile
       * @return CppUnit::Test
       */
      inline static CppUnit::Test *suite()
      {
          CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "test_tmpfile" );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_tmpfile>(
                                    "testEmpty",
                                    &test_tmpfile::testEmpty ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_tmpfile>(
                                    "testAutoClean",
                                    &test_tmpfile::testAutoClean ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_tmpfile>(
                                    "testWriteRead",
                                    &test_tmpfile::testWriteRead ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_tmpfile>(
                                    "testManualClean",
                                    &test_tmpfile::testManualClean ) );

          return suiteOfTests;
      }

  };
  };
};
#endif
