/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <test/test_unsorted_multimap.h>

namespace wool
{
namespace test
{

void test_unsorted_multimap::testEmpty()
{
  multi_map map;
  CPPUNIT_ASSERT(map.size() == 0);
  CPPUNIT_ASSERT(map.empty());
  multi_map::const_iterator it = map.begin();
  CPPUNIT_ASSERT(std::distance<multi_map::const_iterator>(map.begin(), it) == 0);
  it = map.end();
  CPPUNIT_ASSERT(std::distance<multi_map::const_iterator>(map.begin(), it) == 0);
  multi_map::const_reverse_iterator rit = map.rbegin();
  CPPUNIT_ASSERT(std::distance<multi_map::const_iterator>(map.begin(), it) == 0);
  rit = map.rend();
  CPPUNIT_ASSERT(std::distance<multi_map::const_iterator>(map.begin(), it) == 0);

}

void test_unsorted_multimap::testInsert()
{
  multi_map map;
  multi_map::const_iterator it;
  it = map.insert(multi_map::value_type("Test", "1"));
  CPPUNIT_ASSERT(map.size()== 1);
  CPPUNIT_ASSERT(!map.empty());
  CPPUNIT_ASSERT(it->first == "Test");
  CPPUNIT_ASSERT(it->second == "1");
  it = map.begin();
  CPPUNIT_ASSERT(std::distance<multi_map::const_iterator>(map.begin(), it) == 0);
  CPPUNIT_ASSERT(it->first == "Test");
  CPPUNIT_ASSERT(it->second == "1");

  it = map.insert(map.begin(),multi_map::value_type("Hello", "2"));
  CPPUNIT_ASSERT(map.size()== 2);
  CPPUNIT_ASSERT(it->first == "Hello");
  CPPUNIT_ASSERT(it->second == "2");
  it = map.begin();
  CPPUNIT_ASSERT(it->first == "Hello");
  CPPUNIT_ASSERT(it->second == "2");

  it = map.insert(multi_map::value_type("World", "3"));
  CPPUNIT_ASSERT(map.size()== 3);
  CPPUNIT_ASSERT(it->first == "World");
  CPPUNIT_ASSERT(it->second == "3");

  for(it = map.begin();it  != map.end(); ++it)
  {
    if (std::distance<multi_map::const_iterator>(map.begin(), it) == 0)
    {
      CPPUNIT_ASSERT(it->first == "Hello");
      CPPUNIT_ASSERT(it->second == "2");
    }
    if (std::distance<multi_map::const_iterator>(map.begin(), it) == 1)
    {
      CPPUNIT_ASSERT(it->first == "Test");
      CPPUNIT_ASSERT(it->second == "1");
    }
    if (std::distance<multi_map::const_iterator>(map.begin(), it) == 2)
    {
      CPPUNIT_ASSERT(it->first == "World");
      CPPUNIT_ASSERT(it->second == "3");
    }
  }
}

void test_unsorted_multimap::testOrder()
{
  multi_map map;
  map.insert(multi_map::value_type("Test", "1"));
  map.insert(multi_map::value_type("Test", "2"));
  map.insert(multi_map::value_type("Test", "3"));
  map.insert(multi_map::value_type("Test", "4"));
  map.insert(multi_map::value_type("Test", "5"));
  map.insert(multi_map::value_type("Hello", "6"));
  map.insert(multi_map::value_type("Hello", "7"));
  map.insert(multi_map::value_type("Hello", "8"));
  map.insert(multi_map::value_type("Hello", "9"));
  map.insert(multi_map::value_type("Hello", "10"));
  map.insert(multi_map::value_type("World", "11"));
  map.insert(multi_map::value_type("World", "12"));
  map.insert(multi_map::value_type("World", "13"));

  for(multi_map::const_iterator it = map.begin();it  != map.end(); ++it)
  {
    if (std::distance<multi_map::const_iterator>(map.begin(), it) < 5)
    {
        CPPUNIT_ASSERT(it->first == "Test");
        if (std::distance<multi_map::const_iterator>(map.begin(), it) == 0)
          CPPUNIT_ASSERT(it->second == "1");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 1)
          CPPUNIT_ASSERT(it->second == "2");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 2)
          CPPUNIT_ASSERT(it->second == "3");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 3)
          CPPUNIT_ASSERT(it->second == "4");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 4)
          CPPUNIT_ASSERT(it->second == "5");
    }
    else if (std::distance<multi_map::const_iterator>(map.begin(), it) >= 5 && std::distance<multi_map::const_iterator>(map.begin(), it) < 10)
    {
        CPPUNIT_ASSERT(it->first == "Hello");
        if (std::distance<multi_map::const_iterator>(map.begin(), it) == 5)
          CPPUNIT_ASSERT(it->second == "6");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 6)
          CPPUNIT_ASSERT(it->second == "7");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 7)
          CPPUNIT_ASSERT(it->second == "8");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 8)
          CPPUNIT_ASSERT(it->second == "9");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 9)
          CPPUNIT_ASSERT(it->second == "10");
    }
    else
    {
        CPPUNIT_ASSERT(it->first == "World");
        if (std::distance<multi_map::const_iterator>(map.begin(), it) == 10)
          CPPUNIT_ASSERT(it->second == "11");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 11)
          CPPUNIT_ASSERT(it->second == "12");
        else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 12)
          CPPUNIT_ASSERT(it->second == "13");
    }
    }
}

void test_unsorted_multimap::testErase()
{
  multi_map map;
  map.insert(multi_map::value_type("Test", "1"));
  map.insert(multi_map::value_type("Test", "2"));
  map.insert(multi_map::value_type("Test", "3"));
  map.insert(multi_map::value_type("Test", "4"));
  map.insert(multi_map::value_type("Test", "5"));
  map.insert(multi_map::value_type("Hello", "6"));
  map.insert(multi_map::value_type("Hello", "7"));
  map.insert(multi_map::value_type("Hello", "8"));
  map.insert(multi_map::value_type("Hello", "9"));
  map.insert(multi_map::value_type("Hello", "10"));
  map.insert(multi_map::value_type("World", "11"));
  map.insert(multi_map::value_type("World", "12"));
  map.insert(multi_map::value_type("World", "13"));

  CPPUNIT_ASSERT(map.size() == 13);

  multi_map::const_iterator it;
  it = map.erase(map.begin());
//  CPPUNIT_ASSERT(it != NULL);
  CPPUNIT_ASSERT(map.size() == 12);
  CPPUNIT_ASSERT(it->first == "Test");
  CPPUNIT_ASSERT(it->second == "2");

  CPPUNIT_ASSERT(map.erase("Hello") == 5);
  CPPUNIT_ASSERT(map.size() == 7);
  it = map.find("Hello");
  CPPUNIT_ASSERT(it == map.end());

  it = map.erase(map.begin(), map.find("World"));
  CPPUNIT_ASSERT(it->first == "World");
  CPPUNIT_ASSERT(it->second == "11");
  it = map.find("Test");
  CPPUNIT_ASSERT(it == map.end());
  CPPUNIT_ASSERT(map.size() == 3);



  map.clear();
  CPPUNIT_ASSERT(map.size() == 0);
  CPPUNIT_ASSERT(map.empty());

}

void test_unsorted_multimap::testFind()
{
  multi_map map;
  map.insert(multi_map::value_type("Test", "1"));
  map.insert(multi_map::value_type("Test", "2"));
  map.insert(multi_map::value_type("Test", "3"));
  map.insert(multi_map::value_type("Test", "4"));
  map.insert(multi_map::value_type("Test", "5"));
  map.insert(multi_map::value_type("Hello", "6"));
  map.insert(multi_map::value_type("Hello", "7"));
  map.insert(multi_map::value_type("Hello", "8"));
  map.insert(multi_map::value_type("Hello", "9"));
  map.insert(multi_map::value_type("Hello", "10"));
  map.insert(multi_map::value_type("World", "11"));
  map.insert(multi_map::value_type("World", "12"));
  map.insert(multi_map::value_type("World", "13"));

  multi_map::const_iterator it = map.find("World");
  CPPUNIT_ASSERT(std::distance<multi_map::const_iterator>(map.begin(), it) == 10);

  it = map.find("Test");
  CPPUNIT_ASSERT(std::distance<multi_map::const_iterator>(map.begin(), it) == 0);

  it = map.find("Hello");
  CPPUNIT_ASSERT(std::distance<multi_map::const_iterator>(map.begin(), it) == 5);

  it = map.find("Hello", map.find("World"));
  CPPUNIT_ASSERT(it == map.end());
}

void test_unsorted_multimap::testReverseOrder()
{
  multi_map map;
  map.insert(multi_map::value_type("Test", "1"));
  map.insert(multi_map::value_type("Test", "2"));
  map.insert(multi_map::value_type("Test", "3"));
  map.insert(multi_map::value_type("Test", "4"));
  map.insert(multi_map::value_type("Test", "5"));
  map.insert(multi_map::value_type("Hello", "6"));
  map.insert(multi_map::value_type("Hello", "7"));
  map.insert(multi_map::value_type("Hello", "8"));
  map.insert(multi_map::value_type("Hello", "9"));
  map.insert(multi_map::value_type("Hello", "10"));
  map.insert(multi_map::value_type("World", "11"));
  map.insert(multi_map::value_type("World", "12"));
  map.insert(multi_map::value_type("World", "13"));

  multi_map::reverse_iterator rit = map.rbegin();
  multi_map::reverse_iterator rit_end = map.rend();
  for (; rit != rit_end; rit++)
  {
    if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) < 3)
    {
        CPPUNIT_ASSERT(rit->first == "World");
        if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 0)
          CPPUNIT_ASSERT(rit->second == "13");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 1)
          CPPUNIT_ASSERT(rit->second == "12");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 2)
          CPPUNIT_ASSERT(rit->second == "11");

    }
    else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) >= 3 && std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) < 8)
    {
        CPPUNIT_ASSERT(rit->first == "Hello");
        if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 3)
          CPPUNIT_ASSERT(rit->second == "10");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 4)
          CPPUNIT_ASSERT(rit->second == "9");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 5)
          CPPUNIT_ASSERT(rit->second == "8");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 6)
          CPPUNIT_ASSERT(rit->second == "7");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 7)
          CPPUNIT_ASSERT(rit->second == "6");
    }
    else
    {
        CPPUNIT_ASSERT(rit->first == "Test");
        if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 8)
          CPPUNIT_ASSERT(rit->second == "5");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 9)
          CPPUNIT_ASSERT(rit->second == "4");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 10)
          CPPUNIT_ASSERT(rit->second == "3");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 11)
          CPPUNIT_ASSERT(rit->second == "2");
        else if (std::distance<multi_map::reverse_iterator>(map.rbegin(), rit) == 12)
          CPPUNIT_ASSERT(rit->second == "1");
    }
  }

}

void test_unsorted_multimap::testCount()
{
  multi_map map;
  map.insert(multi_map::value_type("Test", "1"));
  map.insert(multi_map::value_type("Test", "2"));
  map.insert(multi_map::value_type("Test", "3"));
  map.insert(multi_map::value_type("Test", "4"));
  map.insert(multi_map::value_type("Test", "5"));
  map.insert(multi_map::value_type("Hello", "6"));
  map.insert(multi_map::value_type("Hello", "7"));
  map.insert(multi_map::value_type("Hello", "8"));
  map.insert(multi_map::value_type("Hello", "9"));
  map.insert(multi_map::value_type("Hello", "10"));
  map.insert(multi_map::value_type("World", "11"));
  map.insert(multi_map::value_type("World", "12"));
  map.insert(multi_map::value_type("World", "13"));

  CPPUNIT_ASSERT(map.count("Test") == 5);
  CPPUNIT_ASSERT(map.count("Hello") == 5);
  CPPUNIT_ASSERT(map.count("World") == 3);
}

void test_unsorted_multimap::testOperatorEqual()
{
  multi_map map;
  map.insert(multi_map::value_type("Test", "1"));
  map.insert(multi_map::value_type("Hello", "6"));
  map.insert(multi_map::value_type("World", "12"));

  multi_map map2;
  map2.insert(multi_map::value_type("Monday", "22"));
  map2.insert(multi_map::value_type("Thuesday", "23"));
  map2.insert(multi_map::value_type("Saturday", "27"));

  CPPUNIT_ASSERT(map == map);
  CPPUNIT_ASSERT(!(map == map2));
}

void test_unsorted_multimap::testOperatorLower()
{
  multi_map map;
  map.insert(multi_map::value_type("Test", "1"));
  map.insert(multi_map::value_type("Hello", "6"));
  map.insert(multi_map::value_type("World", "12"));

  multi_map map2;
  map2.insert(multi_map::value_type("Monday", "22"));
  map2.insert(multi_map::value_type("Thuesday", "23"));
  map2.insert(multi_map::value_type("Saturday", "27"));

  CPPUNIT_ASSERT(!(map < map));
  CPPUNIT_ASSERT( map2 < map );
}

void test_unsorted_multimap::testCopyConstructor()
{
  multi_map map;
  map.insert(multi_map::value_type("Test", "1"));
  map.insert(multi_map::value_type("Hello", "6"));
  map.insert(multi_map::value_type("World", "12"));

  CPPUNIT_ASSERT(map.size() == 3);
  multi_map::const_iterator it = map.begin();
  for (;it != map.end(); ++it)
  {
    if (std::distance<multi_map::const_iterator>(map.begin(), it) == 0)
    {
      CPPUNIT_ASSERT(it->first == "Test");
      CPPUNIT_ASSERT(it->second == "1");
    }
    else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 1)
    {
      CPPUNIT_ASSERT(it->first == "Hello");
      CPPUNIT_ASSERT(it->second == "6");
    }
    else if (std::distance<multi_map::const_iterator>(map.begin(), it) == 2)
    {
      CPPUNIT_ASSERT(it->first == "World");
      CPPUNIT_ASSERT(it->second == "12");
    }
  }

  multi_map map2(map);
  CPPUNIT_ASSERT(map2.size() == 3);
  it = map2.begin();
  for (;it != map2.end(); ++it)
  {
    if (std::distance<multi_map::const_iterator>(map2.begin(), it) == 0)
    {
      CPPUNIT_ASSERT(it->first == "Test");
      CPPUNIT_ASSERT(it->second == "1");
    }
    else if (std::distance<multi_map::const_iterator>(map2.begin(), it) == 1)
    {
      CPPUNIT_ASSERT(it->first == "Hello");
      CPPUNIT_ASSERT(it->second == "6");
    }
    else if (std::distance<multi_map::const_iterator>(map2.begin(), it) == 2)
    {
      CPPUNIT_ASSERT(it->first == "World");
      CPPUNIT_ASSERT(it->second == "12");
    }
  }

  CPPUNIT_ASSERT(map2 == map);
}



};
};



