/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef TEST_UNSORTED_MULTIMAP_H
#define TEST_UNSORTED_MULTIMAP_H

#include <cppunit/TestFixture.h>
#include <cppunit/extensions/HelperMacros.h>

#include "../base/wool_stl_includes.h"
#include "../base/debug.h"
#include "../base/unsorted_multimap.h"


namespace wool
{
/**
 *  This namespace contains all tests for wool and swool.
 * @author Manfred Koch <mkoch@astaro.de> (C) Astaro AG 2005
 */
namespace test
{
/**
 * This is a test for the unsorted_multimap class.
 * @author Manfred Koch
 */

  struct ncase_eqstr
  {
      /**
       *  compare method
       *  @param s1 first string
       *  @param s2 second string
       *  @return true if s1 is equal to s2
       */
      bool operator()(const std::string & s1, const std::string &  s2) const
      {
          return (strcasecmp(s1.c_str(), s2.c_str()) < 0);
      }
  };
  /**
  * test for the unsorted_multimap class.
  * @author Manfred Koch <mkoch@astaro.de> (C) Astaro AG 2005
  */
  class test_unsorted_multimap : public CppUnit::TestFixture
  {

  private:
    typedef std::string string;
    typedef wool::unsorted_multimap<string,string,ncase_eqstr> multi_map;
    typedef std::pair<string,string> value_type ;

  public:
      inline test_unsorted_multimap(){}
      inline ~test_unsorted_multimap(){}

      /**
       *  initialize the test
       */
      inline void setUp(){
        SAVE_HEADER_INFO(multi_map);
      }
      /**
       *  shutdown the test
       */
      inline void tearDown(){}

      /**
       * test if a new unsorted multimap is really empty.
       */
      void testEmpty();


      /**
       * inserts values into the multimap and test the correct order of the values.
       */
      void testInsert();

      /**
       * test of the iterator for the multimap
       */
      void testOrder();


      /**
       * test if the correct values where deleted when calling 'erase'
       */
      void testErase();

      /**
       * test the find - method of the multimap
       */
      void testFind();


      /**
       * test the reverse iterator of the multimap
       */
      void testReverseOrder();

      /**
       * test the count method of the multimap
       */
      void testCount();

      /**
       * test the equal operator of the multimap
       */
      void testOperatorEqual();

      /**
       * test the lower operator of the multimap
       */
      void testOperatorLower();

      /**
       * test the copy constructor of the multimap
       */
      void testCopyConstructor();

      /**
       * creates the test suite for the unsorted multimap
       * @return CppUnit::Test
       */
      inline static CppUnit::Test *suite()
      {
          CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "test_unsorted_multimap" );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testEmpty",
                                    &test_unsorted_multimap::testEmpty ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testInsert",
                                    &test_unsorted_multimap::testInsert ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testOrder",
                                    &test_unsorted_multimap::testOrder ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testErase",
                                    &test_unsorted_multimap::testErase ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testFind",
                                    &test_unsorted_multimap::testFind ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testReverseOrder",
                                    &test_unsorted_multimap::testReverseOrder ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testCount",
                                    &test_unsorted_multimap::testCount ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testOperatorEqual",
                                    &test_unsorted_multimap::testOperatorEqual ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testOperatorLower",
                                    &test_unsorted_multimap::testOperatorLower ) );
          suiteOfTests->addTest( new CppUnit::TestCaller<test_unsorted_multimap>(
                                    "testCopyConstructor",
                                    &test_unsorted_multimap::testCopyConstructor ) );


          return suiteOfTests;
      }

  };

};
};


#endif
