/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <test/test_username.h>

namespace wool {

namespace test {

void test_username::testConstructor()
{
  wool::user u("user","domain");
  CPPUNIT_ASSERT(u.name == "user");
  CPPUNIT_ASSERT(u.domain == "domain");

  wool::user x("user2@domain2");
  CPPUNIT_ASSERT(x.name == "user2");
  CPPUNIT_ASSERT(x.domain == "domain2");

  wool::user z("user3");
  CPPUNIT_ASSERT(z.name == "user3");
  CPPUNIT_ASSERT(z.domain == "");
}

void test_username::testString()
{
  wool::user u("user","domain");
  CPPUNIT_ASSERT(std::string(u) == "user@domain");
}

void test_username::testEqual()
{
  wool::user u("user","domain");
  wool::user x("user@domain");
  CPPUNIT_ASSERT(u == x);

  wool::user y("user","domain");
  CPPUNIT_ASSERT(u == y);

  wool::user z("user2","domain2");
  CPPUNIT_ASSERT(!(u == z));

  wool::user a("user3@");
  wool::user b("user3");
  CPPUNIT_ASSERT(a == b);
}

void test_username::testLower()
{
  wool::user u("user","domain");
  wool::user x("user2","domain");
  CPPUNIT_ASSERT(u < x);
  CPPUNIT_ASSERT(!(x < u));

  wool::user a("user2@");
  wool::user b("user3");
  CPPUNIT_ASSERT(a < b);
}

void test_username::testCount()
{
  wool::user u("user","domain");
  wool::user x("*","domain");
  CPPUNIT_ASSERT(x.count(u) == 1);
  CPPUNIT_ASSERT(u.count(x) == 0);

  wool::user a("user3@");
  wool::user b("user3");
  CPPUNIT_ASSERT(a.count(b) == 1);
  CPPUNIT_ASSERT(b.count(a) == 1);

  wool::user all("*","*");
  CPPUNIT_ASSERT(all.count(a) == 1);
  CPPUNIT_ASSERT(all.count(b) == 1);
  CPPUNIT_ASSERT(all.count(x) == 1);
  CPPUNIT_ASSERT(all.count(u) == 1);

}

};

};
