#include "../base/exception.h"
#include "../swool/name.hh"
#include "../swool/req.hh"
#include "../swool/cert.hh"
#include "../swool/pem.hh"
#include "../swool/ssl_filter.h"
#include "../swool/streambuf_bio.h"

#include "../socket/socket.h"
#include "../socket/inet.h"

#include <iostream>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/rand.h>

#define CA_KEY "/home/vtoroman/NSwO-1.3/ssl/serverCAkey.pem"
#define CA_CERT "/home/vtoroman/NSwO-1.3/ssl/serverCAcert.pem"

int main ( int argc, char ** argv ) {
  SSL_library_init();
  ERR_load_crypto_strings();
  SSL_load_error_strings();	
  OpenSSL_add_all_algorithms();  
  RAND_load_file("/dev/urandom", 1024000);

  std::string line;
  time_t start;
  time_t end;
  int i;
  std::ifstream ca_key_input;
  wool::swool::PemIO pem_loader;
  wool::swool::PemIO pem_no_pass;
  wool::swool::Key cakey;
  wool::swool::X509Cert cacert;
  wool::swool::Rsa rsa_key(1024,17);

  pem_loader.setPassword("pppp");
  ca_key_input.open(CA_KEY);
  pem_loader.load(cakey, ca_key_input);
  ca_key_input.close();
  
  ca_key_input.open(CA_CERT);
  pem_loader.load(cacert, ca_key_input);
  ca_key_input.close();

  wool::swool::Name issuer_name(cacert.getIssuer());
  
  std::cout << std::endl << "Press Enter to start " << std::endl ;
  std::getline(std::cin, line);
    
  ::time(&start);

  wool::socket server (wool::inet::inet_stream);
  wool::inet::address addr = "66.249.93.111:995";
  

  std::cout << "connect..." << std::endl;
  server.connect(addr);
  
  wool::iostream<wool::socket> server_stream(server);
  wool::swool::streambuf_bio sink(server_stream.rdbuf());

  std::cout << "ssl_ctx..." << std::endl;
  wool::swool::SslContext ssl_ctx(SSLv23_client_method);

  ssl_ctx.setCert(cacert);
  ssl_ctx.setPrivateKey(cakey);

  std::cout << "ssl bio..." << std::endl;
  BIO * sslbio_ptr = BIO_new_ssl(ssl_ctx.get_wrapped(), 1);
  if(sslbio_ptr==NULL) {
    std::cerr << "could not create ssl BIO " << std::endl;
    throw wool::swool::exception();
  }
  wool::swool::SslBio ssl_bio(ssl_ctx);

  std::cout << "push..." << std::endl;
  sink.push(ssl_bio);
  

  std::cout << "bio buffer..." << std::endl;
  wool::swool::bio_streambuf biobuffer(sink);


  std::cout << &biobuffer;

  return 0;
}
