/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#include "rwlock.h"
#include <iostream>

namespace wool {
  RWMutex::RWMutex () 
    : m_readLocks(0),
      m_writeLocks(0), 
      m_allowNewLocks(true) {
  }
  
  RWMutex::~RWMutex() {
    assert(m_readLocks==0);
    assert(m_writeLocks==0);
  }
  
  RWMutex::Lock::Lock (RWMutex * p) : m_mutex(p), m_creationThread(NULL) {    
  }

  void RWMutex::Lock::notify() {
    assert(m_mutex);
    m_mutex->m_stateChanged.notify_all();
  }
  
  RWMutex::Lock::~Lock() {
    assert(boost::thread() == *m_creationThread);
  }

  bool RWMutex::Lock::locked() const {
    return (m_mutex!=NULL);
  }

  RWMutex::ReadLock::ReadLock() {
  }

  RWMutex::ReadLock::ReadLock( RWMutex & p ) {
    lock(p);
  }

  RWMutex::ReadLock::~ReadLock() {
    unlock();
  }

  RWMutex::ReadLock & RWMutex::ReadLock::lock ( RWMutex & p) {
    unlock();
    boost::mutex::scoped_lock l(p.m_guard);
    if(p.m_lockRecord.get()==NULL)
      p.m_lockRecord.reset(new ThreadLockRecord());
    bool have_lock = (p.m_lockRecord->m_writeLocks>0) || ( p.m_lockRecord->m_readLocks > 0);
    while(( !p.m_allowNewLocks && !have_lock ) || (p.m_writeLocks > p.m_lockRecord->m_writeLocks)) {
      p.m_stateChanged.wait(l);
    }
    p.m_readLocks ++;
    p.m_lockRecord->m_readLocks++;
    m_mutex = &p;
    m_creationThread = new boost::thread();
    notify();
    return *this;
  }
  
  RWMutex::ReadLock & RWMutex::ReadLock::unlock () {
    if(m_mutex!=NULL) {
      boost::mutex::scoped_lock l(m_mutex->m_guard);
      assert(m_mutex->m_lockRecord.get()!=NULL);
      assert(m_mutex->m_readLocks>0);
      assert(m_mutex->m_lockRecord->m_readLocks>0);
      assert(boost::thread()==*m_creationThread);
      m_mutex->m_readLocks--;
      m_mutex->m_lockRecord->m_readLocks--;
      notify();
      delete m_creationThread;
      m_creationThread = NULL;
      m_mutex = NULL;
    }
    return *this;
  }

  RWMutex::WriteLock::WriteLock() {
  }
  
  RWMutex::WriteLock::WriteLock( RWMutex & p, bool allow_lock_while_waiting  ) {
    lock(p, allow_lock_while_waiting);
  }
  
  RWMutex::WriteLock::~WriteLock() {
    unlock();
  }

  RWMutex::WriteLock & RWMutex::WriteLock::lock ( RWMutex & p, bool allow_lock_while_waiting ) {
    unlock();
    boost::mutex::scoped_lock l(p.m_guard);
    if(p.m_lockRecord.get()==NULL)
      p.m_lockRecord.reset(new ThreadLockRecord());
    while(!p.m_allowNewLocks)
      p.m_stateChanged.wait(l);

    p.m_allowNewLocks = allow_lock_while_waiting;
    
    while((!p.m_allowNewLocks && allow_lock_while_waiting ) 
	  || (p.m_writeLocks > p.m_lockRecord->m_writeLocks)
	  || (p.m_readLocks > p.m_lockRecord->m_readLocks)) {      
      p.m_stateChanged.wait(l);
    }
    p.m_writeLocks ++;
    p.m_lockRecord->m_writeLocks++;
    m_mutex = &p;
    m_creationThread = new boost::thread();
    m_mutex->m_allowNewLocks = true;
    notify();
    return *this;
  }

  RWMutex::WriteLock & RWMutex::WriteLock::unlock () {
    if(m_mutex!=NULL) {
      boost::mutex::scoped_lock l(m_mutex->m_guard);
      assert(m_mutex->m_lockRecord.get()!=NULL);
      assert(m_mutex->m_writeLocks>0);
      assert(m_mutex->m_lockRecord->m_writeLocks>0);
      assert(boost::thread()==*m_creationThread);
      m_mutex->m_writeLocks --;
      m_mutex->m_lockRecord->m_writeLocks--;
      notify();
      m_mutex = NULL;
      delete m_creationThread;
      m_creationThread = NULL;
    }
    return *this;
  }
  
};

