/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef WOOLRWLOCK_H
#define WOOLRWLOCK_H

#include <boost/thread.hpp>

namespace wool {
  class RWMutex {
  public:
    RWMutex();
    ~RWMutex();

    class Lock{
    public:
      ~Lock();
      bool locked() const;
    protected:
      Lock(RWMutex * m = NULL);
      void notify();

      RWMutex * m_mutex;
      boost::thread * m_creationThread;

    private:
      inline Lock(const Lock & l) {}
    };

    class ReadLock : public Lock {
    public:
      ReadLock();
      ReadLock(RWMutex & p);
      ~ReadLock();

      ReadLock & lock (RWMutex & p);
      ReadLock & unlock();
    };
    
    class WriteLock : public Lock{
    public:
      WriteLock ();
      WriteLock (RWMutex & p,bool allow_lock_while_waiting = true );
      ~WriteLock();

      WriteLock & lock (RWMutex & p, bool allow_lock_while_waiting = true);
      WriteLock & unlock();
    };
    
		
  private:
    struct ThreadLockRecord {
      int m_readLocks;
      int m_writeLocks;
    };

    friend class WriteLock;
    friend class ReadLock;
    friend class ThreadLockRecord;
    
    int m_readLocks;
    int m_writeLocks;
    bool m_allowNewLocks;
    
    boost::thread_specific_ptr<ThreadLockRecord> m_lockRecord;
    boost::condition m_stateChanged;
    boost::mutex m_guard;
  };
}
#endif
