/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "thread/thread.h"
#include "base/exception.h"
#include <boost/bind.hpp>

namespace wool {
   thread::thread() :m_thread(NULL),m_should_run(false),m_is_finished(true) {
   }
   
   thread::~thread() {
      boost::mutex::scoped_lock l(m_guard);
      __stop();
      l.unlock();
      wait();
   }
   
   void thread::__start() {
      if(!__running()) {
         m_should_run = true;
         m_is_finished = false;
         m_thread = new boost::thread(boost::bind(&thread::runner,this));
      }
   }
   
   void thread::__stop() {
      if(__running())
         m_should_run = false;
   }

   bool thread::__joinable() const {
      return (__running())?m_is_finished:true;
   }

   void thread::wait() {
      boost::mutex::scoped_lock l(m_guard);
      while(!__joinable())
        m_finished.wait(l);
      if(__running()) {
        m_thread->join();
        delete m_thread;
        m_thread = NULL;
      }
   }
   
   bool thread::__running () const {
      return (m_thread!=NULL);
   }
   
   bool thread::__check_termination() const {
      return !m_should_run;
   }
   
   void thread::runner () {
     try {
       try {
         on_start();
         run();
       } catch ( const std::exception & e ) {
         on_error(e);
       } catch (...) {
         on_error(wool::exception("unknown error in thread"));
       }       
     } catch (...) {
       assert(false);
     }
     
     try {
       on_finish();
     } catch (...) {
       assert(false);
     }
     boost::mutex::scoped_lock l(m_guard);
     m_is_finished = true;
     m_finished.notify_all();
   }
};
