/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef WOOLTHREAD_H
#define WOOLTHREAD_H

#include <boost/thread.hpp>

namespace wool {
    class thread {
        public:
            thread ();
            virtual ~thread();

            virtual inline void start () {
              boost::mutex::scoped_lock l(m_guard);
              __start();
            }

            virtual inline void stop () {
              boost::mutex::scoped_lock l(m_guard);
              __stop();
            }

            virtual void wait ();

	    inline bool is_current() const {
	      boost::mutex::scoped_lock l(m_guard);
	      boost::thread curr;
	      return (__running() && ((*m_thread)==curr));
	    }
        protected:
            inline bool running() const {
              boost::mutex::scoped_lock l(m_guard);
              return __running();
            }

            inline bool joinable() const {
              boost::mutex::scoped_lock l(m_guard);
              return __joinable();
            }


            inline bool check_termination () const {
              boost::mutex::scoped_lock l(m_guard);
              return __check_termination ();
            }

            virtual void run() = 0;
            virtual void on_start() = 0;
            virtual void on_finish() = 0;
            virtual void on_error( const std::exception & e ) = 0;
        private:
        
            void __start ();
            void __stop ();
            bool __running() const;
            bool __joinable() const;
            bool __check_termination () const;
            
            void runner ();
            bool m_should_run;
            mutable boost::mutex m_guard;
            boost::condition m_finished;
            boost::thread * m_thread;
            bool m_is_finished;
            
    };
};

#endif
