/**************************************************************************
*   Copyright (C) 2004 by Vukasin Toroman                                 *
*   vukasin@toroman.org                                                   *
**************************************************************************/
/************************************************************************** 
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "thread/timer.h"
#include "base/exception.h"

namespace wool{

   timer::timer ( long ms ) : m_timeout(ms),m_triggered(false) {
   }
   
   timer::~timer ( ) {
      stop();
      trigger();
   }
   
   void timer::trigger() {
      boost::mutex::scoped_lock l(m_trigger_guard);
      if(!m_triggered) {
         m_triggered = true;
         m_trigger.notify_all();
      }
   }
   
   void timer::on_start() {
      boost::mutex::scoped_lock l(m_trigger_guard);
      m_triggered = false;
   }
   
   void timer::on_finish() {
   }
   
   void timer::on_error( const std::exception & e ) {
      stop();
      trigger();
   }
   
   void timer::set_timeout ( long ms ) {
      boost::mutex::scoped_lock l(m_trigger_guard);
      m_timeout = ms;
   }
   
   long timer::get_timeout ( ) const {
      boost::mutex::scoped_lock l(m_trigger_guard);
      return m_timeout;
   }
   
   void timer::run () {
      while( true ) {
         try {
            boost::xtime t;
            if(boost::xtime_get(&t,boost::TIME_UTC)!=boost::TIME_UTC)
               throw wool::exception("could not access system time");
            boost::mutex::scoped_lock l(m_trigger_guard);
            t.sec += m_timeout/1000l;
            t.nsec += (m_timeout%1000l)*1000000l;
            while((!m_triggered)&&(m_trigger.timed_wait(l,t)));
            if(thread::check_termination())
               return;
            m_triggered = false;
            on_trigger();
         } catch ( const std::exception & e ) {
            this->on_error( e );
         }
      }
   }
}
